from flask import Flask
from oslo_concurrency import processutils
from oslo_config import cfg
from oslo_service import service
from oslo_service import wsgi

from nachos.api.v1.root import initialize as initialize_v1


class WSGIService(service.ServiceBase):
    """Provides ability to launch Nachos API as an OSLO Service."""

    def __init__(self, name):

        self.name = name
        self.app = Flask(__name__, static_folder=None)
        self.app.url_map.strict_slashes = False

        initialize_v1(self.app)

        self.workers = (
            cfg.CONF.api.api_workers or processutils.get_worker_count()
        )

        self.server = wsgi.Server(
            cfg.CONF,
            name,
            self.app,
            host=cfg.CONF.api.host,
            port=cfg.CONF.api.port,
            use_ssl=cfg.CONF.api.enable_ssl_api
        )

    def start(self):
        self.server.start()

    def stop(self):
        self.server.stop()

    def wait(self):
        self.server.wait()

    def reset(self):
        self.server.reset()
