import sys

import eventlet
# Monkeypatch early to avoid concurrency issues
eventlet.monkey_patch(
    os=True,
    select=True,
    socket=True,
    thread=False if '--use-debugger' in sys.argv else True,
    time=True)
import pbr.version  # noqa: E402
from oslo_config import cfg  # noqa: E402
from oslo_service import service  # noqa: E402
from oslo_log import log as logging  # noqa: E402

from nachos.api import service as api_service  # noqa: E402
from nachos.common import config  # noqa: E402


CONF = cfg.CONF


def launch_api():
    launcher = service.ProcessLauncher(cfg.CONF)
    server = api_service.WSGIService('nachos_api')
    launcher.launch_service(server, workers=1)
    launcher.wait()


def launch_any(options):
    threads = [eventlet.spawn(LAUNCH_OPTIONS[option])
               for option in options]

    [thread.wait() for thread in threads]


# Map cli options to appropriate functions. The cli options are
# registered in nachos's config.py.
LAUNCH_OPTIONS = {
    'api': launch_api
}

nachos_TITLE = str("Starting nachos - Naemon Management Server, version %s"
                   % pbr.version.VersionInfo('nachos').version_string())


def print_server_info():
    print(nachos_TITLE)

    comp_str = ("[%s]" % ','.join(LAUNCH_OPTIONS)
                if cfg.CONF.server == ['all'] else cfg.CONF.server)

    print('Launching server components %s...' % comp_str)


def get_properly_ordered_parameters():
    """Orders launch parameters in the right order.
    In oslo it's important the order of the launch parameters.
    if --config-file came after the command line parameters the command
    line parameters are ignored.
    So to make user command line parameters are never ignored this method
    moves --config-file to be always first.
    """
    args = sys.argv[1:]

    for arg in sys.argv[1:]:
        if arg == '--config-file' or arg.startswith('--config-file='):
            if "=" in arg:
                conf_file_value = arg.split("=", 1)[1]
            else:
                conf_file_value = args[args.index(arg) + 1]
                args.remove(conf_file_value)
            args.remove(arg)
            args.insert(0, "--config-file")
            args.insert(1, conf_file_value)

    return args


def main():
    try:
        config.parse_args(get_properly_ordered_parameters())
        print_server_info()

        logging.setup(CONF, 'nachos')

        if cfg.CONF.server == ['all']:
            # Launch all servers.
            launch_any(LAUNCH_OPTIONS.keys())
        else:
            # Validate launch option.
            if set(cfg.CONF.server) - set(LAUNCH_OPTIONS.keys()):
                raise Exception('Valid options are all or any combination of '
                                ', '.join(LAUNCH_OPTIONS.keys()))

            # Launch distinct set of server(s).
            launch_any(set(cfg.CONF.server))

    except RuntimeError as excp:
        sys.stderr.write("ERROR: {}\n".format(excp))
        sys.exit(1)


if __name__ == '__main__':
    sys.exit(main())
