from nachos.engine.export.utils.config_key_helper import get_empty_key_data


def parse_definitions(fh):
    ws = []
    definitions = []
    linenum = 0
    for n, line in enumerate(fh, 1):
        if line.lstrip().startswith(b"define"):
            definitions.append(line)
            linenum = n
        elif definitions:
            definitions.append(line)
            if line.lstrip().startswith(b"}"):
                yield (ws, definitions, linenum)
                ws = []
                definitions = []
        else:
            ws.append(line)
    yield (ws, None, 0)


def parse_type(line, encoding="ascii"):
    return line.lstrip()[len(b"define"):].strip(b" {\n").decode(encoding)


def parse_config_key(lines, encoding="ascii"):
    object_type = parse_type(lines[0], encoding=encoding)
    key_data = get_empty_key_data(object_type)
    name = None
    # Loop over all lines, except first and last.
    for i in range(1, len(lines) - 1):
        line = lines[i].decode(encoding)
        try:
            k, v = line.rstrip().split(None, 1)
        except ValueError:
            continue
        if k in key_data:
            if k in ("contacts", "contact_groups"):
                v = ",".join(sorted(v.split(",")))
            key_data[k] = v
        elif k == "register" and v == "0":
            key_data = {"name": name}
            if name:
                break
        elif k == "name":
            name = v
    return (object_type, frozenset(key_data.items()))
