import subprocess

from oslo_log import log as logging
from oslo_config import cfg
from subprocess import Popen, PIPE
from nachos.common.exceptions import (ConfigVerificationException,
                                      ConfigUnknownException,
                                      NaemonReloadException,)

logger = logging.getLogger(__name__)


def check_error(data):
    msg_warning = []
    msg_error = []
    for line in data:
        line = line.decode('utf-8').lstrip()
        if line.startswith('Warning:'):
            msg_warning.append(line)
        if line.startswith('Error:'):
            msg_error.append(line)

    if msg_error:
        for msg in msg_warning[-9:]:
            logger.warning(msg)

        for msg in msg_error[-9:]:
            logger.warning(msg)

        raise ConfigVerificationException(
            "{err} errors and {warn} warnings encountered "
            "while reloading naemon. "
            "See the Nachos log for further details".format(
                err=len(msg_error),
                warn=len(msg_warning)),
        )


def verify_configuration(config_file):
    """"
        Gets config-verification output from naemon
        Logs the output to nachos.log if an error is encountered
        :param config_file: path to naemon.cfg
    """
    p = Popen(['naemon', '--verify-config', config_file], stdout=PIPE)
    check_error(p.stdout.readlines())
    p.communicate()
    if p.returncode != 0:
        raise ConfigUnknownException(
            'naemon config verification failed, but no error was matched.'
        )


def naemon_reload():
    if subprocess.getstatusoutput('sudo systemctl restart naemon.service')[0] != 0:
        if cfg.CONF.monitor.verify_before_commit is False:
            verify_configuration(cfg.CONF.monitor.naemon_main_config)
        raise NaemonReloadException(
            "Unknown error when reloading Naemon. Check the logs."
        )
