from nachos.nacoma.mappers import BaseContactMapper
from nachos.engine.export.writers.helpers import (write_property,
                                                  write_list,
                                                  write_bool,
                                                  write_subprop,
                                                  write_options)


def write_contact(contact, out):

    assert isinstance(contact, BaseContactMapper), \
        'Expected contact parameter to implement BaseContactMapper'

    # If this is a template (register = 0) contact_name and
    # address are not mandatory
    assert not (contact.contact_name is None and
                contact.register is not False), \
        'Contact contact_name cannot be None on registered contact'

    if contact.register:
        out.write("# contact '{}'\n".format(contact.contact_name))
    else:
        out.write("# contact template '{}'\n".format(contact.name))
    out.write('define contact {\n')
    write_subprop(out, 'use', contact.use, 'name')
    write_property(out, 'contact_name', contact.contact_name)
    write_property(out, 'alias', contact.alias)
    write_list(out, 'contactgroups', contact.contactgroups,
               'contactgroup_name')
    write_bool(out, 'host_notifications_enabled',
               contact.host_notifications_enabled)
    write_bool(out, 'service_notifications_enabled',
               contact.service_notifications_enabled)
    write_bool(out, 'can_submit_commands', contact.can_submit_commands)
    write_bool(out, 'retain_status_information',
               contact.retain_status_information)
    write_bool(out, 'retain_nonstatus_information',
               contact.retain_nonstatus_information)
    write_subprop(out, 'host_notification_period',
                  contact.host_notification_timeperiod, 'timeperiod_name')
    write_subprop(out, 'service_notification_period',
                  contact.service_notification_timeperiod, 'timeperiod_name')
    write_options(out, 'host_notification_options',
                  contact.host_notification_options)
    write_options(out, 'service_notification_options',
                  contact.service_notification_options)
    write_property(out, 'host_notification_commands',
                   contact.host_notification_commands)
    write_property(out, 'service_notification_commands',
                   contact.service_notification_commands)
    write_property(out, 'minimum_value', contact.minimum_value)
    write_property(out, 'email', contact.email)
    write_property(out, 'pager', contact.pager)
    write_property(out, 'address1', contact.address1)
    write_property(out, 'address2', contact.address2)
    write_property(out, 'address3', contact.address3)
    write_property(out, 'address4', contact.address4)
    write_property(out, 'address5', contact.address5)
    write_property(out, 'address6', contact.address6)
    write_property(out, 'name', contact.name)
    write_bool(out, 'register', contact.register)

    if contact.custom_vars is not None:
        for custom_var in contact.custom_vars:
            write_property(out,
                           custom_var.key,
                           custom_var.value)

    out.write('}\n')
