from time import strftime, gmtime
from textwrap import wrap


def write_header(out, exporter=None):
    out.write(
        """###############################################################################
#  Generated by op5 Naemon Management Server
#
#  Exported {} by {}
#\n\n
""".format(strftime("%Y-%m-%d %H:%M:%S", gmtime()), exporter or 'monitor'))


def write_property(out, key, value):
    if value is not None and value != "":
        value = str(value)
        # Naemon interprets backslashes at the end of a property as a line
        # continuation, but we shouldn't escape it elsewhere.
        if value.endswith("\\"):
            value += "\\"
        out.write('    {:<30} {}\n'.format(key, value))


def write_options(out, key, value):
    options = []
    if value:
        for option in list(value):
            options.append(option.name)
        out.write('    {:<30} {}\n'.format(key, ','.join(sorted(options))))


def write_bool(out, key, value):
    write_property(
        out, key, None if value is None else '1' if value else '0')


def write_list(out, key, value, onkey):
    if value and isinstance(value, list) and len(value) > 0:
        new_value = []
        for item in value:
            new_value.append(getattr(item, onkey))
        write_property(
            out, key, ",".join(new_value))


def write_subprop(out, key, value, subprop):
    if value and getattr(value, subprop) is not None:
        write_property(out, key, getattr(value, subprop))


def write_enum(out, key, value):
    if value:
        write_property(out, key, value.name)


def write_config_comment(out, comment):

    out.write('###\n')
    for line in wrap(comment):
        out.write('# {}\n'.format(line))
    out.write('#\n')
