from nachos.nacoma.mappers import BaseHostMapper
from nachos.engine.export.writers.helpers import (write_property,
                                                  write_list,
                                                  write_enum,
                                                  write_bool,
                                                  write_subprop,
                                                  write_options)


def write_host(host, out):

    assert isinstance(host, BaseHostMapper), \
        'Expected host parameter to implement BaseHostMapper'

    # If this is a template (register = 0) host_name and
    # address are not mandatory
    assert not (host.host_name is None and host.register is not False), \
        'Host host_name cannot be None'

    assert not (host.address is None and host.register is not False), \
        'Host address cannot be None'

    if host.register:
        out.write("# host '{}'\n".format(host.host_name))
    else:
        out.write("# host template '{}'\n".format(host.name))
    out.write('define host {\n')

    write_subprop(out, 'use', host.use, 'name')
    write_property(out, 'host_name', host.host_name)
    write_property(out, 'address', host.address)

    write_property(out, 'alias', host.alias)
    write_property(out, 'display_name', host.display_name)
    write_list(out, 'parents', host.parents, 'host_name')

    write_property(out, 'check_command', host.check_command)
    write_list(out, 'contacts', host.contacts, 'contact_name')
    write_list(out, 'contact_groups', host.contact_groups,
               'contactgroup_name')
    write_enum(out, 'initial_state', host.initial_state)
    write_list(out, 'hostgroups', host.hostgroups, 'hostgroup_name')
    write_property(out, 'max_check_attempts', host.max_check_attempts)
    write_property(out, 'check_interval', host.check_interval)
    write_property(out, 'retry_interval', host.retry_interval)
    write_bool(out, 'active_checks_enabled', host.active_checks_enabled)
    write_bool(out, 'passive_checks_enabled', host.passive_checks_enabled)
    write_subprop(out, 'check_period', host.check_period, 'timeperiod_name')
    write_bool(out, 'obsess', host.obsess)
    write_bool(out, 'check_freshness', host.check_freshness)
    write_property(out, 'freshness_threshold', host.freshness_threshold)
    write_property(out, 'event_handler', host.event_handler)
    write_bool(out, 'event_handler_enabled', host.event_handler_enabled)
    write_property(out, 'low_flap_threshold', host.low_flap_threshold)
    write_property(out, 'high_flap_threshold', host.high_flap_threshold)
    write_bool(out, 'flap_detection_enabled', host.flap_detection_enabled)
    write_options(out, 'flap_detection_options',
                  host.flap_detection_options)
    write_bool(out, 'process_perf_data', host.process_perf_data)
    write_bool(out, 'retain_status_information',
               host.retain_status_information)
    write_bool(out, 'retain_nonstatus_information',
               host.retain_nonstatus_information)
    write_property(out, 'notification_interval', host.notification_interval)
    write_property(out, 'first_notification_delay',
                   host.first_notification_delay)
    write_subprop(out, 'notification_period',
                  host.notification_period, 'timeperiod_name')
    write_options(out, 'notification_options', host.notification_options)
    write_bool(out, 'notifications_enabled', host.notifications_enabled)
    write_options(out, 'stalking_options', host.stalking_options)
    write_property(out, 'notes', host.notes)
    write_property(out, 'notes_url', host.notes_url)
    write_property(out, 'action_url', host.action_url)
    write_property(out, 'icon_image', host.icon_image)
    write_property(out, 'icon_image_alt', host.icon_image_alt)
    write_property(out, 'statusmap_image', host.statusmap_image)
    write_property(out, '2d_coords', host.coords_2d)
    write_property(out, 'hourly_value', host.hourly_value)
    write_property(out, 'name', host.name)
    write_bool(out, 'register', host.register)

    if host.custom_vars is not None:
        for custom_var in host.custom_vars:
            write_property(out,
                           custom_var.key,
                           custom_var.value)

    out.write('}\n')
