from nachos.nacoma.mappers import BaseHostescalationMapper
from nachos.engine.export.writers.helpers import (write_property,
                                                  write_list,
                                                  write_subprop,
                                                  write_options)


def write_hostescalation(hostescalation, out):
    """ Takes a mapped hostescalation object and writes it to a configuration
        file in a Naemon compatible format

        :param hostescalation: A mapped hostescalation object
        :param out: File handler
    """
    validate_hostescalation(hostescalation)

    if getattr(hostescalation, 'host', None):
        out.write("# hostescalation for host '{}'\n".format(
            hostescalation.host.host_name))
    elif getattr(hostescalation, 'hostgroup', None):
        out.write("# hostescalation for hostgroup '{}'\n".format(
            hostescalation.hostgroup.hostgroup_name))

    out.write('define hostescalation {\n')

    if getattr(hostescalation, 'host', None):
        write_subprop(out, 'host_name', hostescalation.host, 'host_name')
    elif getattr(hostescalation, 'hostgroup', None):
        write_subprop(out, 'hostgroup_name', hostescalation.hostgroup,
                      'hostgroup_name')

    write_list(out, 'contacts', hostescalation.contacts, 'contact_name')
    write_list(out, 'contact_groups', hostescalation.contact_groups,
               'contactgroup_name')
    write_property(out, 'first_notification',
                   hostescalation.first_notification)
    write_property(out, 'last_notification',
                   hostescalation.last_notification)
    write_property(out, 'notification_interval',
                   hostescalation.notification_interval)
    write_subprop(out, 'escalation_period',
                  hostescalation.escalation_timeperiod, 'timeperiod_name')
    write_options(out, 'escalation_options',
                  hostescalation.escalation_options)
    out.write('}\n')


def validate_hostescalation(hostescalation):
    """ Validates a hostescalation object

        A hostescalation can be on either a host or a hostgroup, so at least
        one need to exist and have a host/hostgroup name. It is not possible
        to have a hostescalation with both a host and hostgroup.

        :param hostescalation: A mapped hostescalation object
    """

    assert isinstance(hostescalation, BaseHostescalationMapper), \
        'Expected hostescalation parameter to implement ' \
        'BaseHostescalationMapper'

    hostescalation_host = getattr(hostescalation, 'host', None)
    hostescalation_hostgroup = getattr(hostescalation, 'hostgroup', None)

    assert hostescalation_host is not None or \
        hostescalation_hostgroup is not None, \
        'A hostescalation needs either a host or hostgroup'

    if hostescalation_host is not None:
        assert getattr(hostescalation_host, 'host_name', None), \
            'Hostescalation host name cannot be None'

    if hostescalation_hostgroup is not None:
        assert getattr(hostescalation_hostgroup, 'hostgroup_name', None), \
            'Hostescalation hostgroup name cannot be None'
