from nachos.nacoma.mappers import BaseServiceMapper
from nachos.engine.export.writers.helpers import (write_property,
                                                  write_list,
                                                  write_enum,
                                                  write_bool,
                                                  write_subprop,
                                                  write_options)


def write_service(service, out):

    validate_service(service)

    if service.register:
        out.write("# service '{}'\n".format(service.description))
    else:
        out.write("# service template '{}'\n".format(service.name))
    out.write('define service {\n')

    # For some reason, Nacoma has a special column for template service
    # called 'name'. Naemon doesn't have any properties called name, only
    # service description, so we handle both cases here
    if getattr(service.use, 'name', None):
        write_subprop(out, 'use', service.use, 'name')
    elif getattr(service.use, 'description', None):
        write_subprop(out, 'use', service.use, 'description')

    if service.register:  # Not a template
        write_property(out, 'service_description', service.description)

    write_subprop(out, 'host_name', service.host, 'host_name')
    write_subprop(out, 'hostgroup_name', service.hostgroup,
                  'hostgroup_name')

    write_property(out, 'display_name', service.display_name)
    write_list(out, 'parents', service.parents, 'host_name')
    write_property(out, 'check_command', service.check_command)
    write_list(out, 'contacts', service.contacts, 'contact_name')
    write_list(out, 'contact_groups', service.contact_groups,
               'contactgroup_name')
    write_list(out, 'servicegroups', service.servicegroups,
               'servicegroup_name')
    write_enum(out, 'initial_state', service.initial_state)
    write_property(out, 'max_check_attempts', service.max_check_attempts)
    write_property(out, 'check_interval', service.check_interval)
    write_property(out, 'retry_interval', service.retry_interval)
    write_bool(out, 'is_volatile', service.is_volatile)
    write_bool(out, 'active_checks_enabled', service.active_checks_enabled)
    write_bool(out, 'passive_checks_enabled', service.passive_checks_enabled)
    write_subprop(out, 'check_period', service.check_period,
                  'timeperiod_name')
    write_bool(out, 'obsess', service.obsess)
    write_bool(out, 'check_freshness', service.check_freshness)
    write_property(out, 'freshness_threshold', service.freshness_threshold)
    write_property(out, 'event_handler', service.event_handler)
    write_bool(out, 'event_handler_enabled', service.event_handler_enabled)
    write_property(out, 'low_flap_threshold', service.low_flap_threshold)
    write_property(out, 'high_flap_threshold', service.high_flap_threshold)
    write_bool(out, 'flap_detection_enabled', service.flap_detection_enabled)
    write_options(out, 'flap_detection_options',
                  service.flap_detection_options)
    write_bool(out, 'process_perf_data', service.process_perf_data)
    write_bool(out, 'retain_status_information',
               service.retain_status_information)
    write_bool(out, 'retain_nonstatus_information',
               service.retain_nonstatus_information)
    write_property(out, 'notification_interval',
                   service.notification_interval)
    write_property(out, 'first_notification_delay',
                   service.first_notification_delay)
    write_subprop(out, 'notification_period',
                  service.notification_period, 'timeperiod_name')
    write_options(out, 'notification_options', service.notification_options)
    write_bool(out, 'notifications_enabled', service.notifications_enabled)
    write_options(out, 'stalking_options', service.stalking_options)
    write_bool(out, 'parallelize_check', service.parallelize_check)
    write_property(out, 'notes', service.notes)
    write_property(out, 'notes_url', service.notes_url)
    write_property(out, 'action_url', service.action_url)
    write_property(out, 'icon_image', service.icon_image)
    write_property(out, 'icon_image_alt', service.icon_image_alt)
    write_property(out, 'statusmap_image', service.statusmap_image)
    write_property(out, '2d_coords', service.coords_2d)
    write_property(out, 'hourly_value', service.hourly_value)
    write_property(out, 'name', service.name)
    write_bool(out, 'register', service.register)

    if service.custom_vars is not None:
        for custom_var in service.custom_vars:
            write_property(out,
                           custom_var.key,
                           custom_var.value)

    out.write('}\n')


def validate_service(service):
    """ Validates a service object

        A service which is not a template (register != 0) must have a
        service description. And it must either belong to host or
        hostgroup.

        :param service: A mapped service object
    """
    assert isinstance(service, BaseServiceMapper), \
        'Expected service parameter to implement BaseServiceMapper'

    assert not (getattr(service, 'description', None) is None and
                getattr(service, 'register', None) is not False), \
        'Service service_description cannot be None on registered service'

    assert not (getattr(service, 'register', None) is True and
                (getattr(service.host, 'host_name', None) is None and
                 getattr(service.hostgroup,
                         'hostgroup_name', None) is None)), \
        'Service host_name or hostgroup_name must be set on registered service'
