from nachos.nacoma.mappers import BaseServiceescalationMapper
from nachos.engine.export.writers.helpers import (write_property,
                                                  write_list,
                                                  write_subprop,
                                                  write_options)


def write_serviceescalation(serviceescalation, out):
    """ Takes a mapped serviceescalation object and writes it to a
        configuration file in a Naemon compatible format

        :param serviceescalation: A mapped serviceescalation object
        :param out: File handler
    """

    validate_serviceescalation(serviceescalation)

    if getattr(serviceescalation, 'host', None) is not None:
        out.write("# serviceescalation for service '{}' on host '{}'\n".format(
            getattr(serviceescalation.service, 'description', None),
            getattr(serviceescalation.host, 'host_name', None)))
    if getattr(serviceescalation, 'hostgroup', None) is not None:
        out.write("# serviceescalation for service '{}' on hostgroup '{}'\n".format(  # noqa: E501
            getattr(serviceescalation.service, 'description', None),
            getattr(serviceescalation.hostgroup, 'hostgroup_name', None)))

    out.write('define serviceescalation {\n')

    if getattr(serviceescalation, 'host', None) is not None:
        write_subprop(out, 'host_name', serviceescalation.host, 'host_name')
    elif getattr(serviceescalation, 'hostgroup', None) is not None:  # noqa: E501
        write_subprop(out, 'hostgroup_name', serviceescalation.hostgroup,
                      'hostgroup_name')

    write_subprop(out, 'service_description',
                  serviceescalation.service,
                  'description')
    write_list(out, 'contacts', serviceescalation.contacts, 'contact_name')
    write_list(out, 'contact_groups', serviceescalation.contact_groups,
               'contactgroup_name')
    write_property(out, 'first_notification',
                   serviceescalation.first_notification)
    write_property(out, 'last_notification',
                   serviceescalation.last_notification)
    write_property(out, 'notification_interval',
                   serviceescalation.notification_interval)
    write_subprop(out, 'escalation_period',
                  serviceescalation.escalation_period, 'timeperiod_name')
    write_options(out, 'escalation_options',
                  serviceescalation.escalation_options)
    out.write('}\n')


def validate_serviceescalation(serviceescalation):
    """ Validates a serviceescalation object

        A serviceescalation can be on either a host or a hostgroup, so at least
        one need to exist and have a host/hostgroup name. It is not possible
        to have a serviceescalation with both a host and a hostgroup.

        :param serviceescalation: A mapped serviceescalation object
    """

    assert isinstance(serviceescalation, BaseServiceescalationMapper), \
        'Expected serviceescalation parameter to implement ' \
        'BaseServiceescalationMapper'

    assert getattr(serviceescalation.service,
                   'description', None) is not None, \
        'Serviceescalation service_description cannot be None'

    serviceescalation_host = getattr(serviceescalation, 'host', None)
    serviceescalation_hostgroup = getattr(serviceescalation,
                                          'hostgroup', None)

    assert serviceescalation_host is not None or \
        serviceescalation_hostgroup is not None, \
        'A serviceescalation needs either a host or hostgroup'

    if serviceescalation_host is not None:
        assert getattr(serviceescalation.host,
                       'host_name', None) is not None, \
            'Serviceescalation host_name cannot be None'

    if serviceescalation_hostgroup is not None:
        assert getattr(serviceescalation.hostgroup,
                       'hostgroup_name', None) is not None, \
            'Serviceescalation hostgroup_name cannot be None'
