from abc import ABC, abstractmethod, abstractproperty


class BaseNacoma(ABC):
    """
    Abstract class
    """

    @abstractmethod
    def aquire_database_lock(self, object_type, username):
        """
        Creates an object lock for a given object type
        """

    @abstractmethod
    def release_database_lock(self, object_type):
        """
        Releases object lock for all objects
        """

    @abstractmethod
    def archive_changes(self, username):
        """
        Move entries from the changelog table to the changelog_history table
        for the specified user
        """

    @abstractmethod
    def log_action(self, action_type, username):
        """
        Make an entry into the action_log table
        """

    @property
    @abstractmethod
    def changelogs(self):
        """
        Returns a list of mapped changelogs
        """

    @abstractmethod
    def changelog_by_id(self, changelog_id):
        """
        Returns the changelog with the matching id
        """

    @abstractproperty
    def commands(self):
        """
        Returns a list of mapped commands
        """

    @abstractmethod
    def command_by_id(self, command_id):
        """
        Returns the command with the matching id
        """

    @abstractproperty
    def contactgroups(self):
        """
        Returns a list of mapped contactgroups
        """

    @abstractproperty
    def contacts(self):
        """
        Returns a list of mapped contacts
        """

    @abstractproperty
    def customvars(self):
        """
        Returns a list of mapped customvars
        """

    @abstractproperty
    def hostdependencies(self):
        """
        Returns a list of mapped hostdependencies
        """

    @abstractproperty
    def hostescalations(self):
        """
        Returns a list of mapped hostescalations
        """

    @abstractproperty
    def hostgroups(self):
        """
        Returns a list of mapped hostgroups
        """

    @abstractproperty
    def hosts(self):
        """
        Returns a list of mapped hosts
        """

    @abstractmethod
    def host_by_id(self, host_id):
        """
        Returns the host with the matching id
        """

    @abstractproperty
    def servicedependencies(self):
        """
        Returns a list of mapped servicedependencies
        """

    @abstractproperty
    def serviceescalations(self):
        """
        Returns a list of mapped serviceescalations
        """

    @abstractproperty
    def servicegroups(self):
        """
        Returns a list of mapped servicegroups
        """

    @abstractproperty
    def services(self):
        """
        Returns a list of mapped services
        """

    @abstractmethod
    def service_by_id(self, service_id):
        """
        Returns the service with the matching id
        """

    @abstractproperty
    def timeperiods(self):
        """
        Returns a list of mapped timeperiods
        """

    @abstractproperty
    def files(self):
        """
        Returns a list of mapped files
        """

    @abstractmethod
    def file_by_id(self, file_id):
        """
        Returns the file with the matching id
        """

    def object_by_type_and_id(self, object_name, object_id):
        """
        Returns the matching object, or None
        """
