""" Helper methods for working with and around Nacoma """
import sqlalchemy_utils

from nachos.nacoma.api.sqlalchemy.models import (
        Command,
        Contactgroup,
        Contact,
        Host,
        Hostdependency,
        Hostescalation,
        Hostgroup,
        Service,
        Servicedependency,
        Serviceescalation,
        Servicegroup,
        Timeperiod
)

from nachos.nacoma.mappers import (
        ChangelogMapper,
        CommandMapper,
        ContactMapper,
        ContactgroupMapper,
        CustomvarMapper,
        FileMapper,
        HostMapper,
        HostdependencyMapper,
        HostescalationMapper,
        HostgroupMapper,
        ServiceMapper,
        ServicedependencyMapper,
        ServiceescalationMapper,
        ServicegroupMapper,
        TimeperiodMapper,
)


_NACOMA_MAPPER_MAP = {
    'changelog': ChangelogMapper,
    'command': CommandMapper,
    'contactgroup': ContactgroupMapper,
    'contact': ContactMapper,
    'contact_template': ContactMapper,
    'custom_vars': CustomvarMapper,
    'file': FileMapper,
    'host': HostMapper,
    'hostdependency': HostdependencyMapper,
    'hostescalation': HostescalationMapper,
    'hostgroup': HostgroupMapper,
    'host_template': HostMapper,
    'service': ServiceMapper,
    'servicedependency': ServicedependencyMapper,
    'serviceescalation': ServiceescalationMapper,
    'servicegroup': ServicegroupMapper,
    'service_template': ServiceMapper,
    'timeperiod': TimeperiodMapper
}


def get_nacoma_mapper(object_type):
    return _NACOMA_MAPPER_MAP.get(object_type)


NACOMA_CLASS_MAP = {
    'command': Command,
    'contactgroup': Contactgroup,
    'contact': Contact,
    'contact_template': Contact,
    'host': Host,
    'hostdependency': Hostdependency,
    'hostescalation': Hostescalation,
    'hostgroup': Hostgroup,
    'host_template': Host,
    'service': Service,
    'servicedependency': Servicedependency,
    'serviceescalation': Serviceescalation,
    'servicegroup': Servicegroup,
    'service_template': Service,
    'timeperiod': Timeperiod
}


def get_nacoma_class(name):
    return NACOMA_CLASS_MAP.get(name, None)


def get_dependent_objects(sqlalchemy_object):
    """ Returns all objects which hold a foreign key reference to this object
    :param sqlalchemy_object: The declarative sqlalchemy object
    :return: Set<Mappers>
    """
    dependent_objects_set = set()
    objects = sqlalchemy_utils.dependent_objects(sqlalchemy_object)
    for obj in objects:
        mapper = get_nacoma_mapper(obj.__tablename__)
        if mapper:
            dependent_objects_set.add(mapper(obj))
    return dependent_objects_set
