from abc import abstractmethod
from nachos.nacoma.mappers import BaseMapper


class BaseChangelogMapper(BaseMapper):
    @property
    @abstractmethod
    def object_type(self):
        """
        :return str|None:
        """
        pass

    @property
    @abstractmethod
    def object_id(self):
        """
        :return str|None:
        """
        pass

    @property
    @abstractmethod
    def object_identifier(self):
        """
        Returns a (object_type, object_id) tuple.

        :return tuple:
        """
        pass

    @property
    @abstractmethod
    def oldname(self):
        """
        :return str|None:
        """
        pass

    @property
    @abstractmethod
    def newname(self):
        """
        :return str|None:
        """
        pass

    @property
    @abstractmethod
    def oldstate(self):
        """
        :return bytes|None:
        """
        pass

    @property
    @abstractmethod
    def newstate(self):
        """
        :return bytes|None:
        """
        pass

    @property
    @abstractmethod
    def username(self):
        """
        :return str|None:
        """
        pass

    @property
    @abstractmethod
    def time(self):
        """
        :return datetime.datetime|None:
        """
        pass

    @property
    @abstractmethod
    def is_new(self):
        """
        :return bool:
        """
        pass

    @property
    @abstractmethod
    def is_deleted(self):
        """
        :return bool:
        """
        pass

    @property
    @abstractmethod
    def is_renamed(self):
        """
        :return bool:
        """
        pass

    @property
    @abstractmethod
    def old_file_id(self):
        """
        :return int|None:
        """

    @property
    @abstractmethod
    def new_file_id(self):
        """
        :return int|None:
        """

    @property
    @abstractmethod
    def is_moved(self):
        """
        :return bool:
        """

    @abstractmethod
    def get_from_oldstate(self, key):
        """ Returns the value from oldstate matching the key
        :return str|None:
        """

    @abstractmethod
    def get_from_newstate(self, key):
        """ Returns the value from newstate matching the key
        :return str|None:
        """

    @abstractmethod
    def get_all_state_members(self, key, type=int):
        """ Returns a set of all member values in the associative arrays
            mapped to the given key, from both old and new state. Member
            values are casted to the given type before added to the set.
        :return set:
        """

    @property
    @abstractmethod
    def is_custom_var(self):
        """ Try to identify if it is a custom variable or not

            Custom variables start with _ and is UPPERCASE, except timeperiod
            exclusions. If the object_type is Timeperiod and the attribute does
            not exist for the object_type, and there is no synonym for it, we
            assume it is a timeperiod exclude. I.e. customvar.

            :return: True if customvar, else False
        """

    @abstractmethod
    def merge_change(self, other):
        """ Merge state with other changelog

        Merges the oldstate and newstate from this change with another change.
        Keeping the oldstate as old as possible and the newstate as new as possible.
        """
