from abc import abstractmethod
from nachos.nacoma.mappers import BaseMapper


class BaseHostMapper(BaseMapper):

    @property
    @abstractmethod
    def host_name(self):
        """
        :return string|None:
        """
        pass

    @property
    @abstractmethod
    def alias(self):
        """
        :return string|None:
        """
        pass

    @property
    @abstractmethod
    def display_name(self):
        """
        :return string|None:
        """
        pass

    @property
    @abstractmethod
    def address(self):
        """
        :return string|None:
        """
        pass

    @property
    @abstractmethod
    def file(self):
        """
        :return BaseFileMapper|None
        """

    @property
    @abstractmethod
    def parents(self):
        """
        :return List<BaseHostMapper>|None:
        """
        pass

    @property
    @abstractmethod
    def hourly_value(self):
        """
        :return int|none:
        """
        pass

    @property
    @abstractmethod
    def hostgroups(self):
        """
        :return List<BaseHostgroupMapper>|None:
        """
        pass

    @property
    @abstractmethod
    def check_command(self):
        """
        :return string|None:
        """
        pass

    @property
    @abstractmethod
    def initial_state(self):
        """
        :return HostStateFlag|None:
        """
        pass

    @property
    @abstractmethod
    def max_check_attempts(self):
        """
        :return int|None:
        """
        pass

    @property
    @abstractmethod
    def check_interval(self):
        """
        :return int|None:
        """
        pass

    @property
    @abstractmethod
    def retry_interval(self):
        """
        :return int|None:
        """
        pass

    @property
    @abstractmethod
    def active_checks_enabled(self):
        """
        :return bool|None:
        """
        pass

    @property
    @abstractmethod
    def passive_checks_enabled(self):
        """
        :return bool|None:
        """
        pass

    @property
    @abstractmethod
    def check_period(self):
        """
        :return BaseTimeperiodMapper|None:
        """
        pass

    @property
    @abstractmethod
    def obsess(self):
        """
        :return bool|None:
        """
        pass

    @property
    @abstractmethod
    def check_freshness(self):
        """
        :return bool|None:
        """
        pass

    @property
    @abstractmethod
    def freshness_threshold(self):
        """
        :return int|None:
        """
        pass

    @property
    @abstractmethod
    def event_handler(self):
        """
        :return string|None:
        """
        pass

    @property
    @abstractmethod
    def event_handler_enabled(self):
        """
        :return bool|None:
        """
        pass

    @property
    @abstractmethod
    def low_flap_threshold(self):
        """
        :return int|None:
        """
        pass

    @property
    @abstractmethod
    def high_flap_threshold(self):
        """
        :return int|None:
        """
        pass

    @property
    @abstractmethod
    def flap_detection_enabled(self):
        """
        :return bool|None:
        """
        pass

    @property
    @abstractmethod
    def flap_detection_options(self):
        """
        :return string|None:
        """
        pass

    @property
    @abstractmethod
    def process_perf_data(self):
        """
        :return bool|None:
        """
        pass

    @property
    @abstractmethod
    def retain_status_information(self):
        """
        :return bool|None:
        """
        pass

    @property
    @abstractmethod
    def retain_nonstatus_information(self):
        """
        :return bool|None:
        """
        pass

    @property
    @abstractmethod
    def contacts(self):
        """
        :return List<BaseContactMapper>|None:
        """
        pass

    @property
    @abstractmethod
    def contact_groups(self):
        """
        :return List<BaseContactgroupMapper>|None:
        """
        pass

    @property
    @abstractmethod
    def notification_interval(self):
        """
        :return int|None:
        """
        pass

    @property
    @abstractmethod
    def first_notification_delay(self):
        """
        :return int|None:
        """
        pass

    @property
    @abstractmethod
    def notification_period(self):
        """
        :return BaseTimeperiodMapper|None:
        """
        pass

    @property
    @abstractmethod
    def notification_options(self):
        """
        :return string|None:
        """
        pass

    @property
    @abstractmethod
    def notifications_enabled(self):
        """
        :return bool|None:
        """
        pass

    @property
    @abstractmethod
    def stalking_options(self):
        """
        :return string|None:
        """
        pass

    @property
    @abstractmethod
    def notes(self):
        """
        :return string|None:
        """
        pass

    @property
    @abstractmethod
    def notes_url(self):
        """
        :return string|None:
        """
        pass

    @property
    @abstractmethod
    def action_url(self):
        """
        :return string|None:
        """
        pass

    @property
    @abstractmethod
    def icon_image(self):
        """
        :return string|None:
        """
        pass

    @property
    @abstractmethod
    def icon_image_alt(self):
        """
        :return string|None:
        """
        pass

    @property
    @abstractmethod
    def statusmap_image(self):
        """
        :return string|None:
        """
        pass

    @property
    @abstractmethod
    def coords_2d(self):
        """
        :return string|None:
        """
        pass

    @property
    @abstractmethod
    def name(self):
        """
        :return string|none:
        """
        pass

    @property
    @abstractmethod
    def use(self):
        """
        :return BaseHostMapper|none:
        """
        pass

    @property
    @abstractmethod
    def register(self):
        """
        :return bool|none:
        """
        pass

    @property
    @abstractmethod
    def services(self):
        """
        :return List<BaseServiceMapper>|None:
        """
        pass
