""" Abstract class for methods which all mappers should implement """
from abc import ABC, abstractmethod


class BaseMapper(ABC):
    @property
    @abstractmethod
    def dependent_objects(self):
        """
        Returns the set of all objects which hold a reference to this objects
        :return Set<Mappers>
        """

    @property
    @abstractmethod
    def changelog_name(self):
        """
        Returns the name of the object as it would appear in a changelog
        :return str
        """

    @abstractmethod
    def generate_changelog(self):
        """
        Returns a very basic changelog entry for this object with its object
        type, object id and old/newname set.
        :return ChangelogMapper
        """
