from nachos.nacoma.mappers.sqlalchemy.mapper import Mapper
from nachos.nacoma.mappers.base.contact import BaseContactMapper
from nachos.nacoma.mappers.sqlalchemy.contactgroup import ContactgroupMapper
from nachos.nacoma.mappers.sqlalchemy.customvar import CustomvarMapper
from nachos.nacoma.mappers.sqlalchemy.file import FileMapper
from nachos.nacoma.mappers.sqlalchemy.timeperiod import TimeperiodMapper
from nachos.nacoma.consts import (HostNotificationOption,
                                  ServiceNotificationOption)


class ContactMapper(Mapper, BaseContactMapper):

    @property
    def id(self):
        return self.mapattr('id')

    @property
    def contact_name(self):
        return self.mapattr('contact_name')

    @property
    def alias(self):
        return self.mapattr('alias')

    @property
    def contactgroups(self):
        return self.mapattr('contactgroups', mapper=[ContactgroupMapper])

    @property
    def file(self):
        return self.mapattr('file', mapper=FileMapper)

    @property
    def host_notifications_enabled(self):
        return self.mapattr('host_notifications_enabled')

    @property
    def service_notifications_enabled(self):
        return self.mapattr('service_notifications_enabled')

    @property
    def can_submit_commands(self):
        return self.mapattr('can_submit_commands')

    @property
    def retain_status_information(self):
        return self.mapattr('retain_status_information')

    @property
    def retain_nonstatus_information(self):
        return self.mapattr('retain_nonstatus_information')

    @property
    def host_notification_timeperiod(self):
        return self.mapattr(
            'host_notification_timeperiod', mapper=TimeperiodMapper
        )

    @property
    def service_notification_timeperiod(self):
        return self.mapattr(
            'service_notification_timeperiod', mapper=TimeperiodMapper
        )

    @property
    def host_notification_options(self):
        options = self.mapattr('host_notification_options')
        if options is not None:
            try:
                return frozenset(
                    HostNotificationOption[option]
                    for option in options.split(',')
                )
            except KeyError:
                return None
        return None

    @property
    def service_notification_options(self):
        options = self.mapattr('service_notification_options')
        if options is not None:
            try:
                return frozenset(
                    ServiceNotificationOption[option]
                    for option in options.split(',')
                )
            except KeyError:
                return None
        return None

    @property
    def host_notification_commands(self):
        command = self.mapattr('host_notification_commands')
        if command is not None:
            cmd = command.command_name
            args = self.mapattr('host_notification_command_arguments')
            if args:
                return '{cmd}!{args}'.format(cmd=cmd, args=args)
            return cmd
        return None

    @property
    def service_notification_commands(self):
        command = self.mapattr('service_notification_commands')
        if command is not None:
            cmd = command.command_name
            args = self.mapattr('service_notification_command_arguments')
            if args:
                return '{cmd}!{args}'.format(cmd=cmd, args=args)
            return cmd
        return None

    @property
    def minimum_value(self):
        return self.mapattr('minimum_value')

    @property
    def email(self):
        return self.mapattr('email')

    @property
    def pager(self):
        return self.mapattr('pager')

    @property
    def address1(self):
        return self.mapattr('address1')

    @property
    def address2(self):
        return self.mapattr('address2')

    @property
    def address3(self):
        return self.mapattr('address3')

    @property
    def address4(self):
        return self.mapattr('address4')

    @property
    def address5(self):
        return self.mapattr('address5')

    @property
    def address6(self):
        return self.mapattr('address6')

    @property
    def name(self):
        return self.mapattr('name')

    @property
    def use(self):
        return self.mapattr('template', mapper=ContactMapper)

    @property
    def register(self):
        return self.mapattr('register')

    @property
    def custom_vars(self):
        return self.mapattr('custom_vars', mapper=[CustomvarMapper])

    @property
    def hosts(self):
        from nachos.nacoma.mappers import HostMapper
        return self.mapattr('hosts', mapper=[HostMapper])

    @property
    def services(self):
        from nachos.nacoma.mappers import ServiceMapper
        return self.mapattr('services', mapper=[ServiceMapper])

    @property
    def hostescalations(self):
        from nachos.nacoma.mappers import HostescalationMapper
        return self.mapattr('hostescalations', mapper=[HostescalationMapper])

    @property
    def serviceescalations(self):
        from nachos.nacoma.mappers import ServiceescalationMapper
        return self.mapattr(
            'serviceescalations', mapper=[ServiceescalationMapper]
        )

    @property
    def dependent_objects(self):
        objects = super().dependent_objects
        objects.update(
            self.hosts,
            self.services,
            self.hostescalations,
            self.serviceescalations,
            self.contactgroups
        )
        return objects

    @property
    def changelog_name(self):
        if self.register:
            return self.contact_name
        return self.name
