from nachos.nacoma.mappers.sqlalchemy.mapper import Mapper
from nachos.nacoma.consts import (
    HostState,
    HostStalkingOption,
    HostNotificationOption,
    HostFlapOption,
)
from nachos.nacoma.mappers.base.host import BaseHostMapper
from nachos.nacoma.mappers.sqlalchemy.file import FileMapper
from nachos.nacoma.mappers.sqlalchemy.timeperiod import TimeperiodMapper
from nachos.nacoma.mappers.sqlalchemy.customvar import CustomvarMapper
from nachos.nacoma.mappers.sqlalchemy.contact import ContactMapper
from nachos.nacoma.mappers.sqlalchemy.contactgroup import ContactgroupMapper
from nachos.nacoma.mappers.sqlalchemy.hostgroup import HostgroupMapper


class HostMapper(Mapper, BaseHostMapper):

    @property
    def id(self):
        return self.mapattr('id')

    @property
    def host_name(self):
        return self.mapattr('host_name')

    @property
    def alias(self):
        return self.mapattr('alias')

    @property
    def display_name(self):
        return self.mapattr('display_name')

    @property
    def file(self):
        return self.mapattr('file', mapper=FileMapper)

    @property
    def parents(self):
        return self.mapattr('parents', mapper=[HostMapper])

    @property
    def hostgroups(self):
        return self.mapattr('hostgroups', mapper=[HostgroupMapper])

    @property
    def address(self):
        return self.mapattr('address')

    @property
    def check_command(self):
        command = self.mapattr('check_command')
        if command is not None:
            cmd = command.command_name
            args = self.mapattr('check_command_arguments')
            if args:
                return '{cmd}!{args}'.format(cmd=cmd, args=args)
            return cmd
        return None

    @property
    def initial_state(self):
        state = self.mapattr('initial_state', mapper=HostState)
        try:
            return HostState(state)
        except Exception:
            return None

    @property
    def max_check_attempts(self):
        return self.mapattr('max_check_attempts')

    @property
    def check_interval(self):
        return self.mapattr('check_interval')

    @property
    def retry_interval(self):
        return self.mapattr('retry_interval')

    @property
    def active_checks_enabled(self):
        return self.mapattr('active_checks_enabled')

    @property
    def passive_checks_enabled(self):
        return self.mapattr('passive_checks_enabled')

    @property
    def check_period(self):
        return self.mapattr('check_period', mapper=TimeperiodMapper)

    @property
    def obsess(self):
        return self.mapattr('obsess')

    @property
    def check_freshness(self):
        return self.mapattr('check_freshness')

    @property
    def freshness_threshold(self):
        return self.mapattr('freshness_threshold')

    @property
    def event_handler(self):
        command = self.mapattr('event_handler')
        if command is not None:
            cmd = command.command_name
            args = self.mapattr('event_handler_arguments')
            if args:
                return '{cmd}!{args}'.format(cmd=cmd, args=args)
            return cmd
        return None

    @property
    def event_handler_enabled(self):
        return self.mapattr('event_handler_enabled')

    @property
    def low_flap_threshold(self):
        return self.mapattr('low_flap_threshold')

    @property
    def high_flap_threshold(self):
        return self.mapattr('high_flap_threshold')

    @property
    def flap_detection_enabled(self):
        return self.mapattr('flap_detection_enabled')

    @property
    def flap_detection_options(self):
        options = self.mapattr('flap_detection_options')
        if options is not None:
            try:
                return frozenset(
                    HostFlapOption[option] for option in options.split(',')
                )
            except Exception:
                return None
        return None

    @property
    def process_perf_data(self):
        return self.mapattr('process_perf_data')

    @property
    def retain_status_information(self):
        return self.mapattr('retain_status_information')

    @property
    def retain_nonstatus_information(self):
        return self.mapattr('retain_nonstatus_information')

    @property
    def contacts(self):
        return self.mapattr('contacts', mapper=[ContactMapper])

    @property
    def contact_groups(self):
        return self.mapattr('contactgroups', mapper=[ContactgroupMapper])

    @property
    def notification_interval(self):
        return self.mapattr('notification_interval')

    @property
    def first_notification_delay(self):
        return self.mapattr('first_notification_delay')

    @property
    def notification_period(self):
        return self.mapattr('notification_timeperiod', mapper=TimeperiodMapper)

    @property
    def notification_options(self):
        options = self.mapattr('notification_options')
        if options is not None:
            try:
                return frozenset(
                    HostNotificationOption[option]
                    for option in options.split(',')
                )
            except Exception:
                return None
        return None

    @property
    def notifications_enabled(self):
        return self.mapattr('notifications_enabled')

    @property
    def stalking_options(self):
        options = self.mapattr('stalking_options')
        if options is not None:
            try:
                return frozenset(
                    HostStalkingOption[option] for
                    option in options.split(',')
                )
            except Exception:
                return None
        return None

    @property
    def notes(self):
        return self.mapattr('notes')

    @property
    def notes_url(self):
        return self.mapattr('notes_url')

    @property
    def action_url(self):
        return self.mapattr('action_url')

    @property
    def icon_image(self):
        return self.mapattr('icon_image')

    @property
    def icon_image_alt(self):
        return self.mapattr('icon_image_alt')

    @property
    def statusmap_image(self):
        return self.mapattr('statusmap_image')

    @property
    def coords_2d(self):
        return self.mapattr('coords_2d')

    @property
    def hourly_value(self):
        return self.mapattr('hourly_value')

    @property
    def name(self):
        return self.mapattr('name')

    @property
    def custom_vars(self):
        return self.mapattr('custom_vars', mapper=[CustomvarMapper])

    @property
    def use(self):
        return self.mapattr('template', mapper=HostMapper)

    @property
    def register(self):
        return self.mapattr('register')

    @property
    def services(self):
        from nachos.nacoma.mappers.sqlalchemy.service import ServiceMapper
        return self.mapattr('services', mapper=[ServiceMapper])

    @property
    def changelog_name(self):
        if self.register:
            return self.host_name
        return self.name
