from nachos.nacoma.mappers.sqlalchemy.mapper import Mapper
from nachos.nacoma.consts import HostEscalationOption
from nachos.nacoma.mappers.base.hostescalation import BaseHostescalationMapper
from nachos.nacoma.mappers.sqlalchemy.contact import ContactMapper
from nachos.nacoma.mappers.sqlalchemy.contactgroup import ContactgroupMapper
from nachos.nacoma.mappers.sqlalchemy.file import FileMapper
from nachos.nacoma.mappers.sqlalchemy.host import HostMapper
from nachos.nacoma.mappers.sqlalchemy.hostgroup import HostgroupMapper
from nachos.nacoma.mappers.sqlalchemy.timeperiod import TimeperiodMapper


class HostescalationMapper(Mapper, BaseHostescalationMapper):

    @property
    def host(self):
        return self.mapattr('host', mapper=HostMapper)

    @property
    def hostgroup(self):
        return self.mapattr('hostgroup', mapper=HostgroupMapper)

    @property
    def contacts(self):
        contacts = self.mapattr('contacts', mapper=[ContactMapper])
        if contacts:
            contacts.sort(key=lambda mapper: mapper.contact_name)
        return contacts

    @property
    def contact_groups(self):
        contact_groups = self.mapattr('contactgroups', mapper=[ContactgroupMapper])
        if contact_groups:
            contact_groups.sort(key=lambda mapper: mapper.contactgroup_name)
        return contact_groups

    @property
    def file(self):
        return self.mapattr('file', mapper=FileMapper)

    @property
    def first_notification(self):
        return self.mapattr('first_notification')

    @property
    def last_notification(self):
        return self.mapattr('last_notification')

    @property
    def notification_interval(self):
        return self.mapattr('notification_interval')

    @property
    def escalation_timeperiod(self):
        return self.mapattr('escalation_timeperiod', mapper=TimeperiodMapper)

    @property
    def escalation_options(self):
        options = self.mapattr('escalation_options')
        if options is not None:
            try:
                return frozenset(
                    HostEscalationOption[option]
                    for option in options.split(',')
                )
            except Exception:
                return None
        return None

    @property
    def name(self):
        return self.mapattr('name')

    @property
    def use(self):
        return self.mapattr('template')

    @property
    def register(self):
        return self.mapattr('register')
