from nachos.nacoma.mappers import BaseMapper


class Mapper(BaseMapper):
    def __init__(self, sqlalchemy_obj):
        self.sqlalchemy_obj = sqlalchemy_obj

    def mapattr(self, attr, mapper=None):
        value = getattr(self.sqlalchemy_obj, attr, None)
        if value is None or mapper is None:
            return value
        if isinstance(mapper, list):
            return [mapper[0](v) for v in value]
        return mapper(value)

    @property
    def object_type(self):
        return self.sqlalchemy_obj.__tablename__

    @property
    def dependent_objects(self):
        from nachos.nacoma.common.utils import get_dependent_objects
        return get_dependent_objects(self.sqlalchemy_obj)

    @property
    def changelog_name(self):
        raise RuntimeError(
            "Object type {} does not have a name".
            format(self.object_type)
        )

    def generate_changelog(self):
        from nachos.nacoma.mappers import ChangelogMapper
        from nachos.nacoma.api.sqlalchemy.models import Changelog
        name = self.changelog_name
        change = ChangelogMapper(
            Changelog(
                object_id=self.sqlalchemy_obj.id,
                object_type=self.object_type,
                oldname=name,
                newname=name,
            )
        )
        return change
