from nachos.nacoma.mappers.sqlalchemy.mapper import Mapper
from nachos.nacoma.consts import ServiceEscalationOption
from nachos.nacoma.mappers.base.serviceescalation import BaseServiceescalationMapper  # noqa: E501
from nachos.nacoma.mappers.sqlalchemy.contact import ContactMapper
from nachos.nacoma.mappers.sqlalchemy.contactgroup import ContactgroupMapper
from nachos.nacoma.mappers.sqlalchemy.file import FileMapper
from nachos.nacoma.mappers.sqlalchemy.service import ServiceMapper
from nachos.nacoma.mappers.sqlalchemy.timeperiod import TimeperiodMapper


class ServiceescalationMapper(Mapper, BaseServiceescalationMapper):

    @property
    def host(self):
        service = self.service
        return service.host if service else None

    @property
    def hostgroup(self):
        service = self.service
        return service.hostgroup if service else None

    @property
    def file(self):
        return self.mapattr('file', mapper=FileMapper)

    @property
    def service(self):
        return self.mapattr('service', mapper=ServiceMapper)

    @property
    def contacts(self):
        return self.mapattr('contacts', mapper=[ContactMapper])

    @property
    def contact_groups(self):
        return self.mapattr('contactgroups', mapper=[ContactgroupMapper])

    @property
    def first_notification(self):
        return self.mapattr('first_notification')

    @property
    def last_notification(self):
        return self.mapattr('last_notification')

    @property
    def notification_interval(self):
        return self.mapattr('notification_interval')

    @property
    def escalation_period(self):
        return self.mapattr('escalation_period', mapper=TimeperiodMapper)

    @property
    def escalation_options(self):
        options = self.mapattr('escalation_options')
        if options is not None:
            try:
                return frozenset(
                    ServiceEscalationOption[option]
                    for option in options.split(',')
                )
            except (ValueError, KeyError):
                return None
        return None

    @property
    def name(self):
        return self.mapattr('name')

    @property
    def use(self):
        return self.mapattr('template')

    @property
    def register(self):
        return self.mapattr('register')
