<?php
$description = <<<END_OF_HTML
<h2>Using the <a href='http://en.wikipedia.org/wiki/Representational_state_transfer#RESTful_web_services' title='Representational state transfer'>REST</a> API</h2>

<h3>Introduction</h3>

<p>Ask Monitor what it knows about your system in this very moment. Filters are written in a made up query language which let's you retrieve data based on certain columns' values, for example "only show me data belonging to hosts whose name begins with an 'A'".</p>

<ul>
	<li>You can be very expressive and ask for e.g. "all hosts with a problem state that are not acknowledged and not in downtime" with a single HTTP request. <strong>Every filter that you have defined in Monitor is copy &amp; pastable into the HTTP API, via the <em>query</em> GET parameter.</strong></li>
	<li>You can limit the data transfered by:
		<ol>
			<li>defining what columns you would like in the output. If you only need to display a list of problem hosts' names and if they are acknowledged or not, you can limit the result to only include those two columns.</br>
			This is done through the <em>columns</em> GET parameter, as in <code>?columns=name,state</code></li>
			<li>using the <em>limit</em> and <em>offset</em> variables, as if you are paginating something. Append the GET parameters <code>?limit=5&amp;offset=3</code> to any request to see them in action.</li>
		</ol>
	</li>
	<li>All tables and columns are documented in the article <a href="${docs_url}topics/other-documents/gui-ninja/listview-filter-columns.html" target="_blank">Listview filter columns</a>.</li>
</ul>

<p class="meta">Start out by choosing any of the sections on the left, either count or query.</p>

<h3>Technical information</h3>

<h4>Authentication</h4>

<p>You send along a pair of regular op5 Monitor user credentials in a basic auth header with every request. View any example and you will understand how to do this.</p>

<h4>Requests</h4>
<p>In a REST-API, there are a couple of important things that make up for a valid request:</p>
<ul>
	<li>
		The URI, in the following form
		<ul>
			<li>/api/filter/</li>
			<li>/api/filter/query</li>
			<li>/api/filter/count</li>
		</ul>
	</li>
	<li>
		The HTTP method
		<ul>
			<li>GET: retrieve data</li>
		</ul>
	</li>
	<li>
		The format, appended to the URI
		<ul>
			<li>?format=xml</li>
			<li>?format=json (default, except for requests from a browser)</li>
			<li>?format=html (default in a browser environment)</li>
		</ul>
	</li>
</ul>

<p>Once again, viewing any of the examples helps you get into the REST mindset.</p>

<h4>Responses</h4>
<p>You need to examine two things in the response: the status code and the body. The status code tells you what Monitor did with your input and the body gives you a detailed response.</p>

<p>Examples of status codes:</p>
<ul>
	<li>200: Everything went fine.</li>
	<li>400: You sent a request Monitor didn't understand. Correct the input and try again.</li>
	<li>401: Authentication required.</li>
	<li>403: You're not authorized to do this.</li>
	<li>404: The URI doesn't exist.</li>
</ul>

<p>These status codes are general for all web traffic and can be found at the <a href='http://en.wikipedia.org/wiki/HTTP_status_code'>List of HTTP status codes</a></p>
END_OF_HTML;
