
/*******************
 * From src: synergy/src/js/10.synergy.validate.js
 *******************/
/**
 * jQuery Validation Plugin 1.9.0
 *
 * http://bassistance.de/jquery-plugins/jquery-plugin-validation/
 * http://docs.jquery.com/Plugins/Validation
 *
 * Copyright (c) 2006 - 2011 Jörn Zaefferer
 *
 * Dual licensed under the MIT and GPL licenses:
 *   http://www.opensource.org/licenses/mit-license.php
 *   http://www.gnu.org/licenses/gpl.html
 */

(function($, jQuery) {

	(function($) {
	
	$.extend($.fn, {
		// http://docs.jquery.com/Plugins/Validation/validate
		validate: function( options ) {
	
			// if nothing is selected, return nothing; can't chain anyway
			if (!this.length) {
				options && options.debug && window.console && console.warn( "nothing selected, can't validate, returning nothing" );
				return;
			}
	
			// check if a validator for this form was already created
			var validator = $.data(this[0], 'validator');
			if ( validator ) {
				return validator;
			}
	
			// Add novalidate tag if HTML5.
			this.attr('novalidate', 'novalidate');
	
			validator = new $.validator( options, this[0] );
			$.data(this[0], 'validator', validator);
	
			if ( validator.settings.onsubmit ) {
	
				var inputsAndButtons = this.find("input, button");
	
				// allow suppresing validation by adding a cancel class to the submit button
				inputsAndButtons.filter(".cancel").click(function () {
					validator.cancelSubmit = true;
				});
	
				// when a submitHandler is used, capture the submitting button
				if (validator.settings.submitHandler) {
					inputsAndButtons.filter(":submit").click(function () {
						validator.submitButton = this;
					});
				}
	
				// validate the form on submit
				this.submit( function( event ) {
					if ( validator.settings.debug )
						// prevent form submit to be able to see console output
						event.preventDefault();
	
					function handle() {
						if ( validator.settings.submitHandler ) {
							if (validator.submitButton) {
								// insert a hidden input as a replacement for the missing submit button
								var hidden = $("<input type='hidden'/>").attr("name", validator.submitButton.name).val(validator.submitButton.value).appendTo(validator.currentForm);
							}
							validator.settings.submitHandler.call( validator, validator.currentForm );
							if (validator.submitButton) {
								// and clean up afterwards; thanks to no-block-scope, hidden can be referenced
								hidden.remove();
							}
							return false;
						}
						return true;
					}
	
					// prevent submit for invalid forms or custom submit handlers
					if ( validator.cancelSubmit ) {
						validator.cancelSubmit = false;
						return handle();
					}
					if ( validator.form() ) {
						if ( validator.pendingRequest ) {
							validator.formSubmitted = true;
							return false;
						}
						return handle();
					} else {
						validator.focusInvalid();
						return false;
					}
				});
			}
	
			return validator;
		},
		// http://docs.jquery.com/Plugins/Validation/valid
		valid: function() {
			if ( $(this[0]).is('form')) {
				return this.validate().form();
			} else {
				var valid = true;
				var validator = $(this[0].form).validate();
				this.each(function() {
					valid &= validator.element(this);
				});
				return valid;
			}
		},
		// attributes: space seperated list of attributes to retrieve and remove
		removeAttrs: function(attributes) {
			var result = {},
				$element = this;
			$.each(attributes.split(/\s/), function(index, value) {
				result[value] = $element.attr(value);
				$element.removeAttr(value);
			});
			return result;
		},
		// http://docs.jquery.com/Plugins/Validation/rules
		rules: function(command, argument) {
			var element = this[0];
	
			if (command) {
				var settings = $.data(element.form, 'validator').settings;
				var staticRules = settings.rules;
				var existingRules = $.validator.staticRules(element);
				switch(command) {
				case "add":
					$.extend(existingRules, $.validator.normalizeRule(argument));
					staticRules[element.name] = existingRules;
					if (argument.messages)
						settings.messages[element.name] = $.extend( settings.messages[element.name], argument.messages );
					break;
				case "remove":
					if (!argument) {
						delete staticRules[element.name];
						return existingRules;
					}
					var filtered = {};
					$.each(argument.split(/\s/), function(index, method) {
						filtered[method] = existingRules[method];
						delete existingRules[method];
					});
					return filtered;
				}
			}
	
			var data = $.validator.normalizeRules(
			$.extend(
				{},
				$.validator.metadataRules(element),
				$.validator.classRules(element),
				$.validator.attributeRules(element),
				$.validator.staticRules(element)
			), element);
	
			// make sure required is at front
			if (data.required) {
				var param = data.required;
				delete data.required;
				data = $.extend({required: param}, data);
			}
	
			return data;
		}
	});
	
	// Custom selectors
	$.extend($.expr[":"], {
		// http://docs.jquery.com/Plugins/Validation/blank
		blank: function(a) {return !$.trim("" + a.value);},
		// http://docs.jquery.com/Plugins/Validation/filled
		filled: function(a) {return !!$.trim("" + a.value);},
		// http://docs.jquery.com/Plugins/Validation/unchecked
		unchecked: function(a) {return !a.checked;}
	});
	
	// constructor for validator
	$.validator = function( options, form ) {
		this.settings = $.extend( true, {}, $.validator.defaults, options );
		this.currentForm = form;
		this.init();
	};
	
	$.validator.format = function(source, params) {
		if ( arguments.length == 1 )
			return function() {
				var args = $.makeArray(arguments);
				args.unshift(source);
				return $.validator.format.apply( this, args );
			};
		if ( arguments.length > 2 && params.constructor != Array  ) {
			params = $.makeArray(arguments).slice(1);
		}
		if ( params.constructor != Array ) {
			params = [ params ];
		}
		$.each(params, function(i, n) {
			source = source.replace(new RegExp("\\{" + i + "\\}", "g"), n);
		});
		return source;
	};
	
	$.extend($.validator, {
	
		defaults: {
			messages: {},
			groups: {},
			rules: {},
			errorClass: "error",
			validClass: "valid",
			errorElement: "label",
			focusInvalid: true,
			errorContainer: $( [] ),
			errorLabelContainer: $( [] ),
			onsubmit: true,
			ignore: ":hidden",
			ignoreTitle: false,
			onfocusin: function(element, event) {
				this.lastActive = element;
	
				// hide error label and remove error class on focus if enabled
				if ( this.settings.focusCleanup && !this.blockFocusCleanup ) {
					this.settings.unhighlight && this.settings.unhighlight.call( this, element, this.settings.errorClass, this.settings.validClass );
					this.addWrapper(this.errorsFor(element)).hide();
				}
			},
			onfocusout: function(element, event) {
				if ( !this.checkable(element) && (element.name in this.submitted || !this.optional(element)) ) {
					this.element(element);
				}
			},
			onkeyup: function(element, event) {
				if ( element.name in this.submitted || element == this.lastElement ) {
					this.element(element);
				}
			},
			onclick: function(element, event) {
				// click on selects, radiobuttons and checkboxes
				if ( element.name in this.submitted )
					this.element(element);
				// or option elements, check parent select in that case
				else if (element.parentNode.name in this.submitted)
					this.element(element.parentNode);
			},
			highlight: function(element, errorClass, validClass) {
				if (element.type === 'radio') {
					this.findByName(element.name).addClass(errorClass).removeClass(validClass);
				} else {
					$(element).addClass(errorClass).removeClass(validClass);
				}
			},
			unhighlight: function(element, errorClass, validClass) {
				if (element.type === 'radio') {
					this.findByName(element.name).removeClass(errorClass).addClass(validClass);
				} else {
					$(element).removeClass(errorClass).addClass(validClass);
				}
			}
		},
	
		// http://docs.jquery.com/Plugins/Validation/Validator/setDefaults
		setDefaults: function(settings) {
			$.extend( $.validator.defaults, settings );
		},
	
		messages: {
			required: "This field is required.",
			remote: "Please fix this field.",
			email: "Please enter a valid email address.",
			url: "Please enter a valid URL.",
			date: "Please enter a valid date.",
			dateISO: "Please enter a valid date (ISO).",
			number: "Please enter a valid number.",
			digits: "Please enter only digits.",
			creditcard: "Please enter a valid credit card number.",
			equalTo: "Please enter the same value again.",
			accept: "Please enter a value with a valid extension.",
			maxlength: $.validator.format("Please enter no more than {0} characters."),
			minlength: $.validator.format("Please enter at least {0} characters."),
			rangelength: $.validator.format("Please enter a value between {0} and {1} characters long."),
			range: $.validator.format("Please enter a value between {0} and {1}."),
			max: $.validator.format("Please enter a value less than or equal to {0}."),
			min: $.validator.format("Please enter a value greater than or equal to {0}.")
		},
	
		autoCreateRanges: false,
	
		prototype: {
	
			init: function() {
				this.labelContainer = $(this.settings.errorLabelContainer);
				this.errorContext = this.labelContainer.length && this.labelContainer || $(this.currentForm);
				this.containers = $(this.settings.errorContainer).add( this.settings.errorLabelContainer );
				this.submitted = {};
				this.valueCache = {};
				this.pendingRequest = 0;
				this.pending = {};
				this.invalid = {};
				this.reset();
	
				var groups = (this.groups = {});
				$.each(this.settings.groups, function(key, value) {
					$.each(value.split(/\s/), function(index, name) {
						groups[name] = key;
					});
				});
				var rules = this.settings.rules;
				$.each(rules, function(key, value) {
					rules[key] = $.validator.normalizeRule(value);
				});
	
				function delegate(event) {
					var validator = $.data(this[0].form, "validator"),
						eventType = "on" + event.type.replace(/^validate/, "");
					validator.settings[eventType] && validator.settings[eventType].call(validator, this[0], event);
				}
				$(this.currentForm)
					   .validateDelegate("[type='text'], [type='password'], [type='file'], select, textarea, " +
							"[type='number'], [type='search'] ,[type='tel'], [type='url'], " +
							"[type='email'], [type='datetime'], [type='date'], [type='month'], " +
							"[type='week'], [type='time'], [type='datetime-local'], " +
							"[type='range'], [type='color'] ",
							"focusin focusout keyup", delegate)
					.validateDelegate("[type='radio'], [type='checkbox'], select, option", "click", delegate);
	
				if (this.settings.invalidHandler)
					$(this.currentForm).bind("invalid-form.validate", this.settings.invalidHandler);
			},
	
			// http://docs.jquery.com/Plugins/Validation/Validator/form
			form: function() {
				this.checkForm();
				$.extend(this.submitted, this.errorMap);
				this.invalid = $.extend({}, this.errorMap);
				if (!this.valid())
					$(this.currentForm).triggerHandler("invalid-form", [this]);
				this.showErrors();
				return this.valid();
			},
	
			checkForm: function() {
				this.prepareForm();
				for ( var i = 0, elements = (this.currentElements = this.elements()); elements[i]; i++ ) {
					this.check( elements[i] );
				}
				return this.valid();
			},
	
			// http://docs.jquery.com/Plugins/Validation/Validator/element
			element: function( element ) {
				element = this.validationTargetFor( this.clean( element ) );
				this.lastElement = element;
				this.prepareElement( element );
				this.currentElements = $(element);
				var result = this.check( element );
				if ( result ) {
					delete this.invalid[element.name];
				} else {
					this.invalid[element.name] = true;
				}
				if ( !this.numberOfInvalids() ) {
					// Hide error containers on last error
					this.toHide = this.toHide.add( this.containers );
				}
				this.showErrors();
				return result;
			},
	
			// http://docs.jquery.com/Plugins/Validation/Validator/showErrors
			showErrors: function(errors) {
				if(errors) {
					// add items to error list and map
					$.extend( this.errorMap, errors );
					this.errorList = [];
					for ( var name in errors ) {
						this.errorList.push({
							message: errors[name],
							element: this.findByName(name)[0]
						});
					}
					// remove items from success list
					this.successList = $.grep( this.successList, function(element) {
						return !(element.name in errors);
					});
				}
				this.settings.showErrors
					? this.settings.showErrors.call( this, this.errorMap, this.errorList )
					: this.defaultShowErrors();
			},
	
			// http://docs.jquery.com/Plugins/Validation/Validator/resetForm
			resetForm: function() {
				if ( $.fn.resetForm )
					$( this.currentForm ).resetForm();
				this.submitted = {};
				this.lastElement = null;
				this.prepareForm();
				this.hideErrors();
				this.elements().removeClass( this.settings.errorClass );
			},
	
			numberOfInvalids: function() {
				return this.objectLength(this.invalid);
			},
	
			objectLength: function( obj ) {
				var count = 0;
				for ( var i in obj )
					count++;
				return count;
			},
	
			hideErrors: function() {
				this.addWrapper( this.toHide ).hide();
			},
	
			valid: function() {
				return this.size() == 0;
			},
	
			size: function() {
				return this.errorList.length;
			},
	
			focusInvalid: function() {
				if( this.settings.focusInvalid ) {
					try {
						$(this.findLastActive() || this.errorList.length && this.errorList[0].element || [])
						.filter(":visible")
						.focus()
						// manually trigger focusin event; without it, focusin handler isn't called, findLastActive won't have anything to find
						.trigger("focusin");
					} catch(e) {
						// ignore IE throwing errors when focusing hidden elements
					}
				}
			},
	
			findLastActive: function() {
				var lastActive = this.lastActive;
				return lastActive && $.grep(this.errorList, function(n) {
					return n.element.name == lastActive.name;
				}).length == 1 && lastActive;
			},
	
			elements: function() {
				var validator = this,
					rulesCache = {};
	
				// select all valid inputs inside the form (no submit or reset buttons)
				return $(this.currentForm)
				.find("input, select, textarea")
				.not(":submit, :reset, :image, [disabled]")
				.not( this.settings.ignore )
				.filter(function() {
					!this.name && validator.settings.debug && window.console && console.error( "%o has no name assigned", this);
	
					// select only the first element for each name, and only those with rules specified
					if ( this.name in rulesCache || !validator.objectLength($(this).rules()) )
						return false;
	
					rulesCache[this.name] = true;
					return true;
				});
			},
	
			clean: function( selector ) {
				return $( selector )[0];
			},
	
			errors: function() {
				return $( this.settings.errorElement + "." + this.settings.errorClass, this.errorContext );
			},
	
			reset: function() {
				this.successList = [];
				this.errorList = [];
				this.errorMap = {};
				this.toShow = $([]);
				this.toHide = $([]);
				this.currentElements = $([]);
			},
	
			prepareForm: function() {
				this.reset();
				this.toHide = this.errors().add( this.containers );
			},
	
			prepareElement: function( element ) {
				this.reset();
				this.toHide = this.errorsFor(element);
			},
	
			check: function( element ) {
				element = this.validationTargetFor( this.clean( element ) );
	
				var rules = $(element).rules();
				var dependencyMismatch = false;
				for (var method in rules ) {
					var rule = { method: method, parameters: rules[method] };
					try {
						var result = $.validator.methods[method].call( this, element.value.replace(/\r/g, ""), element, rule.parameters );
	
						// if a method indicates that the field is optional and therefore valid,
						// don't mark it as valid when there are no other rules
						if ( result == "dependency-mismatch" ) {
							dependencyMismatch = true;
							continue;
						}
						dependencyMismatch = false;
	
						if ( result == "pending" ) {
							this.toHide = this.toHide.not( this.errorsFor(element) );
							return;
						}
	
						if( !result ) {
							this.formatAndAdd( element, rule );
							return false;
						}
					} catch(e) {
						this.settings.debug && window.console && console.log("exception occured when checking element " + element.id
							 + ", check the '" + rule.method + "' method", e);
						throw e;
					}
				}
				if (dependencyMismatch)
					return;
				if ( this.objectLength(rules) )
					this.successList.push(element);
				return true;
			},
	
			// return the custom message for the given element and validation method
			// specified in the element's "messages" metadata
			customMetaMessage: function(element, method) {
				if (!$.metadata)
					return;
	
				var meta = this.settings.meta
					? $(element).metadata()[this.settings.meta]
					: $(element).metadata();
	
				return meta && meta.messages && meta.messages[method];
			},
	
			// return the custom message for the given element name and validation method
			customMessage: function( name, method ) {
				var m = this.settings.messages[name];
				return m && (m.constructor == String
					? m
					: m[method]);
			},
	
			// return the first defined argument, allowing empty strings
			findDefined: function() {
				for(var i = 0; i < arguments.length; i++) {
					if (arguments[i] !== undefined)
						return arguments[i];
				}
				return undefined;
			},
	
			defaultMessage: function( element, method) {
				return this.findDefined(
					this.customMessage( element.name, method ),
					this.customMetaMessage( element, method ),
					// title is never undefined, so handle empty string as undefined
					!this.settings.ignoreTitle && element.title || undefined,
					$.validator.messages[method],
					"<strong>Warning: No message defined for " + element.name + "</strong>"
				);
			},
	
			formatAndAdd: function( element, rule ) {
				var message = this.defaultMessage( element, rule.method ),
					theregex = /\$?\{(\d+)\}/g;
				if ( typeof message == "function" ) {
					message = message.call(this, rule.parameters, element);
				} else if (theregex.test(message)) {
					message = jQuery.format(message.replace(theregex, '{$1}'), rule.parameters);
				}
				this.errorList.push({
					message: message,
					element: element
				});
	
				this.errorMap[element.name] = message;
				this.submitted[element.name] = message;
			},
	
			addWrapper: function(toToggle) {
				if ( this.settings.wrapper )
					toToggle = toToggle.add( toToggle.parent( this.settings.wrapper ) );
				return toToggle;
			},
	
			defaultShowErrors: function() {
				for ( var i = 0; this.errorList[i]; i++ ) {
					var error = this.errorList[i];
					this.settings.highlight && this.settings.highlight.call( this, error.element, this.settings.errorClass, this.settings.validClass );
					this.showLabel( error.element, error.message );
				}
				if( this.errorList.length ) {
					this.toShow = this.toShow.add( this.containers );
				}
				if (this.settings.success) {
					for ( var i = 0; this.successList[i]; i++ ) {
						this.showLabel( this.successList[i] );
					}
				}
				if (this.settings.unhighlight) {
					for ( var i = 0, elements = this.validElements(); elements[i]; i++ ) {
						this.settings.unhighlight.call( this, elements[i], this.settings.errorClass, this.settings.validClass );
					}
				}
				this.toHide = this.toHide.not( this.toShow );
				this.hideErrors();
				this.addWrapper( this.toShow ).show();
			},
	
			validElements: function() {
				return this.currentElements.not(this.invalidElements());
			},
	
			invalidElements: function() {
				return $(this.errorList).map(function() {
					return this.element;
				});
			},
	
			showLabel: function(element, message) {
				var label = this.errorsFor( element );
				if ( label.length ) {
					// refresh error/success class
					label.removeClass( this.settings.validClass ).addClass( this.settings.errorClass );
	
					// check if we have a generated label, replace the message then
					label.attr("generated") && label.html(message);
				} else {
					// create label
					label = $("<" + this.settings.errorElement + "/>")
						.attr({"for":  this.idOrName(element), generated: true})
						.addClass(this.settings.errorClass)
						.html(message || "");
					if ( this.settings.wrapper ) {
						// make sure the element is visible, even in IE
						// actually showing the wrapped element is handled elsewhere
						label = label.hide().show().wrap("<" + this.settings.wrapper + "/>").parent();
					}
					if ( !this.labelContainer.append(label).length )
						this.settings.errorPlacement
							? this.settings.errorPlacement(label, $(element) )
							: label.insertAfter(element);
				}
				if ( !message && this.settings.success ) {
					label.text("");
					typeof this.settings.success == "string"
						? label.addClass( this.settings.success )
						: this.settings.success( label );
				}
				this.toShow = this.toShow.add(label);
			},
	
			errorsFor: function(element) {
				var name = this.idOrName(element);
			return this.errors().filter(function() {
					return $(this).attr('for') == name;
				});
			},
	
			idOrName: function(element) {
				return this.groups[element.name] || (this.checkable(element) ? element.name : element.id || element.name);
			},
	
			validationTargetFor: function(element) {
				// if radio/checkbox, validate first element in group instead
				if (this.checkable(element)) {
					element = this.findByName( element.name ).not(this.settings.ignore)[0];
				}
				return element;
			},
	
			checkable: function( element ) {
				return /radio|checkbox/i.test(element.type);
			},
	
			findByName: function( name ) {
				// select by name and filter by form for performance over form.find("[name=...]")
				var form = this.currentForm;
				return $(document.getElementsByName(name)).map(function(index, element) {
					return element.form == form && element.name == name && element  || null;
				});
			},
	
			getLength: function(value, element) {
				switch( element.nodeName.toLowerCase() ) {
				case 'select':
					return $("option:selected", element).length;
				case 'input':
					if( this.checkable( element) )
						return this.findByName(element.name).filter(':checked').length;
				}
				return value.length;
			},
	
			depend: function(param, element) {
				return this.dependTypes[typeof param]
					? this.dependTypes[typeof param](param, element)
					: true;
			},
	
			dependTypes: {
				"boolean": function(param, element) {
					return param;
				},
				"string": function(param, element) {
					return !!$(param, element.form).length;
				},
				"function": function(param, element) {
					return param(element);
				}
			},
	
			optional: function(element) {
				return !$.validator.methods.required.call(this, $.trim(element.value), element) && "dependency-mismatch";
			},
	
			startRequest: function(element) {
				if (!this.pending[element.name]) {
					this.pendingRequest++;
					this.pending[element.name] = true;
				}
			},
	
			stopRequest: function(element, valid) {
				this.pendingRequest--;
				// sometimes synchronization fails, make sure pendingRequest is never < 0
				if (this.pendingRequest < 0)
					this.pendingRequest = 0;
				delete this.pending[element.name];
				if ( valid && this.pendingRequest == 0 && this.formSubmitted && this.form() ) {
					$(this.currentForm).submit();
					this.formSubmitted = false;
				} else if (!valid && this.pendingRequest == 0 && this.formSubmitted) {
					$(this.currentForm).triggerHandler("invalid-form", [this]);
					this.formSubmitted = false;
				}
			},
	
			previousValue: function(element) {
				return $.data(element, "previousValue") || $.data(element, "previousValue", {
					old: null,
					valid: true,
					message: this.defaultMessage( element, "remote" )
				});
			}
	
		},
	
		classRuleSettings: {
			required: {required: true},
			email: {email: true},
			url: {url: true},
			date: {date: true},
			dateISO: {dateISO: true},
			dateDE: {dateDE: true},
			number: {number: true},
			numberDE: {numberDE: true},
			digits: {digits: true},
			creditcard: {creditcard: true}
		},
	
		addClassRules: function(className, rules) {
			className.constructor == String ?
				this.classRuleSettings[className] = rules :
				$.extend(this.classRuleSettings, className);
		},
	
		classRules: function(element) {
			var rules = {};
			var classes = $(element).attr('class');
			classes && $.each(classes.split(' '), function() {
				if (this in $.validator.classRuleSettings) {
					$.extend(rules, $.validator.classRuleSettings[this]);
				}
			});
			return rules;
		},
	
		attributeRules: function(element) {
			var rules = {};
			var $element = $(element);
	
			for (var method in $.validator.methods) {
				var value;
				// If .prop exists (jQuery >= 1.6), use it to get true/false for required
				if (method === 'required' && typeof $.fn.prop === 'function') {
					value = $element.prop(method);
				} else {
					value = $element.attr(method);
				}
				if (value) {
					rules[method] = value;
				} else if ($element[0].getAttribute("type") === method) {
					rules[method] = true;
				}
			}
	
			// maxlength may be returned as -1, 2147483647 (IE) and 524288 (safari) for text inputs
			if (rules.maxlength && /-1|2147483647|524288/.test(rules.maxlength)) {
				delete rules.maxlength;
			}
	
			return rules;
		},
	
		metadataRules: function(element) {
			if (!$.metadata) return {};
	
			var meta = $.data(element.form, 'validator').settings.meta;
			return meta ?
				$(element).metadata()[meta] :
				$(element).metadata();
		},
	
		staticRules: function(element) {
			var rules = {};
			var validator = $.data(element.form, 'validator');
			if (validator.settings.rules) {
				rules = $.validator.normalizeRule(validator.settings.rules[element.name]) || {};
			}
			return rules;
		},
	
		normalizeRules: function(rules, element) {
			// handle dependency check
			$.each(rules, function(prop, val) {
				// ignore rule when param is explicitly false, eg. required:false
				if (val === false) {
					delete rules[prop];
					return;
				}
				if (val.param || val.depends) {
					var keepRule = true;
					switch (typeof val.depends) {
						case "string":
							keepRule = !!$(val.depends, element.form).length;
							break;
						case "function":
							keepRule = val.depends.call(element, element);
							break;
					}
					if (keepRule) {
						rules[prop] = val.param !== undefined ? val.param : true;
					} else {
						delete rules[prop];
					}
				}
			});
	
			// evaluate parameters
			$.each(rules, function(rule, parameter) {
				rules[rule] = $.isFunction(parameter) ? parameter(element) : parameter;
			});
	
			// clean number parameters
			$.each(['minlength', 'maxlength', 'min', 'max'], function() {
				if (rules[this]) {
					rules[this] = Number(rules[this]);
				}
			});
			$.each(['rangelength', 'range'], function() {
				if (rules[this]) {
					rules[this] = [Number(rules[this][0]), Number(rules[this][1])];
				}
			});
	
			if ($.validator.autoCreateRanges) {
				// auto-create ranges
				if (rules.min && rules.max) {
					rules.range = [rules.min, rules.max];
					delete rules.min;
					delete rules.max;
				}
				if (rules.minlength && rules.maxlength) {
					rules.rangelength = [rules.minlength, rules.maxlength];
					delete rules.minlength;
					delete rules.maxlength;
				}
			}
	
			// To support custom messages in metadata ignore rule methods titled "messages"
			if (rules.messages) {
				delete rules.messages;
			}
	
			return rules;
		},
	
		// Converts a simple string to a {string: true} rule, e.g., "required" to {required:true}
		normalizeRule: function(data) {
			if( typeof data == "string" ) {
				var transformed = {};
				$.each(data.split(/\s/), function() {
					transformed[this] = true;
				});
				data = transformed;
			}
			return data;
		},
	
		// http://docs.jquery.com/Plugins/Validation/Validator/addMethod
		addMethod: function(name, method, message) {
			$.validator.methods[name] = method;
			$.validator.messages[name] = message != undefined ? message : $.validator.messages[name];
			if (method.length < 3) {
				$.validator.addClassRules(name, $.validator.normalizeRule(name));
			}
		},
	
		methods: {
	
			// http://docs.jquery.com/Plugins/Validation/Methods/required
			required: function(value, element, param) {
				// check if dependency is met
				if ( !this.depend(param, element) )
					return "dependency-mismatch";
				switch( element.nodeName.toLowerCase() ) {
				case 'select':
					// could be an array for select-multiple or a string, both are fine this way
					var val = $(element).val();
					return val && val.length > 0;
				case 'input':
					if ( this.checkable(element) )
						return this.getLength(value, element) > 0;
				default:
					return $.trim(value).length > 0;
				}
			},
	
			// http://docs.jquery.com/Plugins/Validation/Methods/remote
			remote: function(value, element, param) {
				if ( this.optional(element) )
					return "dependency-mismatch";
	
				var previous = this.previousValue(element);
				if (!this.settings.messages[element.name] )
					this.settings.messages[element.name] = {};
				previous.originalMessage = this.settings.messages[element.name].remote;
				this.settings.messages[element.name].remote = previous.message;
	
				param = typeof param == "string" && {url:param} || param;
	
				if ( this.pending[element.name] ) {
					return "pending";
				}
				if ( previous.old === value ) {
					return previous.valid;
				}
	
				previous.old = value;
				var validator = this;
				this.startRequest(element);
				var data = {};
				data[element.name] = value;
				$.ajax($.extend(true, {
					url: param,
					mode: "abort",
					port: "validate" + element.name,
					dataType: "json",
					data: data,
					success: function(response) {
						validator.settings.messages[element.name].remote = previous.originalMessage;
						var valid = response === true;
						if ( valid ) {
							var submitted = validator.formSubmitted;
							validator.prepareElement(element);
							validator.formSubmitted = submitted;
							validator.successList.push(element);
							validator.showErrors();
						} else {
							var errors = {};
							var message = response || validator.defaultMessage( element, "remote" );
							errors[element.name] = previous.message = $.isFunction(message) ? message(value) : message;
							validator.showErrors(errors);
						}
						previous.valid = valid;
						validator.stopRequest(element, valid);
					}
				}, param));
				return "pending";
			},
	
			// http://docs.jquery.com/Plugins/Validation/Methods/minlength
			minlength: function(value, element, param) {
				return this.optional(element) || this.getLength($.trim(value), element) >= param;
			},
	
			// http://docs.jquery.com/Plugins/Validation/Methods/maxlength
			maxlength: function(value, element, param) {
				return this.optional(element) || this.getLength($.trim(value), element) <= param;
			},
	
			// http://docs.jquery.com/Plugins/Validation/Methods/rangelength
			rangelength: function(value, element, param) {
				var length = this.getLength($.trim(value), element);
				return this.optional(element) || ( length >= param[0] && length <= param[1] );
			},
	
			// http://docs.jquery.com/Plugins/Validation/Methods/min
			min: function( value, element, param ) {
				return this.optional(element) || value >= param;
			},
	
			// http://docs.jquery.com/Plugins/Validation/Methods/max
			max: function( value, element, param ) {
				return this.optional(element) || value <= param;
			},
	
			// http://docs.jquery.com/Plugins/Validation/Methods/range
			range: function( value, element, param ) {
				return this.optional(element) || ( value >= param[0] && value <= param[1] );
			},
	
			// http://docs.jquery.com/Plugins/Validation/Methods/email
			email: function(value, element) {
				// contributed by Scott Gonzalez: http://projects.scottsplayground.com/email_address_validation/
				return this.optional(element) || /^((([a-z]|\d|[!#\$%&'\*\+\-\/=\?\^_`{\|}~]|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])+(\.([a-z]|\d|[!#\$%&'\*\+\-\/=\?\^_`{\|}~]|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])+)*)|((\x22)((((\x20|\x09)*(\x0d\x0a))?(\x20|\x09)+)?(([\x01-\x08\x0b\x0c\x0e-\x1f\x7f]|\x21|[\x23-\x5b]|[\x5d-\x7e]|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])|(\\([\x01-\x09\x0b\x0c\x0d-\x7f]|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF]))))*(((\x20|\x09)*(\x0d\x0a))?(\x20|\x09)+)?(\x22)))@((([a-z]|\d|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])|(([a-z]|\d|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])([a-z]|\d|-|\.|_|~|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])*([a-z]|\d|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])))\.)+(([a-z]|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])|(([a-z]|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])([a-z]|\d|-|\.|_|~|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])*([a-z]|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])))$/i.test(value);
			},
	
			// http://docs.jquery.com/Plugins/Validation/Methods/url
			url: function(value, element) {
				// contributed by Scott Gonzalez: http://projects.scottsplayground.com/iri/
				return this.optional(element) || /^(https?|ftp):\/\/(((([a-z]|\d|-|\.|_|~|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])|(%[\da-f]{2})|[!\$&'\(\)\*\+,;=]|:)*@)?(((\d|[1-9]\d|1\d\d|2[0-4]\d|25[0-5])\.(\d|[1-9]\d|1\d\d|2[0-4]\d|25[0-5])\.(\d|[1-9]\d|1\d\d|2[0-4]\d|25[0-5])\.(\d|[1-9]\d|1\d\d|2[0-4]\d|25[0-5]))|((([a-z]|\d|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])|(([a-z]|\d|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])([a-z]|\d|-|\.|_|~|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])*([a-z]|\d|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])))\.)+(([a-z]|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])|(([a-z]|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])([a-z]|\d|-|\.|_|~|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])*([a-z]|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])))\.?)(:\d*)?)(\/((([a-z]|\d|-|\.|_|~|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])|(%[\da-f]{2})|[!\$&'\(\)\*\+,;=]|:|@)+(\/(([a-z]|\d|-|\.|_|~|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])|(%[\da-f]{2})|[!\$&'\(\)\*\+,;=]|:|@)*)*)?)?(\?((([a-z]|\d|-|\.|_|~|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])|(%[\da-f]{2})|[!\$&'\(\)\*\+,;=]|:|@)|[\uE000-\uF8FF]|\/|\?)*)?(\#((([a-z]|\d|-|\.|_|~|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])|(%[\da-f]{2})|[!\$&'\(\)\*\+,;=]|:|@)|\/|\?)*)?$/i.test(value);
			},
	
			// http://docs.jquery.com/Plugins/Validation/Methods/date
			date: function(value, element) {
				return this.optional(element) || !/Invalid|NaN/.test(new Date(value));
			},
	
			// http://docs.jquery.com/Plugins/Validation/Methods/dateISO
			dateISO: function(value, element) {
				return this.optional(element) || /^\d{4}[\/-]\d{1,2}[\/-]\d{1,2}$/.test(value);
			},
	
			// http://docs.jquery.com/Plugins/Validation/Methods/number
			number: function(value, element) {
				return this.optional(element) || /^-?(?:\d+|\d{1,3}(?:,\d{3})+)(?:\.\d+)?$/.test(value);
			},
	
			// http://docs.jquery.com/Plugins/Validation/Methods/digits
			digits: function(value, element) {
				return this.optional(element) || /^\d+$/.test(value);
			},
	
			// http://docs.jquery.com/Plugins/Validation/Methods/creditcard
			// based on http://en.wikipedia.org/wiki/Luhn
			creditcard: function(value, element) {
				if ( this.optional(element) )
					return "dependency-mismatch";
				// accept only spaces, digits and dashes
				if (/[^0-9 -]+/.test(value))
					return false;
				var nCheck = 0,
					nDigit = 0,
					bEven = false;
	
				value = value.replace(/\D/g, "");
	
				for (var n = value.length - 1; n >= 0; n--) {
					var cDigit = value.charAt(n);
					var nDigit = parseInt(cDigit, 10);
					if (bEven) {
						if ((nDigit *= 2) > 9)
							nDigit -= 9;
					}
					nCheck += nDigit;
					bEven = !bEven;
				}
	
				return (nCheck % 10) == 0;
			},
	
			// http://docs.jquery.com/Plugins/Validation/Methods/accept
			accept: function(value, element, param) {
				param = typeof param == "string" ? param.replace(/,/g, '|') : "png|jpe?g|gif";
				return this.optional(element) || value.match(new RegExp(".(" + param + ")$", "i"));
			},
	
			// http://docs.jquery.com/Plugins/Validation/Methods/equalTo
			equalTo: function(value, element, param) {
				// bind to the blur event of the target in order to revalidate whenever the target field is updated
				// TODO find a way to bind the event just once, avoiding the unbind-rebind overhead
				var target = $(param).unbind(".validate-equalTo").bind("blur.validate-equalTo", function() {
					$(element).valid();
				});
				return value == target.val();
			}
	
		}
	
	});
	
	// deprecated, use $.validator.format instead
	$.format = $.validator.format;
	
	})(jQuery);
	
	// ajax mode: abort
	// usage: $.ajax({ mode: "abort"[, port: "uniqueport"]});
	// if mode:"abort" is used, the previous request on that port (port can be undefined) is aborted via XMLHttpRequest.abort()
	;(function($) {
		var pendingRequests = {};
		// Use a prefilter if available (1.5+)
		if ( $.ajaxPrefilter ) {
			$.ajaxPrefilter(function(settings, _, xhr) {
				var port = settings.port;
				if (settings.mode == "abort") {
					if ( pendingRequests[port] ) {
						pendingRequests[port].abort();
					}
					pendingRequests[port] = xhr;
				}
			});
		} else {
			// Proxy ajax
			var ajax = $.ajax;
			$.ajax = function(settings) {
				var mode = ( "mode" in settings ? settings : $.ajaxSettings ).mode,
					port = ( "port" in settings ? settings : $.ajaxSettings ).port;
				if (mode == "abort") {
					if ( pendingRequests[port] ) {
						pendingRequests[port].abort();
					}
					return (pendingRequests[port] = ajax.apply(this, arguments));
				}
				return ajax.apply(this, arguments);
			};
		}
	})(jQuery);
	
	// provides cross-browser focusin and focusout events
	// IE has native support, in other browsers, use event caputuring (neither bubbles)
	
	// provides delegate(type: String, delegate: Selector, handler: Callback) plugin for easier event delegation
	// handler is only called when $(event.target).is(delegate), in the scope of the jquery-object for event.target
	;(function($) {
		// only implement if not provided by jQuery core (since 1.4)
		// TODO verify if jQuery 1.4's implementation is compatible with older jQuery special-event APIs
		if (!jQuery.event.special.focusin && !jQuery.event.special.focusout && document.addEventListener) {
			$.each({
				focus: 'focusin',
				blur: 'focusout'
			}, function( original, fix ){
				$.event.special[fix] = {
					setup:function() {
						this.addEventListener( original, handler, true );
					},
					teardown:function() {
						this.removeEventListener( original, handler, true );
					},
					handler: function(e) {
						arguments[0] = $.event.fix(e);
						arguments[0].type = fix;
						return $.event.handle.apply(this, arguments);
					}
				};
				function handler(e) {
					e = $.event.fix(e);
					e.type = fix;
					return $.event.handle.call(this, e);
				}
			});
		};
		$.extend($.fn, {
			validateDelegate: function(delegate, type, handler) {
				return this.bind(type, function(event) {
					var target = $(event.target);
					if (target.is(delegate)) {
						return handler.apply(target, arguments);
					}
				});
			}
		});
	})(jQuery);
	
	})(jQuery, jQuery);
	


/*******************
 * From src: synergy/src/js/20.synergy.address.js
 *******************/
/*! jQuery Address v1.6 | (c) 2009, 2013 Rostislav Hristov | jquery.org/license */
(function ($) {

    $.address = (function () {

        var _trigger = function(name) {
               var e = $.extend($.Event(name), (function() {
                    var parameters = {},
                        parameterNames = $.address.parameterNames();
                    for (var i = 0, l = parameterNames.length; i < l; i++) {
                        parameters[parameterNames[i]] = $.address.parameter(parameterNames[i]);
                    }
                    return {
                        value: $.address.value(),
                        path: $.address.path(),
                        pathNames: $.address.pathNames(),
                        parameterNames: parameterNames,
                        parameters: parameters,
                        queryString: $.address.queryString()
                    };
                }).call($.address));
                $($.address).trigger(e);
                return e;
            },
            _array = function(obj) {
                return Array.prototype.slice.call(obj);
            },
            _bind = function(value, data, fn) {
                $().bind.apply($($.address), Array.prototype.slice.call(arguments));
                return $.address;
            },
            _unbind = function(value,  fn) {
                $().unbind.apply($($.address), Array.prototype.slice.call(arguments));
                return $.address;
            },
            _supportsState = function() {
                return (_h.pushState && _opts.state !== UNDEFINED);
            },
            _hrefState = function() {
                return ('/' + _l.pathname.replace(new RegExp(_opts.state), '') + 
                    _l.search + (_hrefHash() ? '#' + _hrefHash() : '')).replace(_re, '/');
            },
            _hrefHash = function() {
                var index = _l.href.indexOf('#');
                return index != -1 ? _l.href.substr(index + 1) : '';
            },
            _href = function() {
                return _supportsState() ? _hrefState() : _hrefHash();
            },
            _window = function() {
                try {
                    return top.document !== UNDEFINED && top.document.title !== UNDEFINED ? top : window;
                } catch (e) { 
                    return window;
                }
            },
            _js = function() {
                return 'javascript';
            },
            _strict = function(value) {
                value = value.toString();
                return (_opts.strict && value.substr(0, 1) != '/' ? '/' : '') + value;
            },
            _cssint = function(el, value) {
                return parseInt(el.css(value), 10);
            },
            _listen = function() {
                if (!_silent) {
                    var hash = _href(),
                        diff = decodeURI(_value) != decodeURI(hash);
                    if (diff) {
                        if (_msie && _version < 7) {
                            _l.reload();
                        } else {
                            if (_msie && !_hashchange && _opts.history) {
                                _st(_html, 50);
                            }
                            _value = hash;
                            _update(FALSE);
                        }
                    }
                }
            },
            _update = function(internal) {
                _st(_track, 10);
                return _trigger(CHANGE).isDefaultPrevented() || 
                    _trigger(internal ? INTERNAL_CHANGE : EXTERNAL_CHANGE).isDefaultPrevented();
            },
            _track = function() {
                if (_opts.tracker !== 'null' && _opts.tracker !== NULL) {
                    var fn = $.isFunction(_opts.tracker) ? _opts.tracker : _t[_opts.tracker],
                        value = (_l.pathname + _l.search + 
                                ($.address && !_supportsState() ? $.address.value() : ''))
                                .replace(/\/\//, '/').replace(/^\/$/, '');
                    if ($.isFunction(fn)) {
                        fn(value);
                    } else if ($.isFunction(_t.urchinTracker)) {
                        _t.urchinTracker(value);
                    } else if (_t.pageTracker !== UNDEFINED && $.isFunction(_t.pageTracker._trackPageview)) {
                        _t.pageTracker._trackPageview(value);
                    } else if (_t._gaq !== UNDEFINED && $.isFunction(_t._gaq.push)) {
                        _t._gaq.push(['_trackPageview', decodeURI(value)]);
                    }
                }
            },
            _html = function() {
                var src = _js() + ':' + FALSE + ';document.open();document.writeln(\'<html><head><title>' + 
                    _d.title.replace(/\'/g, '\\\'') + '</title><script>var ' + ID + ' = "' + encodeURIComponent(_href()).replace(/\'/g, '\\\'') + 
                    (_d.domain != _l.hostname ? '";document.domain="' + _d.domain : '') + 
                    '";</' + 'script></head></html>\');document.close();';
                if (_version < 7) {
                    _frame.src = src;
                } else {
                    _frame.contentWindow.location.replace(src);
                }
            },
            _options = function() {
                if (_url && _qi != -1) {
                    var i, param, params = _url.substr(_qi + 1).split('&');
                    for (i = 0; i < params.length; i++) {
                        param = params[i].split('=');
                        if (/^(autoUpdate|history|strict|wrap)$/.test(param[0])) {
                            _opts[param[0]] = (isNaN(param[1]) ? /^(true|yes)$/i.test(param[1]) : (parseInt(param[1], 10) !== 0));
                        }
                        if (/^(state|tracker)$/.test(param[0])) {
                            _opts[param[0]] = param[1];
                        }
                    }
                    _url = NULL;
                }
                _value = _href();
            },
            _load = function() {
                if (!_loaded) {
                    _loaded = TRUE;
                    _options();
                    $('a[rel*="address:"]').address();
                    if (_opts.wrap) {
                        var body = $('body'),
                            wrap = $('body > *')
                                .wrapAll('<div style="padding:' + 
                                    (_cssint(body, 'marginTop') + _cssint(body, 'paddingTop')) + 'px ' + 
                                    (_cssint(body, 'marginRight') + _cssint(body, 'paddingRight')) + 'px ' + 
                                    (_cssint(body, 'marginBottom') + _cssint(body, 'paddingBottom')) + 'px ' + 
                                    (_cssint(body, 'marginLeft') + _cssint(body, 'paddingLeft')) + 'px;" />')
                                .parent()
                                .wrap('<div id="' + ID + '" style="height:100%;overflow:auto;position:relative;' + 
                                    (_webkit && !window.statusbar.visible ? 'resize:both;' : '') + '" />');
                        $('html, body')
                            .css({
                                height: '100%',
                                margin: 0,
                                padding: 0,
                                overflow: 'hidden'
                            });
                        if (_webkit) {
                            $('<style type="text/css" />')
                                .appendTo('head')
                                .text('#' + ID + '::-webkit-resizer { background-color: #fff; }');
                        }
                    }
                    if (_msie && !_hashchange) {
                        var frameset = _d.getElementsByTagName('frameset')[0];
                        _frame = _d.createElement((frameset ? '' : 'i') + 'frame');
                        _frame.src = _js() + ':' + FALSE;
                        if (frameset) {
                            frameset.insertAdjacentElement('beforeEnd', _frame);
                            frameset[frameset.cols ? 'cols' : 'rows'] += ',0';
                            _frame.noResize = TRUE;
                            _frame.frameBorder = _frame.frameSpacing = 0;
                        } else {
                            _frame.style.display = 'none';
                            _frame.style.width = _frame.style.height = 0;
                            _frame.tabIndex = -1;
                            _d.body.insertAdjacentElement('afterBegin', _frame);
                        }
                        _st(function() {
                            $(_frame).bind('load', function() {
                                var win = _frame.contentWindow;
                                _value = win[ID] !== UNDEFINED ? win[ID] : '';
                                if (_value != _href()) {
                                    _update(FALSE);
                                    _l.hash = _value;
                                }
                            });
                            if (_frame.contentWindow[ID] === UNDEFINED) {
                                _html();
                            }
                        }, 50);
                    }
                    _st(function() {
                        _trigger('init');
                        _update(FALSE);
                    }, 1);
                    if (!_supportsState()) {
                        if ((_msie && _version > 7) || (!_msie && _hashchange)) {
                            if (_t.addEventListener) {
                                _t.addEventListener(HASH_CHANGE, _listen, FALSE);
                            } else if (_t.attachEvent) {
                                _t.attachEvent('on' + HASH_CHANGE, _listen);
                            }
                        } else {
                            _si(_listen, 50);
                        }
                    }
                    if ('state' in window.history) {
                        $(window).trigger('popstate');
                    }
                }
            },
            _popstate = function() {
                if (decodeURI(_value) != decodeURI(_href())) {
                    _value = _href();
                    _update(FALSE);
                }
            },
            _unload = function() {
                if (_t.removeEventListener) {
                    _t.removeEventListener(HASH_CHANGE, _listen, FALSE);
                } else if (_t.detachEvent) {
                    _t.detachEvent('on' + HASH_CHANGE, _listen);
                }
            },
            _uaMatch = function(ua) {
                ua = ua.toLowerCase();
                var match = /(chrome)[ \/]([\w.]+)/.exec( ua ) ||
                    /(webkit)[ \/]([\w.]+)/.exec( ua ) ||
                    /(opera)(?:.*version|)[ \/]([\w.]+)/.exec( ua ) ||
                    /(msie) ([\w.]+)/.exec( ua ) ||
                    ua.indexOf('compatible') < 0 && /(mozilla)(?:.*? rv:([\w.]+)|)/.exec( ua ) ||
                    [];
                return {
                    browser: match[ 1 ] || '',
                    version: match[ 2 ] || '0'
                };
            },
            _detectBrowser = function() {
                var browser = {},
                    matched = _uaMatch(navigator.userAgent);
                if (matched.browser) {
                    browser[matched.browser] = true;
                    browser.version = matched.version;
                }
                if (browser.chrome) {
                    browser.webkit = true;
                } else if (browser.webkit) {
                    browser.safari = true;
                }
                return browser;
            },
            UNDEFINED,
            NULL = null,
            ID = 'jQueryAddress',
            STRING = 'string',
            HASH_CHANGE = 'hashchange',
            INIT = 'init',
            CHANGE = 'change',
            INTERNAL_CHANGE = 'internalChange',
            EXTERNAL_CHANGE = 'externalChange',
            TRUE = true,
            FALSE = false,
            _opts = {
                autoUpdate: TRUE, 
                history: TRUE, 
                strict: TRUE,
                wrap: FALSE
            },
            _browser = _detectBrowser(),
            _version = parseFloat(_browser.version),
            _webkit = _browser.webkit || _browser.safari,
            _msie = !$.support.opacity,
            _t = _window(),
            _d = _t.document,
            _h = _t.history, 
            _l = _t.location,
            _si = setInterval,
            _st = setTimeout,
            _re = /\/{2,9}/g,
            _agent = navigator.userAgent,
            _hashchange = 'on' + HASH_CHANGE in _t,
            _frame,
            _form,
            _url = $('script:last').attr('src'),
            _qi = _url ? _url.indexOf('?') : -1,
            _title = _d.title, 
            _silent = FALSE,
            _loaded = FALSE,
            _juststart = TRUE,
            _updating = FALSE,
            _listeners = {}, 
            _value = _href();
            
        if (_msie) {
            _version = parseFloat(_agent.substr(_agent.indexOf('MSIE') + 4));
            if (_d.documentMode && _d.documentMode != _version) {
                _version = _d.documentMode != 8 ? 7 : 8;
            }
            var pc = _d.onpropertychange;
            _d.onpropertychange = function() {
                if (pc) {
                    pc.call(_d);
                }
                if (_d.title != _title && _d.title.indexOf('#' + _href()) != -1) {
                    _d.title = _title;
                }
            };
        }
        
        if (_h.navigationMode) {
            _h.navigationMode = 'compatible';
        }
        if (document.readyState == 'complete') {
            var interval = setInterval(function() {
                if ($.address) {
                    _load();
                    clearInterval(interval);
                }
            }, 50);
        } else {
            _options();
            $(_load);
        }
        $(window).bind('popstate', _popstate).bind('unload', _unload);

        return {
            bind: function(type, data, fn) {
                return _bind.apply(this, _array(arguments));
            },
            unbind: function(type, fn) {
                return _unbind.apply(this, _array(arguments));
            },
            init: function(data, fn) {
                return _bind.apply(this, [INIT].concat(_array(arguments)));
            },
            change: function(data, fn) {
                return _bind.apply(this, [CHANGE].concat(_array(arguments)));
            },
            internalChange: function(data, fn) {
                return _bind.apply(this, [INTERNAL_CHANGE].concat(_array(arguments)));
            },
            externalChange: function(data, fn) {
                return _bind.apply(this, [EXTERNAL_CHANGE].concat(_array(arguments)));
            },
            baseURL: function() {
                var url = _l.href;
                if (url.indexOf('#') != -1) {
                    url = url.substr(0, url.indexOf('#'));
                }
                if (/\/$/.test(url)) {
                    url = url.substr(0, url.length - 1);
                }
                return url;
            },
            autoUpdate: function(value) {
                if (value !== UNDEFINED) {
                    _opts.autoUpdate = value;
                    return this;
                }
                return _opts.autoUpdate;
            },
            history: function(value) {
                if (value !== UNDEFINED) {
                    _opts.history = value;
                    return this;
                }
                return _opts.history;
            },
            state: function(value) {
                if (value !== UNDEFINED) {
                    _opts.state = value;
                    var hrefState = _hrefState();
                    if (_opts.state !== UNDEFINED) {
                        if (_h.pushState) {
                            if (hrefState.substr(0, 3) == '/#/') {
                                _l.replace(_opts.state.replace(/^\/$/, '') + hrefState.substr(2));
                            }
                        } else if (hrefState != '/' && hrefState.replace(/^\/#/, '') != _hrefHash()) {
                            _st(function() {
                                _l.replace(_opts.state.replace(/^\/$/, '') + '/#' + hrefState);
                            }, 1);
                        }
                    }
                    return this;
                }
                return _opts.state;
            },
            strict: function(value) {
                if (value !== UNDEFINED) {
                    _opts.strict = value;
                    return this;
                }
                return _opts.strict;
            },
            tracker: function(value) {
                if (value !== UNDEFINED) {
                    _opts.tracker = value;
                    return this;
                }
                return _opts.tracker;
            },
            wrap: function(value) {
                if (value !== UNDEFINED) {
                    _opts.wrap = value;
                    return this;
                }
                return _opts.wrap;
            },
            update: function() {
                _updating = TRUE;
                this.value(_value);
                _updating = FALSE;
                return this;
            },
            title: function(value) {
                if (value !== UNDEFINED) {
                    _st(function() {
                        _title = _d.title = value;
                        if (_juststart && _frame && _frame.contentWindow && _frame.contentWindow.document) {
                            _frame.contentWindow.document.title = value;
                            _juststart = FALSE;
                        }
                    }, 50);
                    return this;
                }
                return _d.title;
            },
            value: function(value) {
                if (value !== UNDEFINED) {
                    value = _strict(value);
                    if (value == '/') {
                        value = '';
                    }
                    if (_value == value && !_updating) {
                        return;
                    }
                    _value = value;
                    if (_opts.autoUpdate || _updating) {
                        if (_update(TRUE)) {
                            return this;
                        }
                        if (_supportsState()) {
                            _h[_opts.history ? 'pushState' : 'replaceState']({}, '', 
                                    _opts.state.replace(/\/$/, '') + (_value === '' ? '/' : _value));
                        } else {
                            _silent = TRUE;
                            if (_webkit) {
                                if (_opts.history) {
                                    _l.hash = '#' + _value;
                                } else {
                                    _l.replace('#' + _value);
                                }
                            } else if (_value != _href()) {
                                if (_opts.history) {
                                    _l.hash = '#' + _value;
                                } else {
                                    _l.replace('#' + _value);
                                }
                            }
                            if ((_msie && !_hashchange) && _opts.history) {
                                _st(_html, 50);
                            }
                            if (_webkit) {
                                _st(function(){ _silent = FALSE; }, 1);
                            } else {
                                _silent = FALSE;
                            }
                        }
                    }
                    return this;
                }
                return _strict(_value);
            },
            path: function(value) {
                if (value !== UNDEFINED) {
                    var qs = this.queryString(),
                        hash = this.hash();
                    this.value(value + (qs ? '?' + qs : '') + (hash ? '#' + hash : ''));
                    return this;
                }
                return _strict(_value).split('#')[0].split('?')[0];
            },
            pathNames: function() {
                var path = this.path(),
                    names = path.replace(_re, '/').split('/');
                if (path.substr(0, 1) == '/' || path.length === 0) {
                    names.splice(0, 1);
                }
                if (path.substr(path.length - 1, 1) == '/') {
                    names.splice(names.length - 1, 1);
                }
                return names;
            },
            queryString: function(value) {
                if (value !== UNDEFINED) {
                    var hash = this.hash();
                    this.value(this.path() + (value ? '?' + value : '') + (hash ? '#' + hash : ''));
                    return this;
                }
                var arr = _value.split('?');
                return arr.slice(1, arr.length).join('?').split('#')[0];
            },
            parameter: function(name, value, append) {
                var i, params;
                if (value !== UNDEFINED) {
                    var names = this.parameterNames();
                    params = [];
                    value = value === UNDEFINED || value === NULL ? '' : value.toString();
                    for (i = 0; i < names.length; i++) {
                        var n = names[i],
                            v = this.parameter(n);
                        if (typeof v == STRING) {
                            v = [v];
                        }
                        if (n == name) {
                            v = (value === NULL || value === '') ? [] : 
                                (append ? v.concat([value]) : [value]);
                        }
                        for (var j = 0; j < v.length; j++) {
                            params.push(n + '=' + v[j]);
                        }
                    }
                    if ($.inArray(name, names) == -1 && value !== NULL && value !== '') {
                        params.push(name + '=' + value);
                    }
                    this.queryString(params.join('&'));
                    return this;
                }
                value = this.queryString();
                if (value) {
                    var r = [];
                    params = value.split('&');
                    for (i = 0; i < params.length; i++) {
                        var p = params[i].split('=');
                        if (p[0] == name) {
                            r.push(p.slice(1).join('='));
                        }
                    }
                    if (r.length !== 0) {
                        return r.length != 1 ? r : r[0];
                    }
                }
            },
            parameterNames: function() {
                var qs = this.queryString(),
                    names = [];
                if (qs && qs.indexOf('=') != -1) {
                    var params = qs.split('&');
                    for (var i = 0; i < params.length; i++) {
                        var name = params[i].split('=')[0];
                        if ($.inArray(name, names) == -1) {
                            names.push(name);
                        }
                    }
                }
                return names;
            },
            hash: function(value) {
                if (value !== UNDEFINED) {
                    this.value(_value.split('#')[0] + (value ? '#' + value : ''));
                    return this;
                }
                var arr = _value.split('#');
                return arr.slice(1, arr.length).join('#');                
            }
        };
    })();
    
    $.fn.address = function(fn) {
        if (!this.data('address')) {
            this.on('click', function(e) {
                if (e.shiftKey || e.ctrlKey || e.metaKey || e.which == 2) {
                    return true;
                }
                var target = e.currentTarget;
                if ($(target).is('a')) {
                    e.preventDefault();
                    var value = fn ? fn.call(target) : 
                        /address:/.test($(target).attr('rel')) ? $(target).attr('rel').split('address:')[1].split(' ')[0] : 
                        $.address.state() !== undefined && !/^\/?$/.test($.address.state()) ? 
                                $(target).attr('href').replace(new RegExp('^(.*' + $.address.state() + '|\\.)'), '') : 
                                $(target).attr('href').replace(/^(#\!?|\.)/, '');
                    $.address.value(value);
                }
            }).on('submit', function(e) {
                var target = e.currentTarget;
                if ($(target).is('form')) {
                    e.preventDefault();
                    var action = $(target).attr('action'),
                        value = fn ? fn.call(target) : (action.indexOf('?') != -1 ? action.replace(/&$/, '') : action + '?') + 
                            $(target).serialize();
                    $.address.value(value);
                }
            }).data('address', true);
        }
        return this;
    };
    
})(jQuery);



/*******************
 * From src: synergy/src/js/20.synergy.colorbox.js
 *******************/
/*!
	Colorbox 1.6.4
	license: MIT
	http://www.jacklmoore.com/colorbox
*/
(function ($, document, window) {
	var
	// Default settings object.
	// See http://jacklmoore.com/colorbox for details.
	defaults = {
		// data sources
		html: false,
		photo: false,
		iframe: false,
		inline: false,

		// behavior and appearance
		transition: "elastic",
		speed: 300,
		fadeOut: 300,
		width: false,
		initialWidth: "600",
		innerWidth: false,
		maxWidth: false,
		height: false,
		initialHeight: "450",
		innerHeight: false,
		maxHeight: false,
		scalePhotos: true,
		scrolling: true,
		opacity: 0.9,
		preloading: true,
		className: false,
		overlayClose: true,
		escKey: true,
		arrowKey: true,
		top: false,
		bottom: false,
		left: false,
		right: false,
		fixed: false,
		data: undefined,
		closeButton: true,
		fastIframe: true,
		open: false,
		reposition: true,
		loop: true,
		slideshow: false,
		slideshowAuto: true,
		slideshowSpeed: 2500,
		slideshowStart: "start slideshow",
		slideshowStop: "stop slideshow",
		photoRegex: /\.(gif|png|jp(e|g|eg)|bmp|ico|webp|jxr|svg)((#|\?).*)?$/i,

		// alternate image paths for high-res displays
		retinaImage: false,
		retinaUrl: false,
		retinaSuffix: '@2x.$1',

		// internationalization
		current: "image {current} of {total}",
		previous: "previous",
		next: "next",
		close: "close",
		xhrError: "This content failed to load.",
		imgError: "This image failed to load.",

		// accessbility
		returnFocus: true,
		trapFocus: true,

		// callbacks
		onOpen: false,
		onLoad: false,
		onComplete: false,
		onCleanup: false,
		onClosed: false,

		rel: function() {
			return this.rel;
		},
		href: function() {
			// using this.href would give the absolute url, when the href may have been intended as a selector (e.g. '#container')
			return $(this).attr('href');
		},
		title: function() {
			return this.title;
		},
		createImg: function() {
			var img = new Image();
			var attrs = $(this).data('cbox-img-attrs');

			if (typeof attrs === 'object') {
				$.each(attrs, function(key, val){
					img[key] = val;
				});
			}

			return img;
		},
		createIframe: function() {
			var iframe = document.createElement('iframe');
			var attrs = $(this).data('cbox-iframe-attrs');

			if (typeof attrs === 'object') {
				$.each(attrs, function(key, val){
					iframe[key] = val;
				});
			}

			if ('frameBorder' in iframe) {
				iframe.frameBorder = 0;
			}
			if ('allowTransparency' in iframe) {
				iframe.allowTransparency = "true";
			}
			iframe.name = (new Date()).getTime(); // give the iframe a unique name to prevent caching
			iframe.allowFullscreen = true;

			return iframe;
		}
	},

	// Abstracting the HTML and event identifiers for easy rebranding
	colorbox = 'colorbox',
	prefix = 'cbox',
	boxElement = prefix + 'Element',

	// Events
	event_open = prefix + '_open',
	event_load = prefix + '_load',
	event_complete = prefix + '_complete',
	event_cleanup = prefix + '_cleanup',
	event_closed = prefix + '_closed',
	event_purge = prefix + '_purge',

	// Cached jQuery Object Variables
	$overlay,
	$box,
	$wrap,
	$content,
	$topBorder,
	$leftBorder,
	$rightBorder,
	$bottomBorder,
	$related,
	$window,
	$loaded,
	$loadingBay,
	$loadingOverlay,
	$title,
	$current,
	$slideshow,
	$next,
	$prev,
	$close,
	$groupControls,
	$events = $('<a/>'), // $({}) would be preferred, but there is an issue with jQuery 1.4.2

	// Variables for cached values or use across multiple functions
	settings,
	interfaceHeight,
	interfaceWidth,
	loadedHeight,
	loadedWidth,
	index,
	photo,
	open,
	active,
	closing,
	loadingTimer,
	publicMethod,
	div = "div",
	requests = 0,
	previousCSS = {},
	init;

	// ****************
	// HELPER FUNCTIONS
	// ****************

	// Convenience function for creating new jQuery objects
	function $tag(tag, id, css) {
		var element = document.createElement(tag);

		if (id) {
			element.id = prefix + id;
		}

		if (css) {
			element.style.cssText = css;
		}

		return $(element);
	}

	// Get the window height using innerHeight when available to avoid an issue with iOS
	// http://bugs.jquery.com/ticket/6724
	function winheight() {
		return window.innerHeight ? window.innerHeight : $(window).height();
	}

	function Settings(element, options) {
		if (options !== Object(options)) {
			options = {};
		}

		this.cache = {};
		this.el = element;

		this.value = function(key) {
			var dataAttr;

			if (this.cache[key] === undefined) {
				dataAttr = $(this.el).attr('data-cbox-'+key);

				if (dataAttr !== undefined) {
					this.cache[key] = dataAttr;
				} else if (options[key] !== undefined) {
					this.cache[key] = options[key];
				} else if (defaults[key] !== undefined) {
					this.cache[key] = defaults[key];
				}
			}

			return this.cache[key];
		};

		this.get = function(key) {
			var value = this.value(key);
			return $.isFunction(value) ? value.call(this.el, this) : value;
		};
	}

	// Determine the next and previous members in a group.
	function getIndex(increment) {
		var
		max = $related.length,
		newIndex = (index + increment) % max;

		return (newIndex < 0) ? max + newIndex : newIndex;
	}

	// Convert '%' and 'px' values to integers
	function setSize(size, dimension) {
		return Math.round((/%/.test(size) ? ((dimension === 'x' ? $window.width() : winheight()) / 100) : 1) * parseInt(size, 10));
	}

	// Checks an href to see if it is a photo.
	// There is a force photo option (photo: true) for hrefs that cannot be matched by the regex.
	function isImage(settings, url) {
		return settings.get('photo') || settings.get('photoRegex').test(url);
	}

	function retinaUrl(settings, url) {
		return settings.get('retinaUrl') && window.devicePixelRatio > 1 ? url.replace(settings.get('photoRegex'), settings.get('retinaSuffix')) : url;
	}

	function trapFocus(e) {
		if ('contains' in $box[0] && !$box[0].contains(e.target) && e.target !== $overlay[0]) {
			e.stopPropagation();
			$box.focus();
		}
	}

	function setClass(str) {
		if (setClass.str !== str) {
			$box.add($overlay).removeClass(setClass.str).addClass(str);
			setClass.str = str;
		}
	}

	function getRelated(rel) {
		index = 0;

		if (rel && rel !== false && rel !== 'nofollow') {
			$related = $('.' + boxElement).filter(function () {
				var options = $.data(this, colorbox);
				var settings = new Settings(this, options);
				return (settings.get('rel') === rel);
			});
			index = $related.index(settings.el);

			// Check direct calls to Colorbox.
			if (index === -1) {
				$related = $related.add(settings.el);
				index = $related.length - 1;
			}
		} else {
			$related = $(settings.el);
		}
	}

	function trigger(event) {
		// for external use
		$(document).trigger(event);
		// for internal use
		$events.triggerHandler(event);
	}

	var slideshow = (function(){
		var active,
			className = prefix + "Slideshow_",
			click = "click." + prefix,
			timeOut;

		function clear () {
			clearTimeout(timeOut);
		}

		function set() {
			if (settings.get('loop') || $related[index + 1]) {
				clear();
				timeOut = setTimeout(publicMethod.next, settings.get('slideshowSpeed'));
			}
		}

		function start() {
			$slideshow
				.html(settings.get('slideshowStop'))
				.unbind(click)
				.one(click, stop);

			$events
				.bind(event_complete, set)
				.bind(event_load, clear);

			$box.removeClass(className + "off").addClass(className + "on");
		}

		function stop() {
			clear();

			$events
				.unbind(event_complete, set)
				.unbind(event_load, clear);

			$slideshow
				.html(settings.get('slideshowStart'))
				.unbind(click)
				.one(click, function () {
					publicMethod.next();
					start();
				});

			$box.removeClass(className + "on").addClass(className + "off");
		}

		function reset() {
			active = false;
			$slideshow.hide();
			clear();
			$events
				.unbind(event_complete, set)
				.unbind(event_load, clear);
			$box.removeClass(className + "off " + className + "on");
		}

		return function(){
			if (active) {
				if (!settings.get('slideshow')) {
					$events.unbind(event_cleanup, reset);
					reset();
				}
			} else {
				if (settings.get('slideshow') && $related[1]) {
					active = true;
					$events.one(event_cleanup, reset);
					if (settings.get('slideshowAuto')) {
						start();
					} else {
						stop();
					}
					$slideshow.show();
				}
			}
		};

	}());


	function launch(element) {
		var options;

		if (!closing) {

			options = $(element).data(colorbox);

			settings = new Settings(element, options);

			getRelated(settings.get('rel'));

			if (!open) {
				open = active = true; // Prevents the page-change action from queuing up if the visitor holds down the left or right keys.

				setClass(settings.get('className'));

				// Show colorbox so the sizes can be calculated in older versions of jQuery
				$box.css({visibility:'hidden', display:'block', opacity:''});

				$loaded = $tag(div, 'LoadedContent', 'width:0; height:0; overflow:hidden; visibility:hidden');
				$content.css({width:'', height:''}).append($loaded);

				// Cache values needed for size calculations
				interfaceHeight = $topBorder.height() + $bottomBorder.height() + $content.outerHeight(true) - $content.height();
				interfaceWidth = $leftBorder.width() + $rightBorder.width() + $content.outerWidth(true) - $content.width();
				loadedHeight = $loaded.outerHeight(true);
				loadedWidth = $loaded.outerWidth(true);

				// Opens inital empty Colorbox prior to content being loaded.
				var initialWidth = setSize(settings.get('initialWidth'), 'x');
				var initialHeight = setSize(settings.get('initialHeight'), 'y');
				var maxWidth = settings.get('maxWidth');
				var maxHeight = settings.get('maxHeight');

				settings.w = Math.max((maxWidth !== false ? Math.min(initialWidth, setSize(maxWidth, 'x')) : initialWidth) - loadedWidth - interfaceWidth, 0);
				settings.h = Math.max((maxHeight !== false ? Math.min(initialHeight, setSize(maxHeight, 'y')) : initialHeight) - loadedHeight - interfaceHeight, 0);

				$loaded.css({width:'', height:settings.h});
				publicMethod.position();

				trigger(event_open);
				settings.get('onOpen');

				$groupControls.add($title).hide();

				$box.focus();

				if (settings.get('trapFocus')) {
					// Confine focus to the modal
					// Uses event capturing that is not supported in IE8-
					if (document.addEventListener) {

						document.addEventListener('focus', trapFocus, true);

						$events.one(event_closed, function () {
							document.removeEventListener('focus', trapFocus, true);
						});
					}
				}

				// Return focus on closing
				if (settings.get('returnFocus')) {
					$events.one(event_closed, function () {
						$(settings.el).focus();
					});
				}
			}

			var opacity = parseFloat(settings.get('opacity'));
			$overlay.css({
				opacity: opacity === opacity ? opacity : '',
				cursor: settings.get('overlayClose') ? 'pointer' : '',
				visibility: 'visible'
			}).show();

			if (settings.get('closeButton')) {
				$close.html(settings.get('close')).appendTo($content);
			} else {
				$close.appendTo('<div/>'); // replace with .detach() when dropping jQuery < 1.4
			}

			load();
		}
	}

	// Colorbox's markup needs to be added to the DOM prior to being called
	// so that the browser will go ahead and load the CSS background images.
	function appendHTML() {
		if (!$box) {
			init = false;
			$window = $(window);
			$box = $tag(div).attr({
				id: colorbox,
				'class': $.support.opacity === false ? prefix + 'IE' : '', // class for optional IE8 & lower targeted CSS.
				role: 'dialog',
				tabindex: '-1'
			}).hide();
			$overlay = $tag(div, "Overlay").hide();
			$loadingOverlay = $([$tag(div, "LoadingOverlay")[0],$tag(div, "LoadingGraphic")[0]]);
			$wrap = $tag(div, "Wrapper");
			$content = $tag(div, "Content").append(
				$title = $tag(div, "Title"),
				$current = $tag(div, "Current"),
				$prev = $('<button type="button"/>').attr({id:prefix+'Previous'}),
				$next = $('<button type="button"/>').attr({id:prefix+'Next'}),
				$slideshow = $('<button type="button"/>').attr({id:prefix+'Slideshow'}),
				$loadingOverlay
			);

			$close = $('<button type="button"/>').attr({id:prefix+'Close'});

			$wrap.append( // The 3x3 Grid that makes up Colorbox
				$tag(div).append(
					$tag(div, "TopLeft"),
					$topBorder = $tag(div, "TopCenter"),
					$tag(div, "TopRight")
				),
				$tag(div, false, 'clear:left').append(
					$leftBorder = $tag(div, "MiddleLeft"),
					$content,
					$rightBorder = $tag(div, "MiddleRight")
				),
				$tag(div, false, 'clear:left').append(
					$tag(div, "BottomLeft"),
					$bottomBorder = $tag(div, "BottomCenter"),
					$tag(div, "BottomRight")
				)
			).find('div div').css({'float': 'left'});

			$loadingBay = $tag(div, false, 'position:absolute; width:9999px; visibility:hidden; display:none; max-width:none;');

			$groupControls = $next.add($prev).add($current).add($slideshow);
		}
		if (document.body && !$box.parent().length) {
			$(document.body).append($overlay, $box.append($wrap, $loadingBay));
		}
	}

	// Add Colorbox's event bindings
	function addBindings() {
		function clickHandler(e) {
			// ignore non-left-mouse-clicks and clicks modified with ctrl / command, shift, or alt.
			// See: http://jacklmoore.com/notes/click-events/
			if (!(e.which > 1 || e.shiftKey || e.altKey || e.metaKey || e.ctrlKey)) {
				e.preventDefault();
				launch(this);
			}
		}

		if ($box) {
			if (!init) {
				init = true;

				// Anonymous functions here keep the public method from being cached, thereby allowing them to be redefined on the fly.
				$next.click(function () {
					publicMethod.next();
				});
				$prev.click(function () {
					publicMethod.prev();
				});
				$close.click(function () {
					publicMethod.close();
				});
				$overlay.click(function () {
					if (settings.get('overlayClose')) {
						publicMethod.close();
					}
				});

				// Key Bindings
				$(document).bind('keydown.' + prefix, function (e) {
					var key = e.keyCode;
					if (open && settings.get('escKey') && key === 27) {
						e.preventDefault();
						publicMethod.close();
					}
					if (open && settings.get('arrowKey') && $related[1] && !e.altKey) {
						if (key === 37) {
							e.preventDefault();
							$prev.click();
						} else if (key === 39) {
							e.preventDefault();
							$next.click();
						}
					}
				});

				if ($.isFunction($.fn.on)) {
					// For jQuery 1.7+
					$(document).on('click.'+prefix, '.'+boxElement, clickHandler);
				} else {
					// For jQuery 1.3.x -> 1.6.x
					// This code is never reached in jQuery 1.9, so do not contact me about 'live' being removed.
					// This is not here for jQuery 1.9, it's here for legacy users.
					$('.'+boxElement).live('click.'+prefix, clickHandler);
				}
			}
			return true;
		}
		return false;
	}

	// Don't do anything if Colorbox already exists.
	if ($[colorbox]) {
		return;
	}

	// Append the HTML when the DOM loads
	$(appendHTML);


	// ****************
	// PUBLIC FUNCTIONS
	// Usage format: $.colorbox.close();
	// Usage from within an iframe: parent.jQuery.colorbox.close();
	// ****************

	publicMethod = $.fn[colorbox] = $[colorbox] = function (options, callback) {
		var settings;
		var $obj = this;

		options = options || {};

		if ($.isFunction($obj)) { // assume a call to $.colorbox
			$obj = $('<a/>');
			options.open = true;
		}

		if (!$obj[0]) { // colorbox being applied to empty collection
			return $obj;
		}

		appendHTML();

		if (addBindings()) {

			if (callback) {
				options.onComplete = callback;
			}

			$obj.each(function () {
				var old = $.data(this, colorbox) || {};
				$.data(this, colorbox, $.extend(old, options));
			}).addClass(boxElement);

			settings = new Settings($obj[0], options);

			if (settings.get('open')) {
				launch($obj[0]);
			}
		}

		return $obj;
	};

	publicMethod.position = function (speed, loadedCallback) {
		var
		css,
		top = 0,
		left = 0,
		offset = $box.offset(),
		scrollTop,
		scrollLeft;

		$window.unbind('resize.' + prefix);

		// remove the modal so that it doesn't influence the document width/height
		$box.css({top: -9e4, left: -9e4});

		scrollTop = $window.scrollTop();
		scrollLeft = $window.scrollLeft();

		if (settings.get('fixed')) {
			offset.top -= scrollTop;
			offset.left -= scrollLeft;
			$box.css({position: 'fixed'});
		} else {
			top = scrollTop;
			left = scrollLeft;
			$box.css({position: 'absolute'});
		}

		// keeps the top and left positions within the browser's viewport.
		if (settings.get('right') !== false) {
			left += Math.max($window.width() - settings.w - loadedWidth - interfaceWidth - setSize(settings.get('right'), 'x'), 0);
		} else if (settings.get('left') !== false) {
			left += setSize(settings.get('left'), 'x');
		} else {
			left += Math.round(Math.max($window.width() - settings.w - loadedWidth - interfaceWidth, 0) / 2);
		}

		if (settings.get('bottom') !== false) {
			top += Math.max(winheight() - settings.h - loadedHeight - interfaceHeight - setSize(settings.get('bottom'), 'y'), 0);
		} else if (settings.get('top') !== false) {
			top += setSize(settings.get('top'), 'y');
		} else {
			top += Math.round(Math.max(winheight() - settings.h - loadedHeight - interfaceHeight, 0) / 2);
		}

		$box.css({top: offset.top, left: offset.left, visibility:'visible'});

		// this gives the wrapper plenty of breathing room so it's floated contents can move around smoothly,
		// but it has to be shrank down around the size of div#colorbox when it's done.  If not,
		// it can invoke an obscure IE bug when using iframes.
		$wrap[0].style.width = $wrap[0].style.height = "9999px";

		function modalDimensions() {
			$topBorder[0].style.width = $bottomBorder[0].style.width = $content[0].style.width = (parseInt($box[0].style.width,10) - interfaceWidth)+'px';
			$content[0].style.height = $leftBorder[0].style.height = $rightBorder[0].style.height = (parseInt($box[0].style.height,10) - interfaceHeight)+'px';
		}

		css = {width: settings.w + loadedWidth + interfaceWidth, height: settings.h + loadedHeight + interfaceHeight, top: top, left: left};

		// setting the speed to 0 if the content hasn't changed size or position
		if (speed) {
			var tempSpeed = 0;
			$.each(css, function(i){
				if (css[i] !== previousCSS[i]) {
					tempSpeed = speed;
					return;
				}
			});
			speed = tempSpeed;
		}

		previousCSS = css;

		if (!speed) {
			$box.css(css);
		}

		$box.dequeue().animate(css, {
			duration: speed || 0,
			complete: function () {
				modalDimensions();

				active = false;

				// shrink the wrapper down to exactly the size of colorbox to avoid a bug in IE's iframe implementation.
				$wrap[0].style.width = (settings.w + loadedWidth + interfaceWidth) + "px";
				$wrap[0].style.height = (settings.h + loadedHeight + interfaceHeight) + "px";

				if (settings.get('reposition')) {
					setTimeout(function () {  // small delay before binding onresize due to an IE8 bug.
						$window.bind('resize.' + prefix, publicMethod.position);
					}, 1);
				}

				if ($.isFunction(loadedCallback)) {
					loadedCallback();
				}
			},
			step: modalDimensions
		});
	};

	publicMethod.resize = function (options) {
		var scrolltop;

		if (open) {
			options = options || {};

			if (options.width) {
				settings.w = setSize(options.width, 'x') - loadedWidth - interfaceWidth;
			}

			if (options.innerWidth) {
				settings.w = setSize(options.innerWidth, 'x');
			}

			$loaded.css({width: settings.w});

			if (options.height) {
				settings.h = setSize(options.height, 'y') - loadedHeight - interfaceHeight;
			}

			if (options.innerHeight) {
				settings.h = setSize(options.innerHeight, 'y');
			}

			if (!options.innerHeight && !options.height) {
				scrolltop = $loaded.scrollTop();
				$loaded.css({height: "auto"});
				settings.h = $loaded.height();
			}

			$loaded.css({height: settings.h});

			if(scrolltop) {
				$loaded.scrollTop(scrolltop);
			}

			publicMethod.position(settings.get('transition') === "none" ? 0 : settings.get('speed'));
		}
	};

	publicMethod.prep = function (object) {
		if (!open) {
			return;
		}

		var callback, speed = settings.get('transition') === "none" ? 0 : settings.get('speed');

		$loaded.remove();

		$loaded = $tag(div, 'LoadedContent').append(object);

		function getWidth() {
			settings.w = settings.w || $loaded.width();
			settings.w = settings.mw && settings.mw < settings.w ? settings.mw : settings.w;
			return settings.w;
		}
		function getHeight() {
			settings.h = settings.h || $loaded.height();
			settings.h = settings.mh && settings.mh < settings.h ? settings.mh : settings.h;
			return settings.h;
		}

		$loaded.hide()
		.appendTo($loadingBay.show())// content has to be appended to the DOM for accurate size calculations.
		.css({width: getWidth(), overflow: settings.get('scrolling') ? 'auto' : 'hidden'})
		.css({height: getHeight()})// sets the height independently from the width in case the new width influences the value of height.
		.prependTo($content);

		$loadingBay.hide();

		// floating the IMG removes the bottom line-height and fixed a problem where IE miscalculates the width of the parent element as 100% of the document width.

		$(photo).css({'float': 'none'});

		setClass(settings.get('className'));

		callback = function () {
			var total = $related.length,
				iframe,
				complete;

			if (!open) {
				return;
			}

			function removeFilter() { // Needed for IE8 in versions of jQuery prior to 1.7.2
				if ($.support.opacity === false) {
					$box[0].style.removeAttribute('filter');
				}
			}

			complete = function () {
				clearTimeout(loadingTimer);
				$loadingOverlay.hide();
				trigger(event_complete);
				settings.get('onComplete');
			};


			$title.html(settings.get('title')).show();
			$loaded.show();

			if (total > 1) { // handle grouping
				if (typeof settings.get('current') === "string") {
					$current.html(settings.get('current').replace('{current}', index + 1).replace('{total}', total)).show();
				}

				$next[(settings.get('loop') || index < total - 1) ? "show" : "hide"]().html(settings.get('next'));
				$prev[(settings.get('loop') || index) ? "show" : "hide"]().html(settings.get('previous'));

				slideshow();

				// Preloads images within a rel group
				if (settings.get('preloading')) {
					$.each([getIndex(-1), getIndex(1)], function(){
						var img,
							i = $related[this],
							settings = new Settings(i, $.data(i, colorbox)),
							src = settings.get('href');

						if (src && isImage(settings, src)) {
							src = retinaUrl(settings, src);
							img = document.createElement('img');
							img.src = src;
						}
					});
				}
			} else {
				$groupControls.hide();
			}

			if (settings.get('iframe')) {

				iframe = settings.get('createIframe');

				if (!settings.get('scrolling')) {
					iframe.scrolling = "no";
				}

				$(iframe)
					.attr({
						src: settings.get('href'),
						'class': prefix + 'Iframe'
					})
					.one('load', complete)
					.appendTo($loaded);

				$events.one(event_purge, function () {
					iframe.src = "//about:blank";
				});

				if (settings.get('fastIframe')) {
					$(iframe).trigger('load');
				}
			} else {
				complete();
			}

			if (settings.get('transition') === 'fade') {
				$box.fadeTo(speed, 1, removeFilter);
			} else {
				removeFilter();
			}
		};

		if (settings.get('transition') === 'fade') {
			$box.fadeTo(speed, 0, function () {
				publicMethod.position(0, callback);
			});
		} else {
			publicMethod.position(speed, callback);
		}
	};

	function load () {
		var href, setResize, prep = publicMethod.prep, $inline, request = ++requests;

		active = true;

		photo = false;

		trigger(event_purge);
		trigger(event_load);
		settings.get('onLoad');

		settings.h = settings.get('height') ?
				setSize(settings.get('height'), 'y') - loadedHeight - interfaceHeight :
				settings.get('innerHeight') && setSize(settings.get('innerHeight'), 'y');

		settings.w = settings.get('width') ?
				setSize(settings.get('width'), 'x') - loadedWidth - interfaceWidth :
				settings.get('innerWidth') && setSize(settings.get('innerWidth'), 'x');

		// Sets the minimum dimensions for use in image scaling
		settings.mw = settings.w;
		settings.mh = settings.h;

		// Re-evaluate the minimum width and height based on maxWidth and maxHeight values.
		// If the width or height exceed the maxWidth or maxHeight, use the maximum values instead.
		if (settings.get('maxWidth')) {
			settings.mw = setSize(settings.get('maxWidth'), 'x') - loadedWidth - interfaceWidth;
			settings.mw = settings.w && settings.w < settings.mw ? settings.w : settings.mw;
		}
		if (settings.get('maxHeight')) {
			settings.mh = setSize(settings.get('maxHeight'), 'y') - loadedHeight - interfaceHeight;
			settings.mh = settings.h && settings.h < settings.mh ? settings.h : settings.mh;
		}

		href = settings.get('href');

		loadingTimer = setTimeout(function () {
			$loadingOverlay.show();
		}, 100);

		if (settings.get('inline')) {
			var $target = $(href).eq(0);
			// Inserts an empty placeholder where inline content is being pulled from.
			// An event is bound to put inline content back when Colorbox closes or loads new content.
			$inline = $('<div>').hide().insertBefore($target);

			$events.one(event_purge, function () {
				$inline.replaceWith($target);
			});

			prep($target);
		} else if (settings.get('iframe')) {
			// IFrame element won't be added to the DOM until it is ready to be displayed,
			// to avoid problems with DOM-ready JS that might be trying to run in that iframe.
			prep(" ");
		} else if (settings.get('html')) {
			prep(settings.get('html'));
		} else if (isImage(settings, href)) {

			href = retinaUrl(settings, href);

			photo = settings.get('createImg');

			$(photo)
			.addClass(prefix + 'Photo')
			.bind('error.'+prefix,function () {
				prep($tag(div, 'Error').html(settings.get('imgError')));
			})
			.one('load', function () {
				if (request !== requests) {
					return;
				}

				// A small pause because some browsers will occasionally report a
				// img.width and img.height of zero immediately after the img.onload fires
				setTimeout(function(){
					var percent;

					if (settings.get('retinaImage') && window.devicePixelRatio > 1) {
						photo.height = photo.height / window.devicePixelRatio;
						photo.width = photo.width / window.devicePixelRatio;
					}

					if (settings.get('scalePhotos')) {
						setResize = function () {
							photo.height -= photo.height * percent;
							photo.width -= photo.width * percent;
						};
						if (settings.mw && photo.width > settings.mw) {
							percent = (photo.width - settings.mw) / photo.width;
							setResize();
						}
						if (settings.mh && photo.height > settings.mh) {
							percent = (photo.height - settings.mh) / photo.height;
							setResize();
						}
					}

					if (settings.h) {
						photo.style.marginTop = Math.max(settings.mh - photo.height, 0) / 2 + 'px';
					}

					if ($related[1] && (settings.get('loop') || $related[index + 1])) {
						photo.style.cursor = 'pointer';

						$(photo).bind('click.'+prefix, function () {
							publicMethod.next();
						});
					}

					photo.style.width = photo.width + 'px';
					photo.style.height = photo.height + 'px';
					prep(photo);
				}, 1);
			});

			photo.src = href;

		} else if (href) {
			$loadingBay.load(href, settings.get('data'), function (data, status) {
				if (request === requests) {
					prep(status === 'error' ? $tag(div, 'Error').html(settings.get('xhrError')) : $(this).contents());
				}
			});
		}
	}

	// Navigates to the next page/image in a set.
	publicMethod.next = function () {
		if (!active && $related[1] && (settings.get('loop') || $related[index + 1])) {
			index = getIndex(1);
			launch($related[index]);
		}
	};

	publicMethod.prev = function () {
		if (!active && $related[1] && (settings.get('loop') || index)) {
			index = getIndex(-1);
			launch($related[index]);
		}
	};

	// Note: to use this within an iframe use the following format: parent.jQuery.colorbox.close();
	publicMethod.close = function () {
		if (open && !closing) {

			closing = true;
			open = false;
			trigger(event_cleanup);
			settings.get('onCleanup');
			$window.unbind('.' + prefix);
			$overlay.fadeTo(settings.get('fadeOut') || 0, 0);

			$box.stop().fadeTo(settings.get('fadeOut') || 0, 0, function () {
				$box.hide();
				$overlay.hide();
				trigger(event_purge);
				$loaded.remove();

				setTimeout(function () {
					closing = false;
					trigger(event_closed);
					settings.get('onClosed');
				}, 1);
			});
		}
	};

	// Removes changes Colorbox made to the document, but does not remove the plugin.
	publicMethod.remove = function () {
		if (!$box) { return; }

		$box.stop();
		$[colorbox].close();
		$box.stop(false, true).remove();
		$overlay.remove();
		closing = false;
		$box = null;
		$('.' + boxElement)
			.removeData(colorbox)
			.removeClass(boxElement);

		$(document).unbind('click.'+prefix).unbind('keydown.'+prefix);
	};

	// A method for fetching the current element Colorbox is referencing.
	// returns a jQuery object.
	publicMethod.element = function () {
		return $(settings.el);
	};

	publicMethod.settings = defaults;

}(jQuery, document, window));



/*******************
 * From src: synergy/src/js/20.synergy.flexselect.js
 *******************/
/**
 * flexselect: a jQuery plugin, version: %RELEASE_VERSION% (%RELEASE_DATE%)
 * @requires jQuery v1.3 or later
 *
 * FlexSelect is a jQuery plugin that makes it easy to convert a select box into
 * a Quicksilver-style, autocompleting, flex matching selection tool.
 *
 * For usage and examples, visit:
 * http://rmm5t.github.io/jquery-flexselect/
 *
 * Licensed under the MIT:
 * http://www.opensource.org/licenses/mit-license.php
 *
 * Copyright (c) 2009-2015, Ryan McGeary (ryan -[at]- mcgeary [*dot*] org)
 */
(function($) {
  $.flexselect = function(select, options) { this.init(select, options); };

  $.extend($.flexselect.prototype, {
    settings: {
      allowMismatch: false,
      allowMismatchBlank: true, // If "true" a user can backspace such that the value is nothing (even if no blank value was provided in the original criteria)
      sortBy: 'score', // 'score' || 'name'
      blankSortBy: 'initial', // 'score' || 'name' || 'initial'
      preSelection: true,
      hideDropdownOnEmptyInput: false,
      selectedClass: "flexselect_selected",
      dropdownClass: "flexselect_dropdown",
      showDisabledOptions: false,
      inputIdTransform:    function(id)   { return id + "_flexselect"; },
      inputNameTransform:  function(name) { return; },
      dropdownIdTransform: function(id)   { return id + "_flexselect_dropdown"; }
    },
    select: null,
    input: null,
    dropdown: null,
    dropdownList: null,
    cache: [],
    results: [],
    lastAbbreviation: null,
    abbreviationBeforeFocus: null,
    selectedIndex: 0,
    picked: false,
    allowMouseMove: true,
    dropdownMouseover: false, // Workaround for poor IE behaviors
    indexOptgroupLabels: false,

    init: function(select, options) {
      this.settings = $.extend({}, this.settings, options);
      this.select = $(select);
      this.reloadCache();
      this.renderControls();
      this.wire();
    },

    reloadCache: function() {
      var name, group, text, disabled;
      var indexGroup = this.settings.indexOptgroupLabels;
      this.cache = this.select.find("option").map(function() {
        name = $(this).text();
        group = $(this).parent("optgroup").attr("label");
        text = indexGroup ? [name, group].join(" ") : name;
        disabled = $(this).parent("optgroup").attr("disabled") || $(this).attr('disabled');
        return { text: $.trim(text), name: $.trim(name), value: $(this).val(), disabled: disabled, score: 0.0 };
      });
    },

    renderControls: function() {
      var selected = this.settings.preSelection ? this.select.find("option:selected") : null;

      this.input = $("<input type='text' autocomplete='off' />").attr({
        id: this.settings.inputIdTransform(this.select.attr("id")),
        name: this.settings.inputNameTransform(this.select.attr("name")),
        accesskey: this.select.attr("accesskey"),
        tabindex: this.select.attr("tabindex"),
        style: this.select.attr("style"),
        placeholder: this.select.attr("data-placeholder")
      }).addClass(this.select.attr("class")).val($.trim(selected ? selected.text():  '')).css({
        visibility: 'visible'
      });

      this.dropdown = $("<div></div>").attr({
        id: this.settings.dropdownIdTransform(this.select.attr("id"))
      }).addClass(this.settings.dropdownClass);
      this.dropdownList = $("<ul></ul>");
      this.dropdown.append(this.dropdownList);

      this.select.after(this.input).hide();
      $("body").append(this.dropdown);
    },

    wire: function() {
      var self = this;

      this.input.click(function() {
        self.lastAbbreviation = null;
        self.focus();
      });

      this.input.mouseup(function(event) {
        // This is so Safari selection actually occurs.
        event.preventDefault();
      });

      this.input.focus(function() {
        self.abbreviationBeforeFocus = self.input.val();
        self.input[0].setSelectionRange(0, self.input.val().length);
        if (!self.picked) self.filterResults();
      });

      this.input.blur(function() {
        if (!self.dropdownMouseover) {
          self.hide();
          if (self.settings.allowMismatchBlank && $.trim($(this).val()) == '')
            self.setValue('');
          else if (!self.settings.allowMismatch && !self.picked)
            self.reset();
        }
      });

      this.dropdownList.mouseover(function(event) {
        if (!self.allowMouseMove) {
          self.allowMouseMove = true;
          return;
        }

        if (event.target.tagName == "LI") {
          var rows = self.dropdown.find("li");
          self.markSelected(rows.index($(event.target)));
        }
      });
      this.dropdownList.mouseleave(function() {
        self.markSelected(-1);
      });
      this.dropdownList.mouseup(function(event) {
        self.pickSelected();
        self.focusAndHide();
      });
      this.dropdownList.bind("touchstart", function(event) {
        if (event.target.tagName == "LI") {
          var rows = self.dropdown.find("li");
          self.markSelected(rows.index($(event.target)));
        }
      });
      this.dropdown.mouseover(function(event) {
        self.dropdownMouseover = true;
      });
      this.dropdown.mouseleave(function(event) {
        self.dropdownMouseover = false;
      });
      this.dropdown.mousedown(function(event) {
        event.preventDefault();
      });

      this.input.keyup(function(event) {
        switch (event.keyCode) {
          case 13: // return
            event.preventDefault();
            self.pickSelected();
            self.focusAndHide();
            break;
          case 27: // esc
            event.preventDefault();
            self.reset();
            self.focusAndHide();
            break;
          default:
            self.filterResults();
            break;
        }
      });

      this.input.keydown(function(event) {
        switch (event.keyCode) {
          case 9:  // tab
            self.pickSelected();
            self.hide();
            break;
          case 33: // pgup
            event.preventDefault();
            self.markFirst();
            break;
          case 34: // pgedown
            event.preventDefault();
            self.markLast();
            break;
          case 38: // up
            event.preventDefault();
            self.moveSelected(-1);
            break;
          case 40: // down
            event.preventDefault();
            self.moveSelected(1);
            break;
          case 13: // return
          case 27: // esc
            event.preventDefault();
            event.stopPropagation();
            break;
        }
      });

      var input = this.input;
      this.select.change(function () {
        input.val($.trim($(this).find('option:selected').text()));
      });
    },

    filterResults: function() {
      var showDisabled = this.settings.showDisabledOptions;
      var abbreviation = $.trim(this.input.val());
      var sortByMechanism = (abbreviation == "") ? this.settings.blankSortBy : this.settings.sortBy;
      if (abbreviation == this.lastAbbreviation) return;

      var results = [];
      $.each(this.cache, function() {
        if (this.disabled && !showDisabled) return;
        this.score = LiquidMetal.score(this.text, abbreviation);
        if (this.score > 0.0) results.push(this);
      });
      this.results = results;

      this.sortResultsBy(sortByMechanism);
      this.renderDropdown();
      this.markFirst();
      this.lastAbbreviation = abbreviation;
      this.picked = false;
      this.allowMouseMove = false;

      if (this.settings.hideDropdownOnEmptyInput){
        if (abbreviation == "")
          this.dropdown.hide();
        else
          this.dropdown.show();
      }
    },

    sortResultsBy: function(mechanism) {
      if (mechanism == "score") {
        this.sortResultsByScore();
      } else if (mechanism == "name") {
        this.sortResultsByName();
      }
    },

    sortResultsByScore: function() {
      this.results.sort(function(a, b) { return b.score - a.score; });
    },

    sortResultsByName: function() {
      this.results.sort(function(a, b) { return a.name < b.name ? -1 : (a.name > b.name ? 1 : 0); });
    },

    renderDropdown: function() {
      var showDisabled = this.settings.showDisabledOptions;
      var dropdownBorderWidth = this.dropdown.outerWidth() - this.dropdown.innerWidth();
      var inputOffset = this.input.offset();
      this.dropdown.css({
        width: (this.input.outerWidth() - dropdownBorderWidth) + "px",
        top: (inputOffset.top + this.input.outerHeight()) + "px",
        left: inputOffset.left + "px",
        maxHeight: ''
      });

      var html = '';
      var disabledAttribute = '';
      $.each(this.results, function() {
        if (this.disabled && !showDisabled) return;
        disabledAttribute = this.disabled ? ' class="disabled"' : '';
        html += '<li' + disabledAttribute + '>' + this.name + '</li>';
      });
      this.dropdownList.html(html);
      this.adjustMaxHeight();
      this.dropdown.show();
    },

    adjustMaxHeight: function() {
      var maxTop = $(window).height() + $(window).scrollTop() - this.dropdown.outerHeight();
      var top = parseInt(this.dropdown.css('top'), 10);
      this.dropdown.css('max-height', top > maxTop ? (Math.max(0, maxTop - top + this.dropdown.innerHeight()) + 'px') : '');
    },

    markSelected: function(n) {
      if (n < 0 || n >= this.results.length) return;

      var rows = this.dropdown.find("li");
      rows.removeClass(this.settings.selectedClass);

      var row = $(rows[n]);
      if (row.hasClass('disabled')) {
        this.selectedIndex = null;
        return;
      }

      this.selectedIndex = n;
      row.addClass(this.settings.selectedClass);
      var top = row.position().top;
      var delta = top + row.outerHeight() - this.dropdown.height();
      if (delta > 0) {
        this.allowMouseMove = false;
        this.dropdown.scrollTop(this.dropdown.scrollTop() + delta);
      } else if (top < 0) {
        this.allowMouseMove = false;
        this.dropdown.scrollTop(Math.max(0, this.dropdown.scrollTop() + top));
      }
    },

    pickSelected: function() {
      var selected = this.results[this.selectedIndex];
      if (selected && !selected.disabled) {
        this.input.val(selected.name);
        this.setValue(selected.value);
        this.picked = true;
        this.input.triggerHandler("item-selected");
      } else if (this.settings.allowMismatch) {
        this.setValue.val("");
      } else {
        this.reset();
      }
    },

    setValue: function(val) {
      if (this.select.val() === val) return;
      this.select.val(val).change();
    },

    hide: function() {
      this.dropdown.hide();
      this.lastAbbreviation = null;
    },

    moveSelected: function(n) { this.markSelected(this.selectedIndex+n); },
    markFirst:    function()  { this.markSelected(0); },
    markLast:     function()  { this.markSelected(this.results.length - 1); },
    reset:        function()  { this.input.val(this.abbreviationBeforeFocus); },
    focus:        function()  { this.input.focus(); },
    focusAndHide: function()  { this.focus(); this.hide(); }
  });

  $.fn.flexselect = function(options) {
    this.each(function() {
      if ($(this).data("flexselect")) {
        $(this).data("flexselect").reloadCache();
      } else if (this.tagName == "SELECT") {
        $(this).data("flexselect", new $.flexselect(this, options));
      }
    });
    return this;
  };
})(jQuery);



/*******************
 * From src: synergy/src/js/20.synergy.json2.js
 *******************/
/*
    json2.js
    2011-10-19

    Public Domain.

    NO WARRANTY EXPRESSED OR IMPLIED. USE AT YOUR OWN RISK.

    See http://www.JSON.org/js.html


    This code should be minified before deployment.
    See http://javascript.crockford.com/jsmin.html

    USE YOUR OWN COPY. IT IS EXTREMELY UNWISE TO LOAD CODE FROM SERVERS YOU DO
    NOT CONTROL.


    This file creates a global JSON object containing two methods: stringify
    and parse.

        JSON.stringify(value, replacer, space)
            value       any JavaScript value, usually an object or array.

            replacer    an optional parameter that determines how object
                        values are stringified for objects. It can be a
                        function or an array of strings.

            space       an optional parameter that specifies the indentation
                        of nested structures. If it is omitted, the text will
                        be packed without extra whitespace. If it is a number,
                        it will specify the number of spaces to indent at each
                        level. If it is a string (such as '\t' or '&nbsp;'),
                        it contains the characters used to indent at each level.

            This method produces a JSON text from a JavaScript value.

            When an object value is found, if the object contains a toJSON
            method, its toJSON method will be called and the result will be
            stringified. A toJSON method does not serialize: it returns the
            value represented by the name/value pair that should be serialized,
            or undefined if nothing should be serialized. The toJSON method
            will be passed the key associated with the value, and this will be
            bound to the value

            For example, this would serialize Dates as ISO strings.

                Date.prototype.toJSON = function (key) {
                    function f(n) {
                        // Format integers to have at least two digits.
                        return n < 10 ? '0' + n : n;
                    }

                    return this.getUTCFullYear()   + '-' +
                         f(this.getUTCMonth() + 1) + '-' +
                         f(this.getUTCDate())      + 'T' +
                         f(this.getUTCHours())     + ':' +
                         f(this.getUTCMinutes())   + ':' +
                         f(this.getUTCSeconds())   + 'Z';
                };

            You can provide an optional replacer method. It will be passed the
            key and value of each member, with this bound to the containing
            object. The value that is returned from your method will be
            serialized. If your method returns undefined, then the member will
            be excluded from the serialization.

            If the replacer parameter is an array of strings, then it will be
            used to select the members to be serialized. It filters the results
            such that only members with keys listed in the replacer array are
            stringified.

            Values that do not have JSON representations, such as undefined or
            functions, will not be serialized. Such values in objects will be
            dropped; in arrays they will be replaced with null. You can use
            a replacer function to replace those with JSON values.
            JSON.stringify(undefined) returns undefined.

            The optional space parameter produces a stringification of the
            value that is filled with line breaks and indentation to make it
            easier to read.

            If the space parameter is a non-empty string, then that string will
            be used for indentation. If the space parameter is a number, then
            the indentation will be that many spaces.

            Example:

            text = JSON.stringify(['e', {pluribus: 'unum'}]);
            // text is '["e",{"pluribus":"unum"}]'


            text = JSON.stringify(['e', {pluribus: 'unum'}], null, '\t');
            // text is '[\n\t"e",\n\t{\n\t\t"pluribus": "unum"\n\t}\n]'

            text = JSON.stringify([new Date()], function (key, value) {
                return this[key] instanceof Date ?
                    'Date(' + this[key] + ')' : value;
            });
            // text is '["Date(---current time---)"]'


        JSON.parse(text, reviver)
            This method parses a JSON text to produce an object or array.
            It can throw a SyntaxError exception.

            The optional reviver parameter is a function that can filter and
            transform the results. It receives each of the keys and values,
            and its return value is used instead of the original value.
            If it returns what it received, then the structure is not modified.
            If it returns undefined then the member is deleted.

            Example:

            // Parse the text. Values that look like ISO date strings will
            // be converted to Date objects.

            myData = JSON.parse(text, function (key, value) {
                var a;
                if (typeof value === 'string') {
                    a =
/^(\d{4})-(\d{2})-(\d{2})T(\d{2}):(\d{2}):(\d{2}(?:\.\d*)?)Z$/.exec(value);
                    if (a) {
                        return new Date(Date.UTC(+a[1], +a[2] - 1, +a[3], +a[4],
                            +a[5], +a[6]));
                    }
                }
                return value;
            });

            myData = JSON.parse('["Date(09/09/2001)"]', function (key, value) {
                var d;
                if (typeof value === 'string' &&
                        value.slice(0, 5) === 'Date(' &&
                        value.slice(-1) === ')') {
                    d = new Date(value.slice(5, -1));
                    if (d) {
                        return d;
                    }
                }
                return value;
            });


    This is a reference implementation. You are free to copy, modify, or
    redistribute.
*/

/*jslint evil: true, regexp: true */

/*members "", "\b", "\t", "\n", "\f", "\r", "\"", JSON, "\\", apply,
    call, charCodeAt, getUTCDate, getUTCFullYear, getUTCHours,
    getUTCMinutes, getUTCMonth, getUTCSeconds, hasOwnProperty, join,
    lastIndex, length, parse, prototype, push, replace, slice, stringify,
    test, toJSON, toString, valueOf
*/


// Create a JSON object only if one does not already exist. We create the
// methods in a closure to avoid creating global variables.

var JSON;
if (!JSON) {
    JSON = {};
}

(function () {
    'use strict';

    function f(n) {
        // Format integers to have at least two digits.
        return n < 10 ? '0' + n : n;
    }

    if (typeof Date.prototype.toJSON !== 'function') {

        Date.prototype.toJSON = function (key) {

            return isFinite(this.valueOf())
                ? this.getUTCFullYear()     + '-' +
                    f(this.getUTCMonth() + 1) + '-' +
                    f(this.getUTCDate())      + 'T' +
                    f(this.getUTCHours())     + ':' +
                    f(this.getUTCMinutes())   + ':' +
                    f(this.getUTCSeconds())   + 'Z'
                : null;
        };

        String.prototype.toJSON      =
            Number.prototype.toJSON  =
            Boolean.prototype.toJSON = function (key) {
                return this.valueOf();
            };
    }

    var cx = /[\u0000\u00ad\u0600-\u0604\u070f\u17b4\u17b5\u200c-\u200f\u2028-\u202f\u2060-\u206f\ufeff\ufff0-\uffff]/g,
        escapable = /[\\\"\x00-\x1f\x7f-\x9f\u00ad\u0600-\u0604\u070f\u17b4\u17b5\u200c-\u200f\u2028-\u202f\u2060-\u206f\ufeff\ufff0-\uffff]/g,
        gap,
        indent,
        meta = {    // table of character substitutions
            '\b': '\\b',
            '\t': '\\t',
            '\n': '\\n',
            '\f': '\\f',
            '\r': '\\r',
            '"' : '\\"',
            '\\': '\\\\'
        },
        rep;


    function quote(string) {

// If the string contains no control characters, no quote characters, and no
// backslash characters, then we can safely slap some quotes around it.
// Otherwise we must also replace the offending characters with safe escape
// sequences.

        escapable.lastIndex = 0;
        return escapable.test(string) ? '"' + string.replace(escapable, function (a) {
            var c = meta[a];
            return typeof c === 'string'
                ? c
                : '\\u' + ('0000' + a.charCodeAt(0).toString(16)).slice(-4);
        }) + '"' : '"' + string + '"';
    }


    function str(key, holder) {

// Produce a string from holder[key].

        var i,          // The loop counter.
            k,          // The member key.
            v,          // The member value.
            length,
            mind = gap,
            partial,
            value = holder[key];

// If the value has a toJSON method, call it to obtain a replacement value.

        if (value && typeof value === 'object' &&
                typeof value.toJSON === 'function') {
            value = value.toJSON(key);
        }

// If we were called with a replacer function, then call the replacer to
// obtain a replacement value.

        if (typeof rep === 'function') {
            value = rep.call(holder, key, value);
        }

// What happens next depends on the value's type.

        switch (typeof value) {
        case 'string':
            return quote(value);

        case 'number':

// JSON numbers must be finite. Encode non-finite numbers as null.

            return isFinite(value) ? String(value) : 'null';

        case 'boolean':
        case 'null':

// If the value is a boolean or null, convert it to a string. Note:
// typeof null does not produce 'null'. The case is included here in
// the remote chance that this gets fixed someday.

            return String(value);

// If the type is 'object', we might be dealing with an object or an array or
// null.

        case 'object':

// Due to a specification blunder in ECMAScript, typeof null is 'object',
// so watch out for that case.

            if (!value) {
                return 'null';
            }

// Make an array to hold the partial results of stringifying this object value.

            gap += indent;
            partial = [];

// Is the value an array?

            if (Object.prototype.toString.apply(value) === '[object Array]') {

// The value is an array. Stringify every element. Use null as a placeholder
// for non-JSON values.

                length = value.length;
                for (i = 0; i < length; i += 1) {
                    partial[i] = str(i, value) || 'null';
                }

// Join all of the elements together, separated with commas, and wrap them in
// brackets.

                v = partial.length === 0
                    ? '[]'
                    : gap
                    ? '[\n' + gap + partial.join(',\n' + gap) + '\n' + mind + ']'
                    : '[' + partial.join(',') + ']';
                gap = mind;
                return v;
            }

// If the replacer is an array, use it to select the members to be stringified.

            if (rep && typeof rep === 'object') {
                length = rep.length;
                for (i = 0; i < length; i += 1) {
                    if (typeof rep[i] === 'string') {
                        k = rep[i];
                        v = str(k, value);
                        if (v) {
                            partial.push(quote(k) + (gap ? ': ' : ':') + v);
                        }
                    }
                }
            } else {

// Otherwise, iterate through all of the keys in the object.

                for (k in value) {
                    if (Object.prototype.hasOwnProperty.call(value, k)) {
                        v = str(k, value);
                        if (v) {
                            partial.push(quote(k) + (gap ? ': ' : ':') + v);
                        }
                    }
                }
            }

// Join all of the member texts together, separated with commas,
// and wrap them in braces.

            v = partial.length === 0
                ? '{}'
                : gap
                ? '{\n' + gap + partial.join(',\n' + gap) + '\n' + mind + '}'
                : '{' + partial.join(',') + '}';
            gap = mind;
            return v;
        }
    }

// If the JSON object does not yet have a stringify method, give it one.

    if (typeof JSON.stringify !== 'function') {
        JSON.stringify = function (value, replacer, space) {

// The stringify method takes a value and an optional replacer, and an optional
// space parameter, and returns a JSON text. The replacer can be a function
// that can replace values, or an array of strings that will select the keys.
// A default replacer method can be provided. Use of the space parameter can
// produce text that is more easily readable.

            var i;
            gap = '';
            indent = '';

// If the space parameter is a number, make an indent string containing that
// many spaces.

            if (typeof space === 'number') {
                for (i = 0; i < space; i += 1) {
                    indent += ' ';
                }

// If the space parameter is a string, it will be used as the indent string.

            } else if (typeof space === 'string') {
                indent = space;
            }

// If there is a replacer, it must be a function or an array.
// Otherwise, throw an error.

            rep = replacer;
            if (replacer && typeof replacer !== 'function' &&
                    (typeof replacer !== 'object' ||
                    typeof replacer.length !== 'number')) {
                throw new Error('JSON.stringify');
            }

// Make a fake root object containing our value under the key of ''.
// Return the result of stringifying the value.

            return str('', {'': value});
        };
    }


// If the JSON object does not yet have a parse method, give it one.

    if (typeof JSON.parse !== 'function') {
        JSON.parse = function (text, reviver) {

// The parse method takes a text and an optional reviver function, and returns
// a JavaScript value if the text is a valid JSON text.

            var j;

            function walk(holder, key) {

// The walk method is used to recursively walk the resulting structure so
// that modifications can be made.

                var k, v, value = holder[key];
                if (value && typeof value === 'object') {
                    for (k in value) {
                        if (Object.prototype.hasOwnProperty.call(value, k)) {
                            v = walk(value, k);
                            if (v !== undefined) {
                                value[k] = v;
                            } else {
                                delete value[k];
                            }
                        }
                    }
                }
                return reviver.call(holder, key, value);
            }


// Parsing happens in four stages. In the first stage, we replace certain
// Unicode characters with escape sequences. JavaScript handles many characters
// incorrectly, either silently deleting them, or treating them as line endings.

            text = String(text);
            cx.lastIndex = 0;
            if (cx.test(text)) {
                text = text.replace(cx, function (a) {
                    return '\\u' +
                        ('0000' + a.charCodeAt(0).toString(16)).slice(-4);
                });
            }

// In the second stage, we run the text against regular expressions that look
// for non-JSON patterns. We are especially concerned with '()' and 'new'
// because they can cause invocation, and '=' because it can cause mutation.
// But just to be safe, we want to reject all unexpected forms.

// We split the second stage into 4 regexp operations in order to work around
// crippling inefficiencies in IE's and Safari's regexp engines. First we
// replace the JSON backslash pairs with '@' (a non-JSON character). Second, we
// replace all simple value tokens with ']' characters. Third, we delete all
// open brackets that follow a colon or comma or that begin the text. Finally,
// we look to see that the remaining characters are only whitespace or ']' or
// ',' or ':' or '{' or '}'. If that is so, then the text is safe for eval.

            if (/^[\],:{}\s]*$/
                    .test(text.replace(/\\(?:["\\\/bfnrt]|u[0-9a-fA-F]{4})/g, '@')
                        .replace(/"[^"\\\n\r]*"|true|false|null|-?\d+(?:\.\d*)?(?:[eE][+\-]?\d+)?/g, ']')
                        .replace(/(?:^|:|,)(?:\s*\[)+/g, ''))) {

// In the third stage we use the eval function to compile the text into a
// JavaScript structure. The '{' operator is subject to a syntactic ambiguity
// in JavaScript: it can begin a block or an object literal. We wrap the text
// in parens to eliminate the ambiguity.

                j = eval('(' + text + ')');

// In the optional fourth stage, we recursively walk the new structure, passing
// each name/value pair to a reviver function for possible transformation.

                return typeof reviver === 'function'
                    ? walk({'': j}, '')
                    : j;
            }

// If the text is not JSON parseable, then a SyntaxError is thrown.

            throw new SyntaxError('JSON.parse');
        };
    }
}());



/*******************
 * From src: synergy/src/js/20.synergy.liquidmetal.js
 *******************/
/**
 * LiquidMetal, version: 1.0 (2011-12-17)
 *
 * A mimetic poly-alloy of Quicksilver's scoring algorithm, essentially
 * LiquidMetal.
 *
 * For usage and examples, visit:
 * http://github.com/rmm5t/liquidmetal
 *
 * Licensed under the MIT:
 * http://www.opensource.org/licenses/mit-license.php
 *
 * Copyright (c) 2009-2012, Ryan McGeary (ryan -[at]- mcgeary [*dot*] org)
 */
var LiquidMetal = (function() {
  var SCORE_NO_MATCH = 0.0;
  var SCORE_MATCH = 1.0;
  var SCORE_TRAILING = 0.8;
  var SCORE_TRAILING_BUT_STARTED = 0.9;
  var SCORE_BUFFER = 0.85;

  return {
    score: function(string, abbreviation) {
      // Short circuits
      if (abbreviation.length === 0) return SCORE_TRAILING;
      if (abbreviation.length > string.length) return SCORE_NO_MATCH;

      var scores = this.buildScoreArray(string, abbreviation);

      // complete miss:
      if ( scores === false )  return 0;

      var sum = 0.0;
      for (var i = 0; i < scores.length; i++) {
        sum += scores[i];
      }

      return (sum / scores.length);
    },

    buildScoreArray: function(string, abbreviation) {
      var scores = new Array(string.length);
      var lower = string.toLowerCase();
      var chars = abbreviation.toLowerCase();

      var lastIndex = -1;
      var started = false;
      for (var i = 0; i < chars.length; i++) {
        var c = chars[i];
        var index = lower.indexOf(c, lastIndex+1);

        if (index === -1) return false; // signal no match
        if (index === 0) started = true;

        if (isNewWord(string, index)) {
          scores[index-1] = 1;
          fillArray(scores, SCORE_BUFFER, lastIndex+1, index-1);
        }
        else if (isUpperCase(string, index)) {
          fillArray(scores, SCORE_BUFFER, lastIndex+1, index);
        }
        else {
          fillArray(scores, SCORE_NO_MATCH, lastIndex+1, index);
        }

        scores[index] = SCORE_MATCH;
        lastIndex = index;
      }

      var trailingScore = started ? SCORE_TRAILING_BUT_STARTED : SCORE_TRAILING;
      fillArray(scores, trailingScore, lastIndex+1, scores.length);
      return scores;
    }
  };

  function isUpperCase(string, index) {
    var c = string.charAt(index);
    return ("A" <= c && c <= "Z");
  }

   function isNewWord(string, index) {
    var c = string.charAt(index-1);
    return (c == " " || c == "\t");
  }

  function fillArray(array, value, from, to) {
    for (var i = from; i < to; i++) { array[i] = value; }
    return array;
  }
})();



/*******************
 * From src: synergy/src/js/20.synergy.tipsy.js
 *******************/
// tipsy, facebook style tooltips for jquery
// version 1.0.0a
// (c) 2008-2010 jason frame [jason@onehackoranother.com]
// released under the MIT license
(function($, jQuery) {

    function maybeCall(thing, ctx) {
        return (typeof thing == 'function') ? (thing.call(ctx)) : thing;
    };

    function Tipsy(element, options) {
        this.$element = $(element);
        this.options = options;
        this.enabled = true;
        this.fixTitle();
    };

    Tipsy.prototype = {
        show: function() {
            var title = this.getTitle();
            if (title && this.enabled) {
                var $tip = this.tip();

                $tip.find('.tipsy-inner')[this.options.html ? 'html' : 'text'](title);
                $tip[0].className = 'tipsy'; // reset classname in case of dynamic gravity
                $tip.remove().css({top: 0, left: 0, visibility: 'hidden', display: 'block'}).prependTo(document.body);

                var pos = $.extend({}, this.$element.offset(), {
                    width: this.$element[0].offsetWidth,
                    height: this.$element[0].offsetHeight
                });

                var actualWidth = $tip[0].offsetWidth,
                    actualHeight = $tip[0].offsetHeight,
                    gravity = maybeCall(this.options.gravity, this.$element[0]);

                var tp;
                switch (gravity.charAt(0)) {
                    case 'n':
                        tp = {top: pos.top + pos.height + this.options.offset, left: pos.left + pos.width / 2 - actualWidth / 2};
                        break;
                    case 's':
                        tp = {top: pos.top - actualHeight - this.options.offset, left: pos.left + pos.width / 2 - actualWidth / 2};
                        break;
                    case 'e':
                        tp = {top: pos.top + pos.height / 2 - actualHeight / 2, left: pos.left - actualWidth - this.options.offset};
                        break;
                    case 'w':
                        tp = {top: pos.top + pos.height / 2 - actualHeight / 2, left: pos.left + pos.width + this.options.offset};
                        break;
                }

                if (gravity.length == 2) {
                    if (gravity.charAt(1) == 'w') {
                        tp.left = pos.left + pos.width / 2 - 15;
                    } else {
                        tp.left = pos.left + pos.width / 2 - actualWidth + 15;
                    }
                }

                $tip.css(tp).addClass('tipsy-' + gravity);
                $tip.find('.tipsy-arrow')[0].className = 'tipsy-arrow tipsy-arrow-' + gravity.charAt(0);
                if (this.options.className) {
                    $tip.addClass(maybeCall(this.options.className, this.$element[0]));
                }

                if (this.options.fade) {
                    $tip.stop().css({opacity: 0, display: 'block', visibility: 'visible'}).animate({opacity: this.options.opacity});
                } else {
                    $tip.css({visibility: 'visible', opacity: this.options.opacity});
                }
            }
        },

        hide: function() {
            if (this.options.fade) {
                this.tip().stop().fadeOut(function() { $(this).remove(); });
            } else {
                this.tip().remove();
            }
        },

        fixTitle: function() {
            var $e = this.$element;
            if ($e.attr('title') || typeof($e.attr('original-title')) != 'string') {
                $e.attr('original-title', $e.attr('title') || '').removeAttr('title');
            }
        },

        getTitle: function() {
            var title, $e = this.$element, o = this.options;
            this.fixTitle();
            var title, o = this.options;
            if (typeof o.title == 'string') {
                title = $e.attr(o.title == 'title' ? 'original-title' : o.title);
            } else if (typeof o.title == 'function') {
                title = o.title.call($e[0]);
            }
            title = ('' + title).replace(/(^\s*|\s*$)/, "");
            return title || o.fallback;
        },

        tip: function() {
            if (!this.$tip) {
                this.$tip = $('<div class="tipsy"></div>').html('<div class="tipsy-arrow"></div><div class="tipsy-inner"></div>');
            }
            return this.$tip;
        },

        validate: function() {
            if (!this.$element[0].parentNode) {
                this.hide();
                this.$element = null;
                this.options = null;
            }
        },

        enable: function() { this.enabled = true; },
        disable: function() { this.enabled = false; },
        toggleEnabled: function() { this.enabled = !this.enabled; }
    };

    $.fn.tipsy = function(options) {

        if (options === true) {
            return this.data('tipsy');
        } else if (typeof options == 'string') {
            var tipsy = this.data('tipsy');
            if (tipsy) tipsy[options]();
            return this;
        }

        options = $.extend({}, $.fn.tipsy.defaults, options);

        function get(ele) {
            var tipsy = $.data(ele, 'tipsy');
            if (!tipsy) {
                tipsy = new Tipsy(ele, $.fn.tipsy.elementOptions(ele, options));
                $.data(ele, 'tipsy', tipsy);
            }
            return tipsy;
        }

        function enter() {
            var tipsy = get(this);
            tipsy.hoverState = 'in';
            if (options.delayIn == 0) {
                tipsy.show();
            } else {
                tipsy.fixTitle();
                setTimeout(function() { if (tipsy.hoverState == 'in') tipsy.show(); }, options.delayIn);
            }
        };

        function leave() {
            var tipsy = get(this);
            tipsy.hoverState = 'out';
            if (options.delayOut == 0) {
                tipsy.hide();
            } else {
                setTimeout(function() { if (tipsy.hoverState == 'out') tipsy.hide(); }, options.delayOut);
            }
        };

        if (!options.live) this.each(function() { get(this); });

        if (options.trigger != 'manual') {
            var binder   = options.live ? 'live' : 'bind',
                eventIn  = options.trigger == 'hover' ? 'mouseenter' : 'focus',
                eventOut = options.trigger == 'hover' ? 'mouseleave' : 'blur';
            this[binder](eventIn, enter)[binder](eventOut, leave);
        }

        return this;

    };

    $.fn.tipsy.defaults = {
        className: null,
        delayIn: 0,
        delayOut: 0,
        fade: false,
        fallback: '',
        gravity: 'n',
        html: false,
        live: false,
        offset: 0,
        opacity: 0.8,
        title: 'title',
        trigger: 'hover'
    };

    // Overwrite this method to provide options on a per-element basis.
    // For example, you could store the gravity in a 'tipsy-gravity' attribute:
    // return $.extend({}, options, {gravity: $(ele).attr('tipsy-gravity') || 'n' });
    // (remember - do not modify 'options' in place!)
    $.fn.tipsy.elementOptions = function(ele, options) {
        return $.metadata ? $.extend({}, options, $(ele).metadata()) : options;
    };

    $.fn.tipsy.autoNS = function() {
        return $(this).offset().top > ($(document).scrollTop() + $(window).height() / 2) ? 's' : 'n';
    };

    $.fn.tipsy.autoWE = function() {
        return $(this).offset().left > ($(document).scrollLeft() + $(window).width() / 2) ? 'e' : 'w';
    };

    /**
     * yields a closure of the supplied parameters, producing a function that takes
     * no arguments and is suitable for use as an autogravity function like so:
     *
     * @param margin (int) - distance from the viewable region edge that an
     *        element should be before setting its tooltip's gravity to be away
     *        from that edge.
     * @param prefer (string, e.g. 'n', 'sw', 'w') - the direction to prefer
     *        if there are no viewable region edges effecting the tooltip's
     *        gravity. It will try to vary from this minimally, for example,
     *        if 'sw' is preferred and an element is near the right viewable
     *        region edge, but not the top edge, it will set the gravity for
     *        that element's tooltip to be 'se', preserving the southern
     *        component.
     */
     $.fn.tipsy.autoBounds = function(margin, prefer) {
        return function() {
            var dir = {ns: prefer[0], ew: (prefer.length > 1 ? prefer[1] : false)},
                boundTop = $(document).scrollTop() + margin,
                boundLeft = $(document).scrollLeft() + margin,
                $this = $(this);

            if ($this.offset().top < boundTop) dir.ns = 'n';
            if ($this.offset().left < boundLeft) dir.ew = 'w';
            if ($(window).width() + $(document).scrollLeft() - $this.offset().left < margin) dir.ew = 'e';
            if ($(window).height() + $(document).scrollTop() - $this.offset().top < margin) dir.ns = 's';

            return dir.ns + (dir.ew ? dir.ew : '');
        }
    };
(jQuery,document,this); })(jQuery, jQuery);



/*******************
 * From src: synergy/src/js/30.synergy.js
 *******************/
var updateTimer; // Holds setTimeout function that iterates update function.
var renameMap = [];

(function() {
/* Only available if using synergy */
if(_controller_name != 'synergy')
    return;

(function($) {
	
    var reg_o = /^\[([^0-9]\w*)\]/,
        reg_v = /^(\w+)/;

    function parse(str, val) {
        var split_val, ret_val;
        if (str.length === 0) {
            return val;
        }
        if (reg_o.test(str)) {
            return (ret_val = {})[(split_val = str.split(reg_o))[1]] = parse(split_val[2], val),ret_val;
        }
        if (reg_v.test(str)) {
            return (ret_val = {})[(split_val = str.split(reg_v))[1]] = parse(split_val[2], val),ret_val;
        }
    }

    $.ns = function(namespace, start) {
        if (!namespace) return;
        var parts = namespace.split('.');
        if (!parts.length) return;
        var head = start || window;
        var part = parts[0];
        for (var i = 0; i < parts.length; i++,part = parts[i]) {
            if (typeof(head[part]) === 'undefined') {
                head = (head[part] = {});
            } else {
                head = head[part];
            }
        }
        return head;
    };
    $.recontext = function(context, callback) {
        return function() {
            return callback.apply(context, arguments);
        };
    };
    $.objectKeys = function(obj) {
        if (!(obj instanceof Object)) return;
        var res = [];
        for (var indx in obj) {
            if (!obj.hasOwnProperty(indx)) continue;
            res.push(indx);
        }
        return res;
    };
    $.filterJson = function(list, rule, strict) {
        strict = strict || true;
        var item, rule_val, item_result, result = {};
        for (var indx in list) {
            if (list.hasOwnProperty(indx)) {
                item_result = true;
                item = list[indx];
                for (var rule_name in rule) {
                    if (rule.hasOwnProperty(rule_name)) {
                        if (!item.hasOwnProperty(rule_name)) {
                            item_result = false;
                            break;
                        }
                        rule_val = rule[rule_name];
                        if ($.isArray(rule_val)) {
                            if(-1 === $.inArray(item[rule_name], rule_val) ) {
                                item_result = false;
                                break;
                            }
                            continue;
                        }
                        if (typeof(rule_val) === "string") {
                            if (rule_val[0] === "~") {
                                if (-1 === item[rule_name].search(new RegExp(rule_val.substring(1), 'i'))) {
                                    item_result = false;
                                    break;
                                }
                                continue;
                            }
                        }
                        if (strict) {
                            if (rule_val !== item[rule_name]) {
                                item_result = false;
                                break;
                            }
                            continue;
                        }
                        if (rule_val != item[rule_name]) {
                            item_result = false;
                            break;
                        }
                    }
                }
                if (!item_result) {
                    continue;
                }
                result[indx] = list[indx];
            }
        }
        return result;
    };
    var __compare = {
        string: function(a, b) {
            if (a.value < b.value) return 1;
            if (a.value === b.value) return 0;
            return -1;
        },
        number: function(a, b) {
            return a.value - b.value;
        }
    };
    $.eachSort = function(data, key, func) {
        if(!key) return;
        var sort_type;
        var map = [];
        if ($.isEmpty(data)) return;
        $.each(data, function(k, v) {
            if (typeof(v[key]) === "undefined") return;
            if (typeof(sort_type) === "undefined") {
                sort_type = typeof(v[key]);
            } else {
                if (sort_type !== typeof(v[key])) {
                    console.error("error value type for sorting");
                    return;
                }
            }
            map.push({key: k, value: v[key]});
        });
        if (typeof(__compare[sort_type]) !== "function") {
            console.error("no compare function for "+sort_type+" type");
            return;
        }
        //map.sort(__compare[sort_type]);
        $.each(map, function(i, key) {
            func.call(data[key.key], key.key, data[key.key]);
        });
        return data;
    },
    $.unparam = function(str) {
        if(str.length === 0) { return {}; }
        var parts = str.split('&'),
            key_value,
            result = {};
        $.each(parts, function() {
            var key = decodeURIComponent((key_value = this.split('='))[0]),
                val = decodeURIComponent(key_value[1]);
            $.extend(true, result, parse(key, val));
        });
        return result;
    };
    $.tmpl = function(str, obj) {
        for(var i in obj) {
            if(!obj.hasOwnProperty(i)) continue;
            while(str.match("{{" + i + "}}")) {
                str = str.replace("{{" + i + "}}", obj[i]);
            }
        }
        return str;
    };
    $.isEmpty = function(ob) {
        if (!(ob instanceof Object)) return true;
        for(var i in ob) if(ob.hasOwnProperty(i)) return false;
        return true;
    };
    $.insertValues = function(parent, values) {
        $.each(values, function(k, v) {
            $(k, parent).html(v);
        });
    };
    $.fn.multiflexselect = function(opts) {
        var g_opts = { delimer: "&" };
        opts = $.extend(g_opts, opts);
        var cls = core.classes.Multiselect;
        $.each(this, function() {
           new cls(this, opts);
        });
    };
    $.fn.materialized = function(is_checked) {
        var cls = core.classes.Materialize;
        $.each(this, function() {
            new cls(this, is_checked);
        });
    };
    $.cloneObject = function(src) {
        if (!(src instanceof Object)) return src;
        var new_level = {}, name, val;
        for (key in src) {
            if(!src.hasOwnProperty(key)) continue;
            val = src[key];
            if(val instanceof Object) {
                new_level[key] = $.cloneObject(val);
            } else {
                new_level[key] = val;
            }
        }
        return new_level;
    };
    $.inObjectList = function(list, key, value) {
        if ($.isEmpty(list)) return false;
        for (var i in list) {
            if (!list.hasOwnProperty(i)) continue;
            if (typeof(list[i][key]) === "undefined") {
                console.error("no "+key+" key in object", list[i]);
                return false;
            }
            if (list[i][key] === value) return true;
        }
        return false;
    };
    $.timeDiffFormat = function(time, start_point) {
        if(!time) return "N/A";
        start_point = start_point || (new Date()).getTime();
        var diff_minutes = parseInt((start_point - time) / (1000 * 60), 10), minutes_part, hours_part;
        if(diff_minutes <= 0) return _("Less than a minute ago");
        var diff_hours = parseInt(diff_minutes / 60, 10);
        if(diff_hours <= 0) return diff_minutes + "m ago";
        minutes_part = parseInt(diff_minutes % 60, 10);
        minutes_part = minutes_part ? (" " + minutes_part + "m") : "";
        var diff_days = parseInt(diff_hours / 24, 10);
        if(diff_days <= 0) return diff_hours + "h" + minutes_part + " ago";
        hours_part = parseInt(diff_hours % 24, 10);
        hours_part = hours_part ? (" " + hours_part + "h") : "";
        return diff_days + "d" + hours_part + minutes_part + " ago";
    };
    var __pre_zero = function(num) {
        if (parseInt(num, 10) >= 10) return num;
        return "0" + num;
    };
    $.timeFormat = function(date) {
        if (parseInt(date, 10) === 0) {
            return _("N/A");
        }
        if(!(date instanceof Date)) {
            if (parseInt(date, 10)) {
                date = new Date(date);
            } else {
                date = new Date();
            }
        }
        var y = date.getFullYear(),
            m = __pre_zero(date.getMonth() + 1),
            d = __pre_zero(date.getDate()),
            h = __pre_zero(date.getHours()),
            min = __pre_zero(date.getMinutes()),
            s = __pre_zero(date.getSeconds());
        return  y+"-"+m+"-"+d+" "+h+":"+min+":"+s;
    };
    $.ns("core.events");
    $.ns("core.classes");
    core.__path =  "/";
})(jQuery);

// Multiselect Plugin
(function($) {
    var wrapper = $("<span class=\"multiselect-wrapper\"><span class=\"step-wrapper\"></span></span>"),
    Multiselect = function(node, opts) {
        this.node = $(node);
        this.wrapper = wrapper.clone();
        this.step_wrapper = $(".step-wrapper", this.wrapper);
        this.node.after(this.wrapper);
        this.delimer = opts.delimer;
        this.data = {};
        this.id = this.node.attr("id");
        this.name = this.node.attr("name");
        this.res = null;
        this.result = {};
        this.current_input = null;
        this.current_hidden = null;
        this.depth = 0;
        this.step = 0;
        this.init();
    };
    Multiselect.prototype = {
        init: function() {
            var that = this;
            var init_val;
            $("option", this.node).each(function() {
                if ($(this).attr("data-populate") === "1") {
                    init_val = $(this).val();
                }
                var line = $(this).val().split(that.delimer), i, head = that.data;
                that.depth = line.length;
                for (i = 0; i < line.length; i ++) {
                    if(!head[line[i]]) {
                        head[line[i]] = {};
                    }
                    head = head[line[i]];
                }
            });
            this.res = $(t = $.tmpl("<input name=\"{{n}}\" id=\"{{i}}\" type=\"hidden\"/>", {
                n: this.name,
                i: this.id
            })).appendTo(this.wrapper);
            this.node.detach();
            this.show_step(0, false);
            if (init_val) {
                var sv = init_val.split("&");
                for (var i = 0; i < sv.length; i++) {
                    this.step_selected(sv[i], false);
                }
            } else {
                this.res.find("input[type=hidden]").val(null);
            }

        },
       show_step: function(step, focus) {
            focus = focus === false ? false : true;
            if(step == this.depth) { return this.set_result(); }
            this.step = step;
            var new_select = "<select name=\"\" id=\""+this.id+"_step"+step+"\">", input;
            var head = this.data;
            for(var i = 0; i < this.step; i++) {
                head = head[this.result[i]];
            }
            for(var name in head) {
                new_select += "<option value=\""+name+"\">"+name+"</option>";
            }
            new_select += "</select>";
            this.wrapper.append(new_select);
            $("select", this.wrapper).flexselect();
            this.current_input = input = $("#" + this.id + "_step" + step + "_flexselect", this.wrapper);
            this.current_hidden = $("#" + this.id + "_step" + step, this.wrapper);
            this.current_input.addClass("ignore");
            this.current_hidden.addClass("ignore");
            input.val(null);
            if(focus) {
                input.trigger("focus");
            }
            input.on("item-selected", {w: this}, function(e) {
                e.data.w.step_selected($(this).val());
            });
        },
        step_selected: function(value, focus) {
            focus = focus === false ? false : true;
            this.result[this.step] = value;
            this.render_result();
            this.current_input.detach();
            this.current_hidden.detach();
            // $(".flexselect_dropdown").remove();
            this.show_step(this.step+1, focus);
        },
        render_result: function() {
            var indx, cur_selected;
            this.step_wrapper.empty();
            for(indx in this.result) {
                cur_selected = $("<span class=\"step-value\" data-step=\""+indx+"\">"+this.result[indx]+"</span>");
                this.step_wrapper.append(cur_selected);
                cur_selected.on("click", {w: this}, function(e) {
                    e.data.w.step_back($(this).attr("data-step"));
                });
            }
        },
        set_result: function() {
            var r = [], t;
            for(var indx in this.result) {
                r.push(this.result[indx]);
            }
            this.res.val(r.join("&"));
        },
        step_back: function(step) {
            step = parseInt(step, 10);
            this.step = step;
            var new_result = {}, new_value;
            for(var i = 0; i < step; i++) { new_result[i] = this.result[i]; }
            this.res.val(null);
            $("input", this.wrapper).not(this.res).remove();
            this.result = new_result;
            this.render_result();
            this.show_step(this.step);
        }
    };
    $.ns("core.classes").Multiselect = Multiselect;
})(jQuery);

// Materialize Plugin
(function($) {
    var Materialize = function(node) {
        this.node = $(node);
        var init_val = $("#materialized", this.node).val();
        this.is_checked = init_val.length !== 0;
        $("label[for=\"Materialized\"]", this.node).before("<input type=\"checkbox\" id=\"is-use\" class=\"is-use\"></input>").on("click", {w: this}, function(e) {
            e.data.w.change();
        });
        this.node.delegate(".is-use", "change", $.recontext(this, this.change));
        this.update(true);
        if (init_val) {
            $("#materialized_flexselect", this.node).val(init_val);
            setTimeout(function() { $("#materialized").val(init_val); }, 100);
        }
    };
    Materialize.prototype = {
        change: function() {
            this.is_checked = ! this.is_checked;
            this.update();
        },
        update: function(just_init) {
            if(this.is_checked) {
                this.node.removeClass("field-disabled");
                $("input[type=\"checkbox\"]", this.node).prop("checked", true);
                var fs = $(".flexselect", this.node).attr("disabled", null);
                if (!just_init) {
                    fs.select();
                }
            } else {
                $("input[type=\"checkbox\"]", this.node).prop("checked", false);
                this.node.addClass("field-disabled");
                $(".flexselect", this.node).attr("disabled", "disabled").clear_flexselect();
            }
        }
    };
    $.ns("core.classes").Materialize = Materialize;
})(jQuery);

// DataWrapper
(function($) {
    var syn_data;
    var __is_changed = false;
    var changed = function() {
        $("#sync-button").addClass("active");
        $("#undo-button").addClass("active");
        $("#notification-board").html(_("You have unsaved configuration changes. Please press SAVE to activate them.")).attr("data-status", "WARNING");
        __is_changed = true;
    };
    var unchange = function() {
        __is_changed = false;
    };
    var initial_data, list_data;
    var deleted_list = [];
    var updated_list = [];
    var _toList = function(res, url, d) {
        url = url || "";
        result = res || {};
        var indx;
        var new_url;
        d = d || syn_data.items;
        for(indx in d) {
            if(!d.hasOwnProperty(indx)) continue;
            new_url = url + (url.length ? "/" : "") + indx;
            result[new_url] = d[indx];
            result[new_url]["url"] = new_url;
            if(typeof(d[indx]["items"]) === "undefined") continue;
            _toList(result, new_url, d[indx]["items"]);
        }
        return result;
    },
    each_sub_element = function(p, func) {
        var r = new RegExp(("^" + p).replace(/\//g, "\\/"));
        $.each(list_data, function(k, v) {
            if (r.test(k)) {
                func.call(this, k, v);
            }
        });
    };
    var _getMaterializedList = function(data, path) {
        if (data.name) {
            path = (path || "") + "/" + data.name;
        } else {
            path = "";
        }
        res = [];
        if (data.materialized) {
            var v = data.materialized.host + "&" + data.name;
            if ($.inArray(v, res) === -1) {
                res.push({value: v, path: path});
            }
        }
        if (data.items) {
            $.each(data.items, function(k,v) {
                res = res.concat(_getMaterializedList(v, path));
            });
        }
        return res;
    };
    /**
     * Gets differences between materialized nodes at initiation of
     * page and saving of configuration.
     */
    var _getDiff = function() {
        var cur = _getMaterializedList(syn_data), add = [];
            init = _getMaterializedList(initial_data), del = [];
        var rename = [];
        $.each(cur, function(k, v) {
            if (!$.inObjectList(init, "value", v.value)) {
                if ($.inObjectList(renameMap, "new_name", v.value)) {
                    $.each(renameMap, function(x, y) {
                        if (v.value === y.new_name) {
                            var service = v.value.split("&");
                            service.push(y.old_name);
                            rename.push({service: service, path: v.path.replace(/^\/+/, "") });
                        }
                    });
                } else {
                    add.push({service: v.value.split("&"), path: v.path.replace(/^\/+/, "")});
                }
            }
        });
        $.each(init, function(k, v) {
            if (!$.inObjectList(cur, "value", v.value)) {
                var service = v.value.split("&");
                if (!$.inObjectList(renameMap, "old_name", service[1])) {
                    del.push({service: service, path: v.path.replace(/^\/+/, "")});
                }
            }
        });
        return {"add": JSON.stringify(add), "del": JSON.stringify(del), "rename": JSON.stringify(rename)};
    };
    var _toSaveObj = function(data) {
        data = data || syn_data;
        if (typeof(data["type"]) === "undefined") {
            console.error("Error object in queue. Ignore:");
            console.dir(data);
            return false;
        }

        var res = {};
        res["name"] = data["name"] || "no-name";
        res["intent"] = data["type"] + "." + data["rule"] + "(";
        if (data["materialized"]) {
            res["materialized"] = data["materialized"];
        }
        var args = []; $.each(data.args || [], function(i) {
            if(actions[data["rule"]].params[i].type == "number") {
                args.push(this);
            } else {
                args.push("\"" + this + "\"");
            }
        });
        res["intent"] += args.join(",") + ")";
        if(!$.isEmpty(data.items)) {
            res["components"] = [];
            $.each(data.items, function() {
                var r = _toSaveObj(this);
                if (r !== false) {
                    res["components"].push(r);
                }
            });
        }
        return res;
    };
    var _updateListUpdateByPath = function(p) {
        var rule_edited = p.split("/")[0];
        if ($.inArray(rule_edited, updated_list) === -1) {
            updated_list.push(rule_edited);
        }
    };
    var _getChangesList = function() {
        var del_list = [];
        $.each(initial_data.items, function(k,v) {
            if(!syn_data.items[k]) {
                del_list.push(k);
            }
        });
        $.each(del_list, function(k,v) {
            var l = $.inArray(v, updated_list);
            if (l !== -1) {
                updated_list.splice(l,1);
            }
        });
        var res = {};
        $.each(updated_list, function(k,v) {
            if (typeof(list_data[v]) !== "undefined" ) {
                res[v] = _toSaveObj(syn_data.items[v]);
            }
        });
        return {update_list: JSON.stringify(res), del_list: del_list};
    };
    var DataWrapper = function() {
        $($.recontext(this, this.load));
        window.onbeforeunload = $.recontext(this, this.quit);
    };

    var get_clone_items = function(parent) {
        if (!parent.items) return;
        var res = {};
        $.each(parent.items, function(k, v) {
            res[k] = {};
            $.each(v, function(sk, sv) {
                if (sk === "items") {
                    return;
                }
                res[k][sk] = $.cloneObject(sv);
            });
        });
        return res;
    };
    var __lastUpdate = 1; // No set this value to 0. 0 - is incorrect result on intval php function
    var __in_update = false;
    var __update = function(statrt_new) {
        if (__in_update) {
            return;
        }
        if (core.Data.isEditing()) {
            if (statrt_new) {
                __startUpdate();
            }
            return;
        }
        __in_update = true;
        $.ajax({
        // @todo clean this up, use GET
            type: "POST",
            data: {
                last_updated: __lastUpdate,
                csrf_token: _csrf_token
            },
            dataType: "json",
            url: core.base_url + "/synergy/update",
            complete: function() {
                __in_update = false;
                if (statrt_new) {
                    __startUpdate();
                }
            },
            success: function(res_data) {
            // @todo clean this up, use HTTP headers, because json object (res_data) might be null
                switch(parseInt(res_data.status, 10)) {
                    case 200:
                        if (core.Data.isEditing()) break;
                        try {
                            var new_json = res_data.data.tree;
                            syn_data = new_json;
                        } catch(err) {
                            break;
                        }
                        __lastUpdate = res_data.data.updated;
                        core.Data.update_time();
                        initial_data = $.cloneObject(syn_data);
                        list_data = _toList();
                        $($.ns("core.events.DataWrapper")).triggerHandler("data-update");
                        break;
                    case 304:
                        // No changes in synergy
                        break;
                    case 503:
                        // TODO: fire error message
                        console.error(res_data.data);
                        break;
                }
            }
        });
    };
    // Function that constantly updates page. Restarted everytime the update runs and page is not being updated.
    var __startUpdate = function() {
        core.refresh_rate = parseInt($("#view-port").attr("data-refresh-rate"), 10) || 2000;
        updateTimer = setTimeout(function() { __update(true); }, core.refresh_rate);
    };
    // Pause the update function after a save has been made. Makes sure old status information is not shown.
    var __pauseUpdate = function() {
        var dateNow = new Date();
        var timeNow = Math.round(dateNow.getTime() / 1000);
        __lastUpdate = timeNow + 10;
        clearTimeout(updateTimer);
        setTimeout(function() { __update(true); }, 10000);
    };
    var a = true;
    DataWrapper.prototype = {
        update: function() {
            __update(false);
        },
        update_time: function() {
            $("#page_last_update").text($.timeFormat(__lastUpdate));
        },
        load: function() {
            syn_data = $.cloneObject(synergy_data);
            synergy_data = undefined;
            initial_data = $.cloneObject(syn_data);
            list_data = _toList();
            __lastUpdate = parseInt($("#view-port").attr("data-timestamp"), 10);
            __startUpdate();
            $("#sync-button").on("click", {w: this}, function(e) {
                if ($(this).hasClass("active")) {
                    core.singeltons.AjaxWait.show();
                    e.data.w.sync($.recontext(e.data.w, DataWrapper.prototype.sync_ready));
                }
            });
            $("#undo-button").on("click", {w: this}, function(e) {
                if ($(this).hasClass("active")) {
                    e.data.w.undo();
                }
            });
        },
        getServerTimestamp: function() {
            return __lastUpdate;
        },
        quit: function() {
            if (this.isEditing()) {
                return _("If you leave without saving, all changes will be thrown away. Do you still want to leave this page without saving?");
            }
        },
        toList: function() {
            if (typeof(list_data) !== "undefined") {
                return list_data;
            }
            return list_data = _toList();
        },
        isServiceExist: function(service, callback) {
            $.ajax({
                data: {service: service},
                dataType: "json",
                type: "GET",
                url: _site_domain + _index_page + "/synergy/check",
                success: function(res) {
                    if (res.status === 200) {
                        callback(res.service_exists);
                    } else {
                        console.error("Check command fail: ", arguments);
                    }
                },
                error: function() {
                    console.error("XHR request fail: ", arguments);
                }
            });
        },
        isTopLevelExist: function(rule, async, callback) {
            var result;
            async = async ? true : false; // to boolean
            $.ajax({
                async: async || false,
                data: {rule: rule},
                dataType: "json",
                type: "GET",
                url: _site_domain + _index_page + "/synergy/rule_check",
                success: function(res) {
                    if (res.status === 200) {
                        if ((async === true) && (typeof(callback) === "function")) {
                            callback(res.data);
                        } else {
                            result = res.data;
                        }
                    } else {
                        console.error(res.data);
                    }
                },
                error: function() {
                    console.error("XHR request fail: ", arguments);
                }
            });
            if (async === false) {
                return result;
            }
        },
        get: function(url, flat) {
            if(!url) {
                if (flat) {
                    return get_clone_items(syn_data);
                } else {
                    return $.extend({}, syn_data);
                }
            }
            url = url.replace(/^\/+/, "");
            if (typeof(list_data[url]) === "undefined") {
                return false;
            }
            if (flat) {
                return get_clone_items(list_data[url]);
            }
            return $.extend({}, list_data[url]);
        },
        getParentUrl: function(url) {
            var parts = url.split("/");
            parts.pop();
            while(typeof(list_data[parts.join("/")]) === "undefined") {
                if (typeof(parts.pop()) === "undefined") {
                    return false;
                }
            }
            return parts.join("/");
        },
        getParent: function(url) {
            return list_data[this.getParentUrl(url)];
        },
        hasItems: function(p) {
            if (!p || typeof p === "undefined") return !$.isEmpty(syn_data.items);
            var split_path = p.split("/");
            var look_for = split_path[split_path.length -1];
            var current_tree = syn_data.items;
            while(le_parent = split_path.shift()) {
                if(!split_path.length) {
                    return le_parent in current_tree && "items" in current_tree[le_parent];
                }
                if(typeof current_tree[le_parent] === "undefined" || !("items" in current_tree[le_parent])) {
                    return false;
                }
                current_tree = current_tree[le_parent].items;
            }
        },
        isItemExist: function(p) {
            if (!p) return false;
            if (p === "/") return true;
            p = p.replace(/^\/+/, "");
            return (typeof(list_data[p]) !== "undefined");
        },
        add: function(data, p) {
            p = p || core.url.getCurrent();
            _updateListUpdateByPath(p);
            head = list_data[p];
            if($.isEmpty(head)) { console.error("No path: " + p); return; }
            head.items = head.items || {};
            head.items[data.name] = data;
            list_data[p + "/" + data.name] = head.items[data.name];
            changed();
        },
        add_rule: function(data) {
            _updateListUpdateByPath(data.name);
            syn_data.items[data.name] = data;
            list_data[data.name] = data;
            changed();
        },
        del: function(path) {
            if(!path) return;
            _updateListUpdateByPath(path);
            var obj = this.getParent(path);
            var e = this.get(path);
            if(typeof(obj.items[e.name]) === "undefined") return;
            delete obj.items[e.name];
            delete list_data[path];
            each_sub_element(path, function(k, v) {
                delete list_data[k];
            });
            changed();
        },
        del_rule: function(rule) {
            var i;
            delete syn_data.items[rule];
            delete list_data[rule];
            each_sub_element(rule, function(k, v) {
                delete list_data[k];
            });
            changed();
        },
        clone_rule: function(rule, callback) {
            if(!syn_data.items[rule]) return;
            var new_rule = $.cloneObject(syn_data.items[rule]);
            while(syn_data.items[new_rule["name"]]) {
                new_rule.name += "_clone";
            }
            syn_data.items[new_rule["name"]] = new_rule;
            list_data = _toList();
            _updateListUpdateByPath(new_rule.name);
            changed();
            if (callback) {
                callback(new_rule);
            }
        },
        edit: function(data, old_name, p) {
            p = p || core.url.getAddr();
            _updateListUpdateByPath(p);
            head = list_data[p];
            if($.isEmpty(head)) { console.error("No path: " + p); return; }
            data.items = head.items[old_name].items;
            delete head.items[old_name];
            delete list_data[p + "/" + old_name];
            head.items[data.name] = data;
            list_data = _toList();
            changed();
        },
        edit_rule: function(data, old_name) {
            _updateListUpdateByPath(data.name);
            if(!syn_data.items[old_name]) return;
            data.items = syn_data.items[old_name].items;
            if (old_name !== data.name && typeof(data.materialized) !== "undefined" && typeof(syn_data.items[old_name].materialized) !== "undefined") {
                if (syn_data.items[old_name].materialized.host === data.materialized.host) {
                    renameMap.push({old_name: old_name, new_name: data.materialized.host + "&" + data.name});
                }
            }
            delete syn_data.items[old_name];
            delete list_data[old_name];
            syn_data.items[data.name] = data;
            list_data = _toList();
            changed();
        },
        sync: function(callback) {
            if (core.auth_edit === false) return;
            // Do not! I repeat: Do not! update gui while sync is running.
            clearTimeout(updateTimer);
            var req_data = $.extend(_getChangesList(), _getDiff());
            req_data.csrf_token = _csrf_token;
            $.ajax({
                data: req_data,
                dataType: "json",
                type: "POST",
                url: _site_domain + _index_page + "/synergy/sync",
                complete: callback,
                error: function(xhr, type, msg) {
                    $("#notification-board").attr("data-status", "CRITICAL");
                    if (xhr.status === 401) {
                        $("#notification-board").html(_("Access denied"));
                    } else {
                        $("#notification-board").html(_("Could not save changes, please try again"));
                    }
                    setTimeout(function() {
                        $("#notification-board").attr("data-status", null).empty();
                        core.Data.undo();
                    }, 2500);
                }
            });
        },
        isEditing: function() {
            return __is_changed;
        },
        sync_ready: function(req) {
            __pauseUpdate();
            core.singeltons.AjaxWait.hide();
            initial_data = $.cloneObject(syn_data);
            list_data = _toList();
            $("#sync-button").removeClass("active");
            $("#undo-button").removeClass("active");
            $("#notification-board").attr("data-status", "OK").html(_("Your changes are saved!"));
            unchange();
            setTimeout(function() {
                $("#notification-board").attr("data-status", null).empty();
            }, 2500);
        },
        undo: function() {
            syn_data = null;
            syn_data = $.cloneObject(initial_data);
            list_data = _toList();
            $("#notification-board").attr("data-status", null).empty();
            $("#sync-button").removeClass("active");
            $("#undo-button").removeClass("active");
            core.url.setAddr(core.url.getCurrent(), true);
            unchange();
        }
    };
    $.ns("core").Data = new DataWrapper();
})(jQuery);

//Popup for terminal elements
(function($) {
    var autoWE = function() { return $(this).offset().left > ($(window).width()*0.75) ? 'e' : 'w'; },
        event_ns = $.ns("core.events.TipsyListener"), current, cur_el, just_opened = false,
        TipsyListener = function() {
            $($.recontext(this, this.init));
            $($.ns("core.events.ContentLoader")).on("load-page", $.recontext(this, this.hide));
            $(event_ns).on("show", $.recontext(this, this.show));
            $(event_ns).on("hide", $.recontext(this, this.hide));
        };
    TipsyListener.prototype = {
        init: function() {
            $(document.body).on("click", $.recontext(this, this.hide));
        },
        show: function(e, el, info) {
            if(current) {
                this.hide();
            }
            if(cur_el === el) { cur_el = null; return; }
            cur_el = el;
            current = $(el).tipsy({ trigger: 'manual', fallback: info, html: true, gravity: autoWE, opacity: 1 });
            current.tipsy("show");
            just_opened = true;
            setTimeout(function() { just_opened = false; }, 50);
            $("#terminal-element-popup").on("click", function(e) { e.stopPropagation(); });
        },
        hide: function() {
            if(just_opened) return;
            if(current) {
                current.tipsy("hide");
            }
        }
    };

    $.ns("core.singeltons").TipsyListener = new TipsyListener();
})(jQuery);

// Content Loader
(function($) {
    var element_html = "<div data-url=\"%%URL%%\" class=\"content-element %%TYPE%%\" type=\"%%OBJ%%\"><div class=\"content-element-head\"><span class=\"element-status-text\"></span></div><div class=\"content-element-body\"><span class=\"delete-element\"></span><span class=\"edit-element\"></span><span class=\"content-element-icon\"></span><span class=\"content-element-name\">%%NAME%%</span></div><div class=\"content-element-msg\">%%MSG%%</div><div class=\"clear\"></div></div></div>",
        event_ns = $.ns("core.events.ContentLoader"),
        cont_tmpl = "<b class=\"italic\">\"{{v}}\"</b></br>",
        pages = {},
        tmp = false,
        _dataSeparate = function(data) {
            var result = [{}, {}, {}, {}, {}];
            $.each(data || [], function(k, v) {
                if(v.type != "nagios") {
                    return result[0][k] = v;
                }
                if(v.rule == "host_group_up" || v.rule == "host_group_unfold") {
                    return result[1][k] = v;
                }
                if(v.rule == "service_group_unfold") {
                    return result[2][k] = v;
                }
                if(v.rule == "vhost_up" || v.rule == "host") {
                    return result[3][k] = v;
                }
                return result[4][k] = v;
            });

            return result;
        };
    var ContentLoader = function(node) {
        this.node = $(node);
        $(event_ns).on("load-page", {w: this}, function(e, p, m) {
            if(m == "flat") {
                e.data.w.show();
                e.data.w.load_page(e, p);
            } else {
                e.data.w.hide();
            }
        });
        $(function(){ tmp = $("#display").children(); });
        $($.ns("core.events.NodeFacet")).on("facet-change", $.recontext(this, this.render));
        $($.ns("core.events.NewNodeDialog")).on("node-add", $.recontext(this, this.render));
    };
    ContentLoader.prototype = {
        load_page: function(e, page) {
            var data = core.Data.get(page);
            if (!data) return;
            this.data = data;
            this.page = page;
            this.render(null, null, page);
        },
        render: function(e, filter, page) {
            if (core.url.getMode() !== "flat") return;
            page = page || core.__path;
            filter = filter || false;
            $(".content-element", this.node).remove();
            var filtered_data = core.Data.get(page).items;
            if(filter) {
                filtered_data = $.filterJson(this.data.items, filter);
            }
            var after = $("#nodes-new-element", this.node);
            var sd  = _dataSeparate(filtered_data);
            for(var i = 0; i < sd.length; i++) {
                var that = this;
                $.eachSort(sd[i], "name", function(k, v) {
                    var cont = "";
                    var type = "unknown";
                    if(v.status) {
                        type = v.status.toLowerCase();
                    }
                    obj = "folder";
                    if(v.type === "nagios") obj = v.rule;
                    cont = $.tmpl(cont_tmpl, {v: v.result ? (v.result.msg.length > 58 ? v.result.msg.substring(0,58) + "...": v.result.msg) : _("no result yet.")});
                    var node = $(element_html.replace("%%URL%%", encodeURIComponent(that.page + "/" + v.name))
                        .replace("%%NAME%%", v.name)
                        .replace("%%TYPE%%", type)
                        .replace("%%OBJ%%", obj)
                        .replace("%%MSG%%", cont));
                    $(".element-status-text", node).html(v.status);
                    if (v.type === "rule") {
                        node.click(function() {
                            core.url.setAddr("/" + decodeURIComponent($(this).attr("data-url")));
                        });
                    }
                    if(actions[v.rule].links) {
                        var links = actions[v.rule].links;
                        if(links.length) {
                            var popup_html = "<div id=\"terminal-element-popup\"><b class=\"bold\">More information:</b></br>";
                            $.each(links, function() {
                                var href = this.href;
                                $.each(actions[v.rule].params, function(i) {
                                    if(this.type == "nagios_service") {
                                        href = href.replace("{{nagios_service.name}}", v.args[i].split("&")[0]).replace("{{nagios_service.description}}", v.args[i].split("&")[1]);
                                    } else {
                                        href = href.replace("{{" + this.name + "}}", v.args[i]);
                                    }
                                });
                                popup_html += "<a href=\""+ core.base_url + href+"\">"+this.title+"</a><br />";
                            });
                            popup_html += "</div>";
                            node.on("click", {html: popup_html}, function(e) {
                                $(core.events.TipsyListener).triggerHandler("show", [this, e.data.html]);
                            });
                        }
                    }
                    if(core.mode == 1) {
                        that.node.append(node);
                        $(".delete-element").hide();
                        $(".edit-element").hide();
                        $(".content-element-name").attr("style", "padding-top:16px");
                    } else {
                        after.before(node);
                    }
                });
            }

            $(".delete-element").on("click", {w: this}, function(e) {
                if(confirm("Are you sure want to delete this "+$(this).parents(".content-element").attr("type")+"?") === true)
                    e.data.w.delete_element(e, decodeURIComponent($(this).parents(".content-element").attr("data-url")), this);
                e.stopPropagation();
            });
            $(".edit-element").on("click", {w: this}, function(e) {
                e.data.w.edit_element(e, decodeURIComponent($(this).parents(".content-element").attr("data-url")), this);
                e.stopPropagation();
            });
            this.node.append($("<div class=\"clear\"></div>"));
        },
        delete_element: function(e, url, el) {
            core.Data.del(url);
            $(el).parents(".content-element").remove();
        },
        edit_element: function(e, url) {
            var el = core.Data.get(url);
            $(core.events.ModalWindow).triggerHandler("show", [$("#modal-windows #new-node-win")]);
            $(core.events.NewNodeDialog).triggerHandler("show-dialog", [el.rule, el]);
        },
        show: function() {
            $("#display").append(tmp);
        },
        hide: function() {
            if (tmp) {
                tmp = tmp.detach();
            }
        }
    };
    $.ns("core.classes").ContentLoader = ContentLoader;
})(jQuery);

// Scroll
(function($) {
    var event_ns = $.ns("core.events.Scroll");
    var wrapper_html = "<div></div>";
    var elem_html = "<div class=\"slider-element {{sl}}\" data-url=\"{{n}}\"><div class=\"slider-element-header\">{{su}}</div><div class=\"slider-element-body\">{{n2}}</div></div></div>";
    var scroll_step = 200;
    var Scroll = function(node) {
        this.node = $(node);
        this.left = 0;
        this.render_elements(this.node);
        $(event_ns).on("move-left", $.recontext(this, this.move_left));
        $(event_ns).on("move-right", $.recontext(this, this.move_right));
        $($.ns("core.events.NewNodeDialog")).on("rule-add", {w:this}, function(e) {
            e.data.w.render_elements(e.data.w.node);
        });
        this.select_current(core.__path.split("/")[0]);
    };
    Scroll.prototype = {
        render_elements: function(node) {
            node.children().remove();
            var data = core.Data.get(), that = this;
            $.each(data.items, function(k, v) {
                node.prepend($.tmpl(elem_html, {
                    "sl": v.status.toLowerCase(),
                    "su": v.status.toUpperCase(),
                    "n": encodeURIComponent(k),
                    "n2": k
                }));
            });
            node.css({ position: "relative", overflow: "hidden" });
            this.sub_wrapper = $(wrapper_html).append(node.children().remove()).css("float", "left");
            this.wrapper = $(wrapper_html).append(this.sub_wrapper).appendTo(node).css({position: "absolute", "top": 0, "left": 0, "width": "10000px", "clear": "both"});
            if(core.mode == 2) {
                $(this.sub_wrapper).append("<div class=\"add-new-rule\" id=\"add-new-rule\"></div>");
                document.getElementById("add-new-rule").onclick = function() {
                    $(core.events.ModalWindow).triggerHandler("show", [$("#modal-windows #new-node-win"), true]);
                };
            }
            $(".slider-element").each(function() {
                this.onclick = function() {
                    core.url.setAddr("/" + decodeURIComponent($(this).attr("data-url")), true);
                };
            });
            $($.ns("core.events.ContentLoader")).on("load-page", {w:this}, function(e, url) {
                e.data.w.select_current(url.split("/")[0]);
            });
            this.select_current(core.__path.split("/")[0]);
        },
        move_right: function() {
            var diff = (this.sub_wrapper.width() + this.left) - this.node.width();
            if (diff > 0) {
                var step = 0;
                if (diff >= scroll_step) {
                    step = scroll_step;
                } else {
                    step = diff;
                }
                this.left -= step;
                this.wrapper.animate({
                    left: "-=" + step
                }, 250);
                //this.wrapper.css("left", this.left + "px");
            } else {
                $(event_ns).triggerHandler("no-right", [this]);
            }
        },
        move_left: function() {
            if (this.left !== 0) {
                var step = 0;
                if ((-this.left) >= scroll_step) {
                    step = scroll_step;
                } else {
                    step = -this.left;
                }
                this.left += step;
                this.wrapper.animate({
                    left: "+=" + step
                }, 250);
                //this.wrapper.css("left", this.left + "px");
            } else {
                $(event_ns).triggerHandler("no-left", [this]);
            }
        },
        select_current: function(current_rule) {
            $(".slider-element").removeClass("selected");
            $(".slider-element[data-url=\""+encodeURIComponent(current_rule)+"\"    ]").addClass("selected");
        }
    };
    $.ns("core.classes").Scroll = Scroll;
})(jQuery);

// BreadCrumbs
(function($) {
    var element_html = "<div class=\"breadcrumbs-element %%TYPE%%\" data-url=\"%%URL%%\"></div>";
    var BreadCrumbs = function(node) {
        this.node = $(node);
        $($.ns("core.events.ContentLoader")).on("load-page", $.recontext(this, this.render));
    };
    BreadCrumbs.prototype = {
        render: function(e, addr) {
            if(core.url.getMode() == "tree") return;
            addr = core.url.getAddr();
            window.core.__path = decodeURIComponent(addr);
            var pieces = addr.split('/'), list_data = core.Data.toList();
            this.node.empty();
            var url = "";
            for (var i = 0; i < pieces.length; i++) {
                url += pieces[i] + "/";
                var type = list_data[url.slice(0, -1)]["status"].toLowerCase();
                var el = $((element_html + "").replace("%%URL%%", encodeURIComponent(url.slice(0, -1))).replace("%%TYPE%%", type));
                if (i + 1 == pieces.length) {
                    el.addClass("last").html(pieces[i]);
                } else {
                    el.html(pieces[i]);
                    el.click($.recontext(el, this.select));
                }
                this.node.append(el);
            }
            this.node.append($("<div class=\"clear\"></div>"));
        },
        // this -> .breadcrumbs-element
        select: function() {
            core.url.setAddr("/" + decodeURIComponent(this.attr("data-url")));
        }
    };
    $.ns("core.classes").BreadCrumbs = BreadCrumbs;
})(jQuery);

// CurrentNode
(function($) {
    var cont_tmpl = "<b class=\"bold\">{{k}}</b>: {{v}}</br>";
    var CurrentNode = function(node) {
        this.node = $(node);
        $($.ns("core.events.ContentLoader")).on("load-page", $.recontext(this, this.render));
    };
    CurrentNode.prototype = {
        render: function(e, addr) {
            if(core.url.getMode() == "tree") return;
            var data = core.Data.get(addr), last_check = last_state_change = _("No result yet");
            var cont, conf = "";
            if(core.mode == 2) {
                conf = "<span class=\"delete-node-link\" data-url=\""+encodeURIComponent(core.__path)+"\"></span>";
                conf += "<span class=\"edit-node-link\" data-url=\""+encodeURIComponent(core.__path)+"\"></span>";
                if(core.__path.split("/").length === 1) {
                    conf += "<span class=\"clone-node-link\" data-url=\""+encodeURIComponent(core.__path)+"\">clone</span>";
                }
            }
            $("#current-head",this.node).html("<span class=\"current-top-node-icon\"></span><span id=\"current-top-node-name\">"+data.name+"</span>"+conf+"<div class=\"clear\"></div>");
            this.node.attr("data-type", data.status.toLowerCase());
            cont = $.tmpl(cont_tmpl, {v: data.status+"<span id =\"status-shild\"></span>", k: "status" });
            cont += $.tmpl(cont_tmpl, {v: actions[data.rule].name, k: "rule"});
            $.each(actions[data.rule].params, function(i) {
                cont += $.tmpl(cont_tmpl, {v: data.args[i], k: this.name});
            });
            if(data.result) {
                last_check = new Date(data.result.last_check *1000 );
                last_check = last_check.toLocaleTimeString()+' '+last_check.toDateString();
                last_state_change =  new Date(data.result.last_state_change * 1000);
                last_state_change = last_state_change.toLocaleTimeString()+' '+last_state_change.toDateString();
            }
            cont += $.tmpl(cont_tmpl, {v: data.result ? data.result.msg : _("No result yet."), k: "message"});
            cont += $.tmpl(cont_tmpl, {v: last_check, k: _("Last check")} );
            cont += $.tmpl(cont_tmpl, {v: last_state_change, k: _("Last state change") } );
            $("#current-body", this.node).html(cont);
            $(".delete-node-link", this.node).click(function() {
                var rule = decodeURIComponent($(this).attr("data-url"));
                if(confirm(_("Are you sure want to delete '"+rule+"'?")) === false) {
                    return;
                }
                core.Data.del_rule(rule, function() {
                    core.url.setAddr("/");
                    $(core.events.NewNodeDialog).triggerHandler("rule-add");
                });
            });
            $(".edit-node-link").click(function() {
                el = core.Data.get(core.__path);
                $(core.events.ModalWindow).triggerHandler("show", [$("#modal-windows #new-node-win"), "NewNodeDialog", true]);
                $(core.events.NewNodeDialog).triggerHandler("show-dialog", [el.rule, el]);
            });
            $(".clone-node-link").click(function() {
                core.Data.clone_rule(decodeURIComponent($(this).attr("data-url")), function() {
                    core.url.setAddr(this.new_rule_name);
                    $(core.events.NewNodeDialog).triggerHandler("rule-add");
                    $(".edit-node-link").triggerHandler("click");
                }, this);
            });
        }
    };
    $.ns("core.classes").CurrentNode = CurrentNode;
})(jQuery);

// Node Facet
(function($) {
    var event_ns = $.ns("core.events.NodeFacet");
    var reg_exp = /[\w\d_]+/i;
    var NodeFacet = function(node) {
        this.node = $(node);
        this.filter = {};
        this.check_buttons = $(".facet-element-toggle", this.node);
        this.quick_search = $("#quick-filter", this.node);
        this.quick_search.on("change", $.recontext(this, this.text_change));
        this.quick_search.on("keyup", $.recontext(this, this.text_change));
        this.check_buttons.on("change", $.recontext(this, this.radio_change));
        this.check_buttons.click(function() { if($(this).hasClass("toggle")) { $(this).removeClass("toggle"); } else { $(this).addClass("toggle"); } $(this).triggerHandler("change"); } );
    };
    NodeFacet.prototype = {
        text_change: function() {
            var val = this.quick_search.val();
            if(!reg_exp.test(val)) {
                delete this.filter["name"];
            } else {
                this.filter["name"] = "~"+val;
            }
            $(event_ns).triggerHandler("facet-change", [this.filter]);
        },
        radio_change: function() {
            var values = [];
            $.each(this.check_buttons.filter(function() { return $(this).hasClass("toggle"); }), function() { values.push($(this).attr("data-type")); });
            if(values.length === 3) {
                delete this.filter["status"];
            } else {
                this.filter["status"] = values;
            }
            $(event_ns).triggerHandler("facet-change", [this.filter]);
        }
    };
    $.ns("core.classes").NodeFacet = NodeFacet;
})(jQuery);

// Live Search
(function($) {
    var reg_exp = /[\w\d_]+/i;
    var element_html = "<div class=\"search-result {{s}}\" data-url=\"{{e}}\"><span class=\"result-name\">{{n}}</span><span class=\"result-url\">{{u}}</span><span class=\"result-status {{t}}\">{{s}}</span></div>";
    var max_result = 5,
        open_link = function(e) {
            var url = decodeURIComponent($(this).attr("data-url")), eu = $(this).attr("data-url");
            if (!core.Data.hasItems(url)) {
                url = core.Data.getParentUrl(url);
            }
            core.url.setAddr(url, true);
            $('html, body').animate({
                scrollTop: $(".tree-wrapper [data-url=\""+eu+"\"]").offset().top
            }, 1000);
            e.data.w.hide_popup();
            e.stopPropagation();
        };
    var LiveSearch = function(node) {
        this.node = $(node).click(function(e) { e.stopPropagation(); });
        this.popup = $("<div></div>");
        this.popup.css({
            position: "absolute",
            top: 35,
            right: "4px",
            "border": "1px solid #D0D0D0",
            "z-index": "100",
            display: "none"
        }).appendTo(this.node.parent());
        this.node.on("keyup", $.recontext(this, this.search));
        this.node.on("change", $.recontext(this, this.search));
        this.node.on("focus", $.recontext(this, this.search));
        $("body").on("click", $.recontext(this, this.hide_popup));
    };
    LiveSearch.prototype = {
        search: function(e) {
            var val = this.node.val();
            if(reg_exp.test(val)) {
                this.show_popup($.filterJson(core.Data.toList(), { name: "~" + val }), val);
            } else {
                this.hide_popup();
            }
        },
        show_popup: function(list, search_val) {
            var scores_list = [], count = 0, that = this;
            $.each($.objectKeys(list) || {}, function(k, v) {
                if (count === 5) return;
                scores_list.push({key: v, score: LiquidMetal.score(key, search_val)});
                count++;
            });
            if (!scores_list.length) return this.hide_popup();
            this.popup.empty();

            $.eachSort(scores_list, "score", function(k, v) {
                v = list[v.key];
                that.popup.append($($.tmpl(element_html, {
                    n: v.name,
                    u: v.url,
                    e: encodeURIComponent(v.url),
                    t: v.status.toLowerCase(),
                    s: v.status
                })).on("click", {w: that}, open_link));
            });
            this.popup.show();
        },
        hide_popup: function() {
            this.popup.empty();
            this.popup.hide();
        }
    };
    $.ns("core.classes").LiveSearch = LiveSearch;
})(jQuery);

// Modal Window
(function($) {
    var ModalWindow = function() {
        $($.recontext(this, this.init));
        $($.ns("core.events.ModalWindow")).on("hide", $.recontext(this, this.hide));
        $($.ns("core.events.ModalWindow")).on("show", $.recontext(this, this.show));
        $($.ns("core.events.NewNodeDialog")).on("show-dialog", function() {
            setTimeout($.colorbox.resize, 100);
        });
        $($.ns("core.events.ModalWindow")).on("show", function() {
            $.colorbox.resize();
        });
    };
    ModalWindow.prototype = {
        init: function() {
        },
        show: function(e, element, is_new_rule) {
            $(element).attr({ "data-new-rule": is_new_rule ? "1" : "0" });
            $.colorbox({inline: true, href: element, close: false, height: 300, onComplete: function() { $.colorbox.resize(); }});
            $('.close-win-button', element).on("click", function() {$(core.events.ModalWindow).triggerHandler("hide");});
        },
        hide: function() {
            $.colorbox.close();
        }
    };
    $.ns("core.singeltons").ModalWindow = new ModalWindow();
})(jQuery);

// Validation init
(function($) {
    var ValidationInit = function(form, action) {
        var rules = {};
        if(!actions[action]) { console.error("Try validate unknown action", action); return; }
        if(actions[action].tag !== "nagios") {
            rules["name"] = {
                required: true,
                unique_in_scope: true,
                not_allowed_chars: "~!$%^&*|'\"<>?,()=/\\",
                unique_on_server: true };
        }
        $.each(actions[action].params, function() {
            rules[this.name] = this.validation;
        });
        var old_name = $("#new-node-win #materialized").val()+";"+$("#new-node-win #name").val();
        $(form).validate({
            debug: true,
            ignore: ".flexselect, .multiflexselect, .ignore, .is-use",
            rules: rules,
            submitHandler: function() {
                var m;
                if ((m = $("#new-node-win #materialized")).val()) {
                    var hostname = $("#new-node-win #materialized").val();
                    var svc_desc = $("#new-node-win #name").val();
                    var new_name = hostname+";"+svc_desc;
                    core.Data.isServiceExist(m.val()+"&"+$("#new-node-win #name").val(), function(exist) {
                        if (old_name !== new_name && exist) {
                            if (!confirm("A service with the name "+svc_desc+" already exist on the host "+hostname+". Do you want to transform that service into a Business Service?")) {
                                return;
                            }
                        }
                        $($.ns("core.events.NewNodeDialog")).triggerHandler("create", [$(form).serializeArray()]);
                    });
                    return;
                }
                $($.ns("core.events.NewNodeDialog")).triggerHandler("create", [$(form).serializeArray()]);
            },
            errorPlacement: function(error, element) {
                error.appendTo(element.parents(".field-wrapper").find(".error-wrapper"));
            }
        });

    };
    $.ns("core.functions").ValidationInit = ValidationInit;
    jQuery.validator.addMethod("unique_in_scope", function(value, element) {
        var v = value;
        var old = $(element).parents(".new-node-form").find("[name=\"old_name\"]");
        if ($(element).attr("id") === "nagios_service") {
            v = value.split("&")[1] + " on " + value.split("&")[0];
        }
        if ($(element).attr("id") === "nagios_host_services") {
            v = _("Services on ") + value;
        }
        if (old.val() === v) {
            return true;
        }
        return !core.Data.isItemExist((parseInt($("#new-node-win").attr("data-new-rule"), 10) === 1) ? v : (core.url.getCurrent() + "/" + v));
    }, _("Sorry, an element with the same name exist in this scope."));
    jQuery.validator.addMethod("not_allowed_chars", function(value, element, chars) {
        for (var i = 0; i < value.length; i++) {
            if (chars.indexOf(value[i]) !== -1) {
                return false;
            }
        }
        return true;
    }, jQuery.validator.format(_("These symbols are not allowed in names:")+" {0}"));
    jQuery.validator.addMethod("unique_on_server", function(value, element, chars) {
        if (1 !== parseInt($(element).parents("#new-node-win").attr("data-new-rule"), 10)) {
            return true;
        }
        var old = $(element).parents(".new-node-form").find("[name=\"old_name\"]"), v = value;
        if (1 !== parseInt($(element).parents("#new-node-win").attr("data-new-rule"), 10)) {
            return true;
        }
        if ($(element).attr("id") === "nagios_service") {
            v = value.split("&")[1] + " on " + value.split("&")[0];
        }
        if ($(element).attr("id") === "nagios_host_services") {
            v = "Services on " + value;
        }
        if (old.val() === v) {
            return true;
        }
        return !core.Data.isTopLevelExist(value, false);
    }, jQuery.validator.format(_("Sorry, an element with the same name already exists exists.")));
    jQuery.validator.addMethod("gt", function(value, element, params) {
        var compare = $('#'+params[1]);
        var compare_value = compare.val();
        if (parseInt(value, 10) > parseInt(compare_value, 10)) {
            compare.trigger('change');
            return true;
        }
    }, jQuery.validator.format(_("Must be greater than: {2}")));
    jQuery.validator.addMethod("lt", function(value, element, params) {
        var compare = $('#'+params[1]);
        var compare_value = compare.val();
        if (parseInt(value, 10) < parseInt(compare_value, 10)) {
            compare.trigger('change');
            return true;
        }
    }, jQuery.validator.format(_("Must be less than: {2}")));
    jQuery.validator.addMethod("percent", function(value, element) {
        var unit = $('#unit').val();
        if (unit !== "%") {
            return true;
        } else {
            if (value >= 0 && value <= 100) {
                return true;
            }
        }
    }, jQuery.validator.format(_("Must be a percentage between 0 and 100")));
})(jQuery);

// New Node Dialog
(function($) {
    var is_opened = false;
    var event_ns = $.ns("core.events.NewNodeDialog"), prev_page, cur_page;
    var NewNodeDialog = function(node) {
        this.node = $(node);
        this.dialog = $("#dialog-new-node");

        $(event_ns).on("show-dialog", $.recontext(this, this.show_dialog));
        $(event_ns).on("create", $.recontext(this, this.create));
        $(document).on('cbox_closed', $.recontext(this, this.hide_dialog));
        $(core.events.ModalWindow).on("hide", $.recontext(this, this.hide_dialog));
    };
    NewNodeDialog.prototype = {
        create: function(e, data) {
            var new_data = {}, old_name;
            $.each(data, function() {
                 new_data[this.name] = this.value;
            });
            data = new_data;
            delete data["items"];
            data["status"] = "UNKNOWN";
            if($.inArray(data["rule"], ["host", "host_services", "service", "service_group", "host_group"]) !== -1) {
                data["type"] = "nagios";
            } else {
                data["type"] = "rule";
            }
            if(!data["name"]) {
                if(data["nagios_host"]) {
                    data["name"] = data["nagios_host"];
                }
                if(data["nagios_host_services"]) {
                    data["name"] = "Services on " + data["nagios_host_services"];
                }
                if(data["nagios_service"]) {
                    var p = data["nagios_service"].split("&");
                    data["name"] = p[1] + " on " + p[0];
                }
                if(data["nagios_hostgroup"]) {
                    data["name"] = data["nagios_hostgroup"];
                }
                if(data["nagios_servicegroup"]) {
                    data["name"] = data["nagios_servicegroup"];
                }
            }
            if (data["materialized"]) {
                data["materialized"] = {host: data["materialized"]};
            } else {
                delete data["materialized"];
            }
            data.args = [];
            $.each(actions[data.rule].params, function() {
                data.args.push(data[this.name]);
                delete data[this.name];
            });
            var is_rule_add =  parseInt(this.dialog.parents("#new-node-win").attr("data-new-rule"), 10);
            if (is_rule_add) {
                if (data["is_edit"]) {
                    old_name = data["old_name"];
                    delete data["old_name"];
                    delete data["is_edit"];
                    core.Data.edit_rule(data, old_name);
                    $(event_ns).triggerHandler("rule-add");
                    core.url.setAddr("/" + data.name);
                } else {
                    core.Data.add_rule(data);
                    $(event_ns).triggerHandler("rule-add");
                    core.url.setAddr("/" + data.name);
                }
            } else {
                if (data["is_edit"]) {
                    old_name = data["old_name"];
                    delete data["is_edit"];
                    core.Data.edit(data, old_name);
                } else {
                    core.Data.add(data);
                }
                $(event_ns).triggerHandler("node-add");
            }
            $(core.events.ModalWindow).triggerHandler("hide");
            this.hide_dialog();
        },
        show_dialog: function(e, type, obj) {
            var edit_path = $("#new-node-win").attr("data-change");
            if (edit_path) {
                $("#new-node-win").attr("data-change", null);
            }
            is_opened = true;
            $(".win-title").text("Configure element");
            $(event_ns).triggerHandler("show-current");
            prev_page = (p = $(".modal-win-body", this.node)).children().detach();
            if(edit_path) {
                obj = core.Data.get(edit_path);
            }
            var new_dialog = this.dialog.clone();
            this.dialog.after(new_dialog);
            this.dialog.remove();
            this.dialog = new_dialog;
            $(".change-action", this.dialog).on("click", {w:this}, function(e) {
                e.data.w.hide_dialog(e, true);
            });
            this.dialog.find("#type").val(type);
            $(".configure-content", this.dialog).empty();
            $(".configure-content", this.dialog).append($.ns("core.templates.dialog_configs")[type]);
            p.append(this.dialog);
            if(obj) {
                if (obj.materialized) {
                    $("#materialized [value=\""+obj.materialized.host+"\"]", this.dialog).attr("data-populate", "1");
                }
                $("input[type=\"submit\"]", this.dialog).val("save");
                $("input[name=\"name\"]", this.dialog).val(obj.name);
                $(".configure-content")
                    .append($("<input type=\"hidden\" name=\"is_edit\" value=\"1\" />"))
                    .append($("<input type=\"hidden\" name=\"old_name\" value=\""+obj.name+"\" />"));
                var w = this;
                var cur_action = actions[obj.rule];
                if(obj.args) {
                    $.each(obj.args, function(indx, val) {
                        if (!cur_action.params[indx]) {
                            return;
                        }
                        if(cur_action.params[indx]) {
                            $("input[name=\"" + cur_action.params[indx].name + "\"]", w.dialog).val(val);
                        }
                        if (cur_action.params[indx].type == "enum") {
                            $("#"+cur_action.params[indx].name+" [value=\""+val+"\"]", this.dialog).attr("selected", "selected");
                        }
                        if($.inArray(cur_action.params[indx].type, ["nagios_host", "nagios_service", "nagios_hostgroup", "nagios_servicegroup"]) !== -1) {
                            $("#" + cur_action.params[indx].type + " [value=\""+val+"\"]", this.dialog).attr("data-populate", "1");
                        }
                    });
                }
            }
            $("select.flexselect", this.dialog).flexselect();
            $("#materialized", this.dialog).parents(".field-wrapper").materialized();
            // $("input.flexselect", this.dialog).val(null);
            $("select.multi-flexselect", this.dialog).multiflexselect();
            $.insertValues(this.dialog, {
                ".action-name": actions[type].name,
                ".action-descript": actions[type].description
            });
            $(".rule", this.dialog).val(type);
            core.functions.ValidationInit($("form.new-node-form", this.dialog), type);
        },
        hide_dialog: function(e, remember) {
            if(is_opened) {
                var p, edit = $("input[name=\"old_name\"]", this.node);
                cur_page = (p = $(".modal-win-body", this.node)).children().detach();
                $(".modal-win-body", this.node).append(prev_page);
                $.colorbox.resize();
                if (remember) {
                    var data_url = core.url.getCurrent() !== "/" ? core.url.getCurrent() + "/" + edit.val() : edit.val();
                    $(event_ns).triggerHandler("show-prev", [edit.length ? data_url : false]);
                } else {
                    this.node.attr("data-change", null);
                    $(event_ns).triggerHandler("show-prev");
                }
            }
            is_opened = false;
        }
    };
    $.ns("core.classes").NewNodeDialog = NewNodeDialog;
})(jQuery);

// Dialog Config
(function($) {
    var form_entety = "<tr class=\"field-wrapper\"><td class=\"field-description field-part\"><label for=\"{{n}}\">{{l}}: </label><br\><span class = \"action-field-hint\">{{h}}</span></div>";
        form_entety += "<td class=\"field-input field-part\">{{i}}</td>";
        form_entety += "<td class=\"field-messages field-part\"><span class=\"error-wrapper\"></span></td></tr>";
    var DialogConfig = function(action, act_name) {
        res = "";
        if(action.tag !== "nagios") {
            res = $.tmpl(form_entety, {n: "name", l: "Name", h: _("Object name displayed in Business Services"), i: "<input type=\"text\" name=\"name\" id=\"name\" />"});
        } else {
            action.params.push({
                hint: "State type",
                label: "State type",
                name: "state_type",
                type: "enum",
                values: ["hard", "all"]
            });
        }
        var input = "";
        for(var key in action.params) {
            switch(action.params[key]["type"]) {
                case "number":
                case "string":
                input = $.tmpl("<input type=\"text\" id=\"{{n}}\" name=\"{{n}}\"/>", { "n": action.params[key]["name"] });
                break;
                case "enum":
                input = $.tmpl("<select id=\"{{n}}\" name=\"{{n}}\">", { "n": action.params[key]["name"] });
                for(var e_index in action.params[key]["values"]) {
                    input += $.tmpl("<option value=\"{{v}}\">{{v}}</option>", {"v": action.params[key]["values"][e_index]});
                }
                input += "</select>";
                break;
                case "boolean":
                input = $.tmpl("<input type=\"checkbox\" name=\"{{n}}\" id=\"{{n}}\" />", { "n": action.params[key]["name"] });
                break;
                case "nagios_host":
                input = $.tmpl(core.Nagios.get("hosts"), {n: action.params[key]["name"]});
                break;
                case "nagios_hostgroup":
                input = $.tmpl(core.Nagios.get("hostgroups"), {n: action.params[key]["name"]});
                break;
                case "nagios_servicegroup":
                input = $.tmpl(core.Nagios.get("servicegroups"), {n: action.params[key]["name"]});
                break;
                case "nagios_service":
                input = $.tmpl(core.Nagios.get("services"), {n: action.params[key]["name"]});
                break;
            }
            res += $.tmpl(form_entety, {"n": action.params[key]["name"], "l": action.params[key]["label"], "h": action.params[key]["hint"], "i": input});
        }
        if(action.tag === "synergy_group") {
            res += $.tmpl(form_entety, {
                n: "Materialized",
                l: _("as a Service at"),
                h: _("Associate with host for reporting and notification"),
                i:  $.tmpl(core.Nagios.get("hosts"), {n: "materialized"})
            });
        }
        $.ns("core.templates.dialog_configs")[act_name] = res;
    };
    $.ns("core.classes").DialogConfig = DialogConfig;
})(jQuery);

// UI FIX
(function($) {
    var resize = function() {
        $("#current-node-top").css("height", parseInt($(window).height(), 10) - 235);
    };
    $(window).on("resize", $.recontext(window, resize));
    $($.recontext(window, resize));
})(jQuery);

// New Element List
(function($){
    var event_ns = $.ns("core.events.NewElementList"), is_edit = false;
    var action_html = "<div class=\"new-node-element\" data-type=\"{{t}}\"><div class=\"new-action-body\"><div class=\"new-action-icon\"></div><a href=\"#\" class=\"new-action-name\">{{n}}</a><div class=\"clear\"></div></div><div><span class=\"new-action-description\">{{d}}</span></div></div>";
    var action_items = {};
    var detached;
    var not_syn_groups;
    var NewElementList = function(parent) {
        this.parent = $(parent);
        $(event_ns).on("show", $.recontext(this, this.show));
        $(core.events.ModalWindow).on("show", {w: this}, function(e, el, t) {
            if (parseInt(e.data.w.parent.attr("data-new-rule"), 10) === 1) {
                e.data.w.render(true);
            } else {
                e.data.w.render();
            }
            if(t === "NewNodeDialog") {
                $(".win-title").text(_("Select"));
            }
        });
        $(core.events.NewNodeDialog).on("show-prev", {w: this}, function(e, ie) {
            $(".win-title").text("Select");
            e.data.w.parent.attr("data-change", ie ? ie : null);
        });
        $("#display").on("click", "#nodes-new-element", function() {
            $(core.events.ModalWindow).triggerHandler("show", [$("#modal-windows #new-node-win")]);
        });
        this.parent.on("click", ".new-node-element", function(ev) {
            $(core.events.NewNodeDialog).triggerHandler("show-dialog", [$(this).attr("data-type")]);
            ev.preventDefault();
        });
        not_syn_groups = $(".action-tag-group").not(".tag-synergy_group").clone();
        this.render();
    };
    NewElementList.prototype = {
        render: function(top_level) {
            top_level = top_level ? true : false;
            $(".new-node-element", this.parent).remove();
            if (top_level) {
                $(".action-tag-group").not(".tag-synergy_group").remove();
            } else {
                if ($(".action-tag-group").not(".tag-synergy_group").length === 0) {
                    $(".modal-win-body").append(not_syn_groups);
                }
            }
            $.eachSort(actions, "order", function(k, v) {
                $(".tag-" + v["tag"]+">.group-wrapper>.clear").before($($.tmpl(action_html, { t: k, n: v["name"], d: v["description"] })));
            });
        }
    };
    $.ns("core.classes").NewElementList = NewElementList;
})(jQuery);

// Nagios
(function($) {
    var objects = {},
    Nagios = function() {
        $($.recontext(this, this.init));
    };
    Nagios.prototype = {
        init: function() {
            var types = ["hosts", "hostgroups", "servicegroups"];
            for(var i = 0; i < types.length; i++) {
                if(!objects[types[i]]) {
                    var res = "<select name=\"{{n}}\" class=\"flexselect\" id=\"{{n}}\">";
                    $.each(nagios[types[i]], function(k, v) {
                        res += "<option value=\""+k+"\">"+k+"</option>";
                    });
                    res += "</select>";
                    objects[types[i]] = res;
                }
            }
            if (!objects["services"]) {
                objects["services"] = "<select name=\"{{n}}\" class=\"multi-flexselect\" id=\"{{n}}\">";
                $.each(nagios.services, function(k, v) {
                    $.each(v, function() {
                        objects["services"] += "<option value=\""+k+"&"+this+"\">"+k+"&"+this+"</option>";
                    });
                });
                objects["services"] += "</select>";
            }
        },
        get: function(name) {
            if(!objects[name]) {
                return;
            }
            return objects[name];
        }
    };
    core.Nagios = new Nagios();
})(jQuery);

// Ajax Wait
(function($) {
    var AjaxWait = function() {
        $(event_ns).on("show", $.recontext(this, this.show));
        $(event_ns).on("hide", $.recontext(this, this.hide));
        $(window).on("resize", $.recontext(this, center));
    }, event_ns = $.ns("core.events.AjaxWait"),
        node = $("<div id=\"wait-modal-wrapper\"></div>"),
        cont = $("<div id=\"modal-content-wrapper\"><span class=\"loading\">"+_('Saving')+"...</span></div>"),
        hide_div = "<div id=\"hide-all-wrapper\"></div>",
        dt,
        is_showed = false,
        center = function() {
            if(!is_showed) return;
            cont.css({
                left: (node.width() / 2) - (cont.width() / 2),
                top: (node.height() / 2) - (cont.height() / 2)
            });
        };
    AjaxWait.prototype = {
        show: function(e) {
            if(is_showed) return;
            dt = $(document.body).children().detach();
            var hd = $(hide_div).append(dt);
            hd.appendTo(document.body);
            $(document.body).append(node);
            $("#menu").css("overflow", "hidden");
            $(document.body).append(cont);
            is_showed = true;
            $(document.body).trigger("resize");

             // It takes every banner.
             let banners = document.querySelectorAll('.notify-notification-bar .info');
            
             // Iterate over the banners and comparing with the next one making sure that they are unique.
             for(let i = 0; i < banners.length; i++){
                 for (let j = i; j < banners.length; j++) {
                     let compare = banners[j+1];
                     if(compare && banners[i].isEqualNode(compare)){
                         compare.remove();
                     }
                 }
             }
        },
        hide: function() {
            if(!is_showed) return;
            node.detach();
            cont.detach();
            $(document.body).append(dt);
            $("#hide-all-wrapper").remove();
            $("#menu").css("overflow-y", "auto");
            is_showed = false;
        }
    };
    $.ns("core.singeltons").AjaxWait = new AjaxWait();
})(jQuery);

// Action Builder
(function($) {
    var cfl = function(string) {
        return string.charAt(0).toUpperCase() + string.slice(1);
    };
    var action_tmpl = '<a class="action-button {{t}}" onclick="return false;" title="{{h}}"></a>';
    var compile = function(items) {
        var res = "";
        $.each(items, function() {
            var hint = this;
            if (this == "add") {
                hint += " subelement";
            }
            res += $.tmpl(action_tmpl, {t: this, h: cfl(hint)});
        });
        return res;
    };
    var toHtml = function(node) {
        return ($("<div></div>").append(node))[0].innerHTML;
    };
    var compile_link = function(el, indx) {
        var links = actions[el.rule].links || [];
        var link = $.cloneObject(links[indx]);
        $.each(actions[el.rule].params, function(i) {
            if (el.rule !== "service" && this.name !== "nagios_service") {
                link.href = link.href.replace("{{"+this.name+"}}", el.args[i]);
            } else {
                var host_service = el.args[i].split("&");
                link.href = link.href
                    // URI encode host_service to get the ninja and nacoma links to work if special chars has been used
                    .replace("{{nagios_service.name}}", encodeURIComponent(host_service[0]))
                    .replace("{{nagios_service.description}}", encodeURIComponent(host_service[1]));
            }
        });
        return link;
    };
    var t_actions_tmpl = compile(["view", "configure", "add", "edit", "delete", "clone"]);
    var actions_tmpl;
    var noauth_tmpl = compile(["view"]);
    $.ns("core.functions").ActionBuilder = function(el, is_top_level) {
        actions_tmpl = $(core.auth_edit ? t_actions_tmpl : noauth_tmpl);
        var a = 1;
        if ($.isEmpty(el.materialized) && el.type !== "nagios") {
            actions_tmpl.filter(".configure").addClass('disabled');
        }
        if (actions[el.rule].tag == "synergy_group") {
            if (!$.isEmpty(el.materialized)) {
                var e = actions_tmpl.filter(".configure").show().attr("onclick", null);
                e.attr({
                    title: "Service configure",
                    // URI encode names to get the ninja and nacoma links to work if special chars has been used
                    href: core.base_url + "/configuration/configure/service/"+encodeURIComponent(el.materialized.host)+"?service="+encodeURIComponent(el.name)
                });
                var view_e = actions_tmpl.filter(".view").show().attr("onclick", null);
                view_e.attr({
                    title: "Service details",
                    // URI encode names to get the ninja and nacoma links to work if special chars has been used
                    href: core.base_url + "/extinfo/details/service/"+encodeURIComponent(el.materialized.host) + "?service=" + encodeURIComponent(el.name)
                });
                if (!is_top_level) {
                    actions_tmpl.filter(".clone").addClass('disabled');
                }
                return toHtml(actions_tmpl);
            }
            if(is_top_level) {
                actions_tmpl.filter(".view").addClass('disabled');
                return toHtml(actions_tmpl);
            }
            actions_tmpl.filter(".view, .clone").addClass('disabled');
            return toHtml(actions_tmpl);
        }
        var links = actions[el.rule].links || [], res = actions_tmpl.filter(".add, .clone, .view").addClass('disabled');
        if (links.length === 0) return toHtml(actions_tmpl);
        var view_link = compile_link(el, 0), conf_link = compile_link(el, 1);
        var ve = actions_tmpl.filter(".view").show().attr("onclick", null);
        var ce = actions_tmpl.filter(".configure").show().attr("onclick", null);
        ve.attr({
            "title": cfl(view_link.title),
            "href": core.base_url+view_link.href
        });
        ce.attr({
            "title": cfl(conf_link.title),
            "href": core.base_url+conf_link.href
        });
        return toHtml(actions_tmpl);
    };
})(jQuery);

// Tree view
(function($) {
    var tmpl = "<table class=\"tree-wrapper\"><thead><tr><th><div>"+_('Business Object')+"</div><div><a class=\"expand-all\" onclick=\"return false;\" title=\""+_('Expand all')+"\"></a></div><div><a class=\"collapse-all\" onclick=\"return false;\" title=\""+_('Collapse all')+"\"></a></div><div class=\"clear\"></div></th><th>"+_('Rule')+"</th><th>"+_('Actions')+"</th><th>"+_('Last check')+"</th><th>"+_('Duration')+"</th><th>"+_('Status information')+"</th></tr></thead><tbody><tr class=\"start-point\"></tr></tbody></table>", elements, c;
    elem_temp = '<tr data-url="{{u}}" data-level="{{l}}"><td class="obj-name {{s}}"><span class="margin"></span><span class="arrow"></span><a class="shild-img" title="{{h}}"></a><span class="name">{{n}}</span></td><td>{{r}} {{a}}</td><td class="obj-actions">{{d}}</td><td class="obj-last-check">{{lc}}</td><td class="obj-last-state-change">{{lstate}}</td><td>{{m}}</td></tr>',
        statuses_hists = {
            "host": {
                0: "UP",
                1: "DOWN",
                2: "UNREACHABLE",
                6: "PENDING"
            },
            "service": {
                0: "OK",
                1: "WARNING",
                2: "CRITICAL",
                3: "UNKNOWN",
                6: "PENDING"
            }
        };
        // Widget context
        _getOpenedNodes = function() {
            var res = [];
            this.node.find(".tree-wrapper tbody tr.opened").each(function() { res.push($(this).attr("data-url")); });
            return res;
        },
        _openNode = function(elem) {
            var url = decodeURIComponent(elem.attr("data-url")),
                level = parseInt(elem.attr("data-level"), 10), elements,
                items = _dataSeparate(core.Data.get(url, true));
            if(elem.hasClass("opened")) { return; }
            for(var i = 0; i < items.length; i ++) {
                $.eachSort(items[i], "name", function(k, v) {
                    elem.after(factory_element(v, url + "/" + v.name, level + 1));
                });
            }
            elem.addClass("opened");
        },
        _dataSeparate = function(data) {
            var result = [{}, {}, {}, {}, {}];
            $.each(data || [], function(k, v) {
                if(v.type != "nagios") {
                    return result[0][k] = v;
                }
                if(v.rule == "host_group_up" || v.rule == "host_group_unfold") {
                    return result[1][k] = v;
                }
                if(v.rule == "service_group_unfold") {
                    return result[2][k] = v;
                }
                if(v.rule == "vhost_up" || v.rule == "host") {
                    return result[3][k] = v;
                }
                return result[4][k] = v;
            });
            return result;
        },
        factory_element = function(elem, path, l) {
            var allargs, hint = "bla";
            elem.result = elem.result || {};
            if (typeof(elem.result.status) === "undefined") {
                elem.result.status = 6;
                elem.status = "PENDING";
            }
            if ((elem.type === "nagios") && (/^host/.test(elem.rule))) {
                hint = statuses_hists["host"][elem.result.status];
            } else {
                hint = statuses_hists["service"][elem.result.status];
            }
            msg = elem.result.msg || "pending";
            last_check = $.timeFormat(parseInt(elem.result.last_check || 0, 10) * 1000);
            last_state_change = $.timeDiffFormat(parseInt(elem.result.last_state_change, 10) * 1000, core.Data.getServerTimestamp() * 1000);
            allargs = [];
            $.each(elem.args, function(k,v) {
                if(elem.rule == "service") {
                    allargs = allargs.concat(v.split("&"));
                } else {
                    allargs.push(v);
                }
            });
            allargs = allargs.length ? ("(" + allargs.join(", ") + ")") : "";
            var new_v = $($.tmpl(elem_temp, {
                n: elem.name,
                m: msg,
                s: elem.status,
                u: encodeURIComponent(path),
                r: actions[elem.rule].name,
                a: allargs,
                d: core.functions.ActionBuilder(elem, l === 0),
                l: l,
                lc: last_check,
                h: hint,
                lstate: last_state_change
            }));
            if(elem.materialized && !elem.materialized_and_exists_in_livestatus) {
                new_v
                    .find('span.name')
                        .html(elem.name + " (service <strong>"+decodeURIComponent(elem.name)+"</strong> "+
                            "does not exist on host <strong>"+decodeURIComponent(elem.materialized.host)+"</strong>)");
                new_v
                    .find('td')
						.addClass('warning')
                        .addClass('state-background');
                new_v
                    .attr('title', "The service "+
                            decodeURIComponent(elem.name)+" on the host "+
                            decodeURIComponent(elem.materialized.host) +
                            " cannot be found, please recreate this service, or"+
                            " edit this BSM object to point at another service."+
                            " If you just added the service, this warning" +
                            " will soon disappear.");
            }
            if(core.Data.hasItems(path)) {
                new_v.addClass("has-subitems");
            }
            $(".margin", new_v).width(l * 16);
            return new_v;
        };
    TreeView = function () {
        $($.recontext(this, this.init));
        $($.ns("core.events.DataWrapper")).on("data-update", $.recontext(this, this.update_tree));
        $($.ns("core.events.NewNodeDialog")).on("rule-add", {w: this}, function(e) { e.data.w.render(core.url.getAddr()); });
        $($.ns("core.events.NewNodeDialog")).on("node-add", {w: this}, function(e) { e.data.w.render(core.url.getAddr()); });
        $($.ns("core.events.ContentLoader")).on("load-page", {w: this}, function(e, p, m, is_internal) {
            if(m == "tree") {
                if(!is_internal) {
                    e.data.w.render(p);
                }
            } else {
                if(elements) {
                    elements.remove();
                    elements = false;
                    $("#display").css("padding-left", "340px");
                }
            }
        });
    };
    TreeView.prototype = {
        init: function() {
            this.node = $("#display");
            $("#view-port").on("submit", "#new-object-wrapper", {w: this}, function(e) {
                $(core.events.ModalWindow).triggerHandler("show", [$("#modal-windows #new-node-win"), true]);
            });
            this.node.on("click", ".expand-all", {w: this}, function(e) {
                e.data.w.show_all();
                $(".start-point", e.data.w.node).remove();
            });
            this.node.on("click", ".collapse-all", {w: this}, function(e) {
                core.url.setAddr("/", true);
            });
            this.node.on("click", ".has-subitems .arrow", function() {
                $(this).siblings(".name").trigger("click");
            });
            this.node.on("click", ".has-subitems .obj-name .name", {w: this}, function(e) {
                var pnode = $(this).parents("tr"),
                    url = decodeURIComponent(pnode.attr("data-url")),
                    level = parseInt(pnode.attr("data-level"), 10), elements;
                if(!pnode.hasClass("opened")) {
                    var items = _dataSeparate(core.Data.get(url, true));
                    for(var i = 0; i < items.length; i ++) {
                        $.eachSort(items[i], "name", function(k, v) {
                            pnode.after(factory_element(v, url + "/" + v.name, level + 1));
                        });
                    }
                    pnode.addClass("opened");
                    core.url.setAddr(url);
                } else {
                    pnode.removeClass("opened");
                    $("[data-url^=\""+encodeURIComponent(url + "/")+"\"]", e.data.w.node).remove();
                    var parts = url.split("/");
                    parts.pop();
                    var u = parts.join("/");
                    core.url.setAddr(u);
                }
                e.data.w.recolor();
            });
            this.node.on("click", ".action-button.edit", function() {
                var url = decodeURIComponent($(this).parents("tr").attr("data-url")), new_url;
                if (url[0] === "/") { url = url.slice(1); }
                var el = core.Data.get(url);
                (new_url = url.split("/")).pop(), new_url = new_url.join("/");
                core.url.setAddr(new_url);
                $(core.events.ModalWindow).triggerHandler("show", [$("#modal-windows #new-node-win"), url.split("/").length == 1]);
                $(core.events.NewNodeDialog).triggerHandler("show-dialog", [el.rule, el]);
            });

            this.node.on("click", ".action-button.add", function() {
                var url = decodeURIComponent($(this).parents("tr").attr("data-url"));
                if (url[0] === "/") { url = url.slice(1); }
                core.url.setAddr(url);
                $(core.events.ModalWindow).triggerHandler("show", [$("#modal-windows #new-node-win")]);
            });

            this.node.on("click", ".action-button.delete", {w: this}, function(e) {
                var url = decodeURIComponent($(this).parents("tr").attr("data-url"));
                var parts = url.split("/");
                if(confirm("Are you sure you want to delete '"+decodeURIComponent(parts[parts.length-1])+"'?") === false) {
                    return;
                }
                if (parts.length === 1) {
                    core.Data.del_rule(url);
                    core.url.setAddr("/");
                } else {
                    core.Data.del(url);
                    core.url.setAddr(core.Data.getParentUrl(url));
                }
                e.data.w.render();
            });

            this.node.on("click", ".action-button.clone", {w: this}, function(e) {
                var widget = e.data.w, url = decodeURIComponent($(this).parents("tr").attr("data-url"));
                core.Data.clone_rule(url.split("/")[0], function(new_obj) {
                    core.url.setAddr(new_obj.name);
                    $(core.events.ModalWindow).triggerHandler("show", [$("#modal-windows #new-node-win"), true]);
                    $(core.events.NewNodeDialog).triggerHandler("show-dialog", [new_obj.rule, new_obj]);
                });
                widget.render();
            });
        },
        recolor: function() {
            var i = 0;
            $("tbody tr", this.node).removeClass("odd");
            $.each($("tbody tr", this.node), function(k, v) {
                $(v).addClass((k%2) ? "odd" : "even");
            });
        },
        update_tree: function(e, l, prefix) {
            l = l || 0;
            var opened = _getOpenedNodes.call(this), that = this;
            this.render(core.url.getCurrent(), true);
            $.each(opened, function(k, v) { _openNode.call(that, $("[data-url=\""+v+"\"]", that.node)); });
            this.recolor();
        },
        render_path: function(node, path, pre_path, l) {
            l = l || 0;
            pre_path = pre_path || "";
            var level_elements = core.Data.get(pre_path, true);
            var sd = _dataSeparate(level_elements);
            var allargs = [], last_check, last_state_change, msg, cur_path;
            for (var i = 0; i < sd.length; i++) {
                $.eachSort(sd[i], "name", function(k, v) {
                    node.after(factory_element(v, pre_path + (pre_path.length ? "/" : "") + v.name, l));
                });
            }
            path.replace(/^\/+/, "");
            if (!path) {
                return;
            }
            var p_parts = path.split("/");
            if (pre_path.length) {
                var rpa = pre_path.split("/");
                rpa.push(p_parts[0]);
                pre_path = rpa.join("/");
            }
            pre_path = pre_path.length ? pre_path : p_parts[0];
            p_parts.splice(0, 1);
            path = p_parts.join("/");
            var a = node.siblings("[data-url=\""+encodeURIComponent(pre_path)+"\"]").addClass("opened");
            this.render_path(a, path, pre_path, l + 1);
        },
        render: function(p, no_recolor) {
            p = p || core.url.getAddr();
            $("#display").css("padding-left", 0);
            elements = $(tmpl);
            this.node.empty();
            this.node.append(elements);
            this.render_path(elements.find(".start-point"), p);
            $(".start-point", this.node).remove();
            if (!no_recolor) {
                this.recolor();
            }
            this.node.show();
        },
        show_all: function(node, p, l) {
            l = l || 0;
            p = p || "";
            if (!node) {
                this.node.find("tbody").empty().append("<tr class=\"start-point\"></tr>");
                node = this.node.find(".start-point");
            }
            var data = core.Data.get(p, true), cur_node, cur_path, that = this;
            var sd = _dataSeparate(data);
            for(var i = 0; i < sd.length; i++) {
                $.eachSort(sd[i], "name", function(k, v) {
                    node.after(cur_node = factory_element(v, cur_path = (p + (p.length ? "/" : "") + v.name), l));
                    if (core.Data.hasItems(cur_path)) {
                        cur_node.addClass("opened");
                        that.show_all(cur_node, cur_path, l + 1);
                    }
                });
            }
            this.recolor();
        }
    };
    $.ns("core.singeltons").TreeView = new TreeView();
})(jQuery);

// Url builder
(function($) {
    var Url = function() {
        $(function() {
            $("#view-mode-switcher").on("click", function() {
                core.url.setMode($(this).text(), true);
            });
        });
        $($.ns("core.events.ContentLoader")).on("load-page", {w: this}, function(e) {
            e.data.w.cur_url = e.data.w.getAddr();
            $("#view-mode-switcher").text(core.url.getMode() == "tree" ? "flat" : "tree");
        });
    };
    Url.prototype = {
        setAddr: function(val, external) {
            if(val[0] == "/") {
                val = val.slice(1);
            }
            var parts = $.address.value().split(":"), url;
            parts[1] = encodeURIComponent(val);
            $.address.value(url = parts.join(":"));
            if(external) {
                core.changeFn.apply({}, [{path: url}, false]);
            }
        },
        getCurrent: function() {
            return this.cur_url;
        },
        getAddr: function() {
            return decodeURIComponent($.address.value().split(":")[1] || "/");
        },
        setMode: function(val, external) {
            var parts = $.address.value().split(":"), url;
            parts[0] = "/"+ val;
            $.address.value(url = parts.join(":"));
            if(external) {
                core.changeFn.apply({}, [{path: url}, false]);
            }
        },
        getMode: function() {
            return ($.address.value().split(":")[0] || "/tree").slice(1);
        }
    };
    core.url = new Url();
})(jQuery);

// Init functions
(function($) {
    $(function() {
        core.base_url = $("#view-port").attr("data-base-url");
        core.base_url = core.base_url.substring(0, core.base_url.length - 1);
        core.mode = $("#view-port").attr("data-mode");
        core.auth_edit = parseInt($("#view-port").attr("data-auth-edit"), 10) === 1;
        if (core.auth_edit === false) {
            $("#new-object-wrapper").remove();
            $("#facet>a").each(function() { $(this).css("left", $(this).offset().left - 110); });
        }
        var cls = core.classes.Scroll;
        s = new cls("#slider");
        $("#slider-go-left").on("click", function() {
            $(core.events.Scroll).triggerHandler("move-left");
        });
        $("#slider-go-right").on("click", function() {
            $(core.events.Scroll).triggerHandler("move-right");
        });
        cls = core.classes.BreadCrumbs;
        bc = new cls("#breadcrumbs");
        cls = core.classes.ContentLoader;
        cl = new cls("#nodes-wrapper");
        cls = core.classes.CurrentNode;
        cn = new cls("#current-node-top");
        if(core.mode == 2) {
            $("#nodes-new-element").hover(function(){ $(this).animate({opacity: 1}, 150); }, function() { $(this).animate({opacity: 0.5}, 150); });
        }
        cls = core.classes.NewElementList;
        (new cls("#new-node-win"));
        cls = core.classes.DialogConfig;
        for(var name in actions) {
            if(!actions.hasOwnProperty(name)) continue;
            new cls(actions[name], name);
        }
        cls = core.classes.NewNodeDialog;
        new cls("#new-node-win");
        cls = core.classes.NodeFacet;
        new cls("#facet-form");
        cls = core.classes.LiveSearch;
        ls = new cls("#live-search");

        // Hints with tipsy (tooltips for jquery)
        $("#mode-swicher").tipsy();
        $("#quick-filter").tipsy({ title: 'hint', trigger: 'focus', gravity: 'e', offset: 5 });
        $("#icons>ul>li").filter(function() { return $(this).attr("onclick") === "window.location.reload()"; }).attr("onclick", null).on("click", function() {
            core.Data.update();
        });
    });

    core.changeFn = function(e, type) {
        var addr = decodeURIComponent(e.path.substring(1)), def, parts = addr.split(":");
        if(parts.length === 1) {
            parts.unshift("tree");
            type = false;
            $.address.value("/tree:");
            return core.changeFn({path: "/tree:"}, false);
        }
        if(!parts[1].length && parts[0] != "tree") {
            for(var indx in core.Data.get(false, true)) {
                def = indx;
                break;
            }
            parts[1] = def;
            return $.address.value(parts.join(":"));
        }
        if(parts[0] == "tree") {
            $("#slider-breadcrumbs").hide();
            $("#display").addClass("with_margin");
            $("#new-object-wrapper").show();
        } else {
            $("#display").show();
            $("#navigator").show();
            $("#slider-breadcrumbs").show();
            $("#display").removeClass("with_margin");
            $("#new-object-wrapper").hide();
        }
        $(core.events.ContentLoader).triggerHandler("load-page", [parts[1], parts[0], type]);
    };

    $.address.internalChange(function(e) { core.changeFn.apply(this, [e, true]); });
    $.address.externalChange(function(e) { core.changeFn.apply(this, [e, false]); });
})(jQuery);

})(); /* Maps to topmost test if inside synergy */


